// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * SelectionManager.java
 *
 * Created on 9 mai 2006, 19:29
 *
 * @author Arnaud Daret, CERTIC, Université de Caen, daret@info.unicaen.fr
 */

package fr.unicaen.openmap.dataStorage;

import com.bbn.openmap.util.ColorFactory;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.dataStorage.gui.DataAndSelectionModel;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.util.Util;
import java.awt.Color;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Vector;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;


/**
 *
 * @author Arnaud Daret, CERTIC, Université de Caen, daret@info.unicaen.fr
 * @author Christophe Turbout, CERTIC, Université de Caen, christophe.turbout@info.unicaen.fr
 */
public class SelectionManager {
    
    private ArrayList<SelectionMatrix> selections = null;
    private ArrayList<String> selectionNames = null; // store the name of the selection.
    private ArrayList<ArrayList<Color>> selectionColors = null; // store the color of each subselection of each selection
    private int currentSelection = -1;
    private ArrayList<Color> defaultColors = null;
    private String defaultSelectionName = null;
    private String defaultSubSelectionName = null;
    
    // we use this to manage deletion.
    private ArrayList<Boolean> composedSelection = null;
    private int nbZoneCreated=0;
    
    /** Creates a new instance of SelectionManager */
    public SelectionManager(DataTableModel model, String defaultSelectionName, String defaultSubSelectionName,ArrayList<Color> defaultColors, Boolean multipleSelectionModel) {
        // all the selections
        selections = new ArrayList<SelectionMatrix>();
        // all the colors
        selectionColors = new ArrayList<ArrayList<Color>>();
        // all the selections names
        selectionNames = new ArrayList<String>();
        this.defaultColors=defaultColors;
        this.defaultSelectionName=defaultSelectionName;
        this.defaultSubSelectionName=defaultSubSelectionName;
        composedSelection = new ArrayList<Boolean>();
        createEmptySelection( model,multipleSelectionModel);
    }
    
    public void createEmptySelection(DataTableModel model,  Boolean multipleSelectionModel){
        nbZoneCreated++;
        currentSelection=selections.size();
        // an empty selection
        SelectionMatrix matrix = new SelectionMatrix(model.getRowCount(),defaultSubSelectionName,true,multipleSelectionModel,nbZoneCreated);
        selections.add(matrix);
        // the name of the selectionMatrix
        selectionNames.add(defaultSelectionName + " "+(getNbZoneCreated()));
        // the default color for this selection.
        ArrayList<Color> subselectionColors = new ArrayList<Color>();
        Color selColor = defaultColors.get(0);
        subselectionColors.add(selColor);
        selectionColors.add(subselectionColors);
    }
    
    /**
     * Add a new Empty Matrix.
     **/
    public int createNewEmptySelection(String name,DataTableModel model, Boolean multipleSelectionModel){
        nbZoneCreated++;
        int nbsel = selections.size();
        SelectionMatrix matrix = new SelectionMatrix(model.getRowCount(),defaultSubSelectionName,true,multipleSelectionModel,nbZoneCreated);
        selections.add(matrix);
        // the name of the matrix
        selectionNames.add(name);
        // the default colors for this selection.
        ArrayList<Color> subselectionColors = new ArrayList<Color>();
        Color selColor = defaultColors.get(0);
        subselectionColors.add(selColor);
        selectionColors.add(subselectionColors);
        // set the new selection has default selection
        setCurrentSelection(nbsel);
        return nbsel;
    }
    /**
     * Set the active selection in SelectionDataStorage and notifies the change to all components
     *
     * @see SelectionMatrix
     * @see DataAndSelectionModel
     */
    public void setCurrentSelection(int selection){
        Debug.message(Util.className(this) , this + " Sélection "+selection+" active.");
        if (selection<selections.size() && selection>=0) {
            currentSelection=selection;
        } else new Exception("Erreur, index incorrect pour SetCurrentSelection()");
    }
    
    public SelectionMatrix getSelectionMatrix(int index){
        return selections.get(index);
    }
    
    public SelectionMatrix getCurrentSelectionMatrix(){
        return getSelectionMatrix(getCurrentSelection());
    }
    
    public String getSelectionName(int index){
        return selectionNames.get(index);
    }
    
    public String getCurrentName(){
        return getSelectionName(getCurrentSelection());
    }
    
    public int getCurrentSelection(){
        return currentSelection;
    }
    
    public void loadSelections(String datas, DataTableModel model, Boolean multipleSelectionModel){
        // reset all variables to load what is stored.
        
        // all the selections
        selections = new ArrayList<SelectionMatrix>();
        // all the colors
        selectionColors = new ArrayList<ArrayList<Color>>();
        // all the selections names
        selectionNames = new ArrayList<String>();
        // create an empty SelctionMatrix.
        try {
            // parse an XML document into a DOM tree
            DocumentBuilder parser = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            // read a document
            StringReader reader = new StringReader(datas);
            InputSource source = new InputSource(reader);
            Document document = parser.parse(source);
            
            // find the document root
            Element root = document.getDocumentElement();
            // first we have to find the selection pattern.
            NodeList selectionNode = root.getElementsByTagName("selection");
            if (selectionNode.getLength()==0) {
                Debug.message(Util.className(this) , this + "Error : no element in list");
                return;
            }
            setCurrentSelection(0);
            
            // add all SelectionMatrix to Storage
            for (int i=0;i<selectionNode.getLength();i++){
                Element element = (Element) selectionNode.item(i);
                SelectionMatrix matrix = new SelectionMatrix(model.getRowCount(),defaultSubSelectionName,element,multipleSelectionModel,-1);
                // add the matrix
                selections.add(matrix);
                Debug.message(Util.className(this) , this + " curent matrix's size : "+matrix.getNbRows() +" rows - "+matrix.getNbCols() +" cols");
                Debug.message(Util.className(this) , this + " number of matrix : "+selections.size());
                // we have to store the name of the selection
                NodeList nameList = element.getElementsByTagName("name");
                String selectionName = nameList.item(0).getFirstChild().getNodeValue();
                selectionNames.add(selectionName);
                Debug.message(Util.className(this) , this + " name of te selection : "+selectionName);
                
                // For each selection, we have to add Colors
                NodeList colorNodes = (NodeList) element.getElementsByTagName("color");
                ArrayList<Color> subselectionColors = new ArrayList<Color>();
                for (int j=0;j<colorNodes.getLength();j++){
                    String color = colorNodes.item(j).getFirstChild().getNodeValue();
                    subselectionColors.add(createColor(color));
                }
                selectionColors.add(subselectionColors);
            }
            NodeList nbcreated = root.getElementsByTagName("nbZoneCreated");
            if (nbcreated!=null && nbcreated.getLength()>0) {
                Debug.message(Util.className(this) , this + " number of selection created: "+nbcreated);
                nbZoneCreated=new Integer(nbcreated.item(0).getFirstChild().getNodeValue());
            }
            else {
                //if nbZoneCreated does not existe, this is an old propertie file so we try to define nbZoneCreated with 
                //created zone stored in the propertie file.
                nbZoneCreated=selections.size();
            }
            
        } catch(Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
    }
   /*
    * Need to use a thread to read information. This is not finished at all !
    **/
    public String saveSelections(){
        Debug.message(Util.className(this) , this + "Begin of selection Matrix save method");
        StringBuffer buffer = new StringBuffer();
        buffer.append("<nbZoneCreated>");
        buffer.append(getNbZoneCreated());
        buffer.append("</nbZoneCreated>");
        for (int i=0;i<selections.size();i++){
            SelectionMatrix matrix = getSelectionMatrix(i);
            buffer.append(matrix.saveToXML(getSelectionName(i),getSelectionColors(i)));
        }
        Debug.message(Util.className(this) , this + buffer.toString());
        return buffer.toString();
    }
    
    /*
     * return the size of the current SelectionManager (the number of selections).
     */
    public int size(){
        return selections.size();
    }
    
        /*
         * Build a Color with String object.
         */
    private Color createColor(String colorString){
        Color col = null;
        try {
            col = ColorFactory.parseColor(colorString);
            return col;
        } catch (Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
        return null;
    }
    
    /**
     * Retrun the name of the subselection in the selection.
     **/
    public String getSubSelectionName(int selection, int subselection){
        String name = null;
        try {
            SelectionMatrix matrix = selections.get(selection);
            name = matrix.getColumnName(subselection);
        } catch (Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
        return name;
    }
    /**
     * set the name of the subselection in the selection.
     **/
    public void setSubSelectionName(int selection, int subselection,String name){
        try {
            SelectionMatrix matrix = selections.get(selection);
            matrix.setSubSelectionName(subselection,name);
        } catch (Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
    }
    
    /**
     * Retrun the names of the selection.
     **/
    public ArrayList<String> getSubSelectionNames(int selection){
        ArrayList<String> name = null;
        try {
            SelectionMatrix matrix = selections.get(selection);
            name = matrix.getColumnNames();
        } catch (Exception e){
            System.out.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
        return name;
    }
    
    /**
     * return the Color vector used by the selection, null otherwise
     **/
    public ArrayList<Color> getSelectionColors(int selection){
        if (selectionColors.size() == 0) return null;
        try {
            ArrayList<Color> colors = selectionColors.get(selection);
            return colors;
        } catch (Exception e){
            System.out.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
            return null;
        }
    }
    
    /**
     * get the color of the selected omgraphics if it is selected.
     * Do nothing otherwise.
     **/
    public Color getOMGraphicColor(int index){
        SelectionMatrix currentMatrix = selections.get(getCurrentSelection());
        if (currentMatrix.isSelected(index)) { return selectionColors.get(getCurrentSelection()).get(currentMatrix.getSelectedColumn(index)); }
        return null;
    }
    
    /*
     * Retru nthe current subselection in currentselection
     */
    public int getCurrentSubSelection(){
        return selections.get(getCurrentSelection()).getCurrentSubSelection();
    }
    
    /**
     * Return the current color in the current selection.
     *
     * @return Color
     **/
    public Color getCurrentColorSelection() {
        try {
            SelectionMatrix matrix = selections.get(getCurrentSelection());
            return getSubSelectionColor(getCurrentSelection(),matrix.getCurrentSubSelection());
        } catch(Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
            return null;
        }
    }
    
    /**
     * Return the color of the subselection in the selection
     **/
    public Color getSubSelectionColor(int selection, int subselection){
        Color color = null;
        try {
            color = getSelectionColors(selection).get(subselection);
        } catch(Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
        return color;
    }
    
    /**
     * Set the color of the subselection in the currentSelection.
     **/
    public void setSubSelectionColor(int subselection, Color color){
        try {
            selectionColors.get(getCurrentSelection()).set(subselection,color);
        } catch (Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
    }
    
    /*
     * chaneg the name of the current selection.
     */
    public void setCurrentSelectionName(String newName){
        try {
            selectionNames.set(getCurrentSelection(),newName);
        } catch (Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
    }
    /**
     * Set the active subselection in SelectionDataStorage. We store this information and change the
     * data in selectionMatrixModel and notifies the change to DataAndSelectionModel listeners.
     *
     * @see SelectionMatrix
     * @see SelectionDataMatrixModel
     * @see DataAndSelectionModel
     */
    public void setCurrentSubSelection(int subselection){
        SelectionMatrix matrix = selections.get(getCurrentSelection());
        matrix.setCurrentSubSelection(subselection);
    }
    
    /**
     * Add a new Selection Matrix which is the copy of an exiting selectionMatrix.
     **/
    public int createSelectionCopy(int selection, String name){
        nbZoneCreated++;
        int nbsel = selections.size();
        SelectionMatrix matrix = selections.get(selection);
        SelectionMatrix copymatrix = matrix.getCopy(nbZoneCreated);
        selections.add(copymatrix);
        // the name of the copyMatrix
        selectionNames.add(name);
        // the default colors for this selection.
        ArrayList<Color> subselectionColors = new ArrayList<Color>();
        for (int i=0;i<copymatrix.getNbCols();i++){
            Color selColor = defaultColors.get(i);
            subselectionColors.add(selColor);
        }
        selectionColors.add(subselectionColors);
        // set the new selection has default selection
        setCurrentSelection(nbsel);
        return nbsel;
    }
    
    /**
     * remove current subselection.
     */
    public int deleteCurrentSubSelection(){
        // get the current matrix
        SelectionMatrix currentMatrix = selections.get(getCurrentSelection());
        // get old currentSubSelection.
        int oldSelection = currentMatrix.getCurrentSubSelection();
        // delete in Selection Matrix.
        currentMatrix.deleteCurrentSelection();
        // delete Color
        selectionColors.get(getCurrentSelection()).remove(oldSelection);
        return oldSelection;
    }
    
    /**
     * Create a new subselection.
     */
    public int createSubSelection(String name){
        //Debug.message(Util.className(this),this+"Création de la sous-sélection : "+currentSelection);
        // get the current matrix
        SelectionMatrix currentMatrix = getCurrentSelectionMatrix();
        // add the column and get the name
        // String subname = null;
        // if (name==null) subname = currentMatrix.addColumn(false,defaultSubSelectionName,true);
        // else subname = currentMatrix.addColumn(false,name,true);
        if (name==null) currentMatrix.addColumn(false,defaultSubSelectionName,true);
        else currentMatrix.addColumn(false,name,false);
        // get the new color
        Color selColor = defaultColors.get((currentMatrix.getNbColumnCreated()-1) % 8);
        // add it to the list
        selectionColors.get(getCurrentSelection()).add(selColor);
        return  currentMatrix.getCurrentSubSelection();
    }
    
    /**
     * Return the number of zone created. Whit this mecanism we are sure that automatic name is unique.
     **/
    public int getNbZoneCreated(){
        return nbZoneCreated;
    }
    
    /**
     * Return true if the rowindex row is selected in the current selection.
     **/
    public boolean isSelected(int rowindex) {
        return composedSelection.get(rowindex);
    }
   /*
    * delete the current selection. If selections is empty, nothing to do.
    */
    public void deleteCurrentSelection(){
        if (selections.size()>0) {
            selections.remove(getCurrentSelection());
            selectionNames.remove(getCurrentSelection());
            selectionColors.remove(getCurrentSelection());
            // we set the new selection.
            setCurrentSelection(selections.size()-1);
        }
    }
    
}
