// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.expr;

import com.bbn.openmap.util.Debug;
import com.bbn.openmap.LayerHandler;
import com.bbn.openmap.MapBean;
import com.bbn.openmap.util.PropUtils;


import fr.unicaen.openmap.app.ThemaMap;
import fr.unicaen.openmap.dataStorage.DataStorage;
import fr.unicaen.openmap.dataStorage.DataStorageListener;
import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.OMDataLocation;
import fr.unicaen.openmap.dataStorage.gui.MultiDataTableModel;
import fr.unicaen.openmap.dataStorage.gui.DataTableStoragePanel;

import fr.unicaen.openmap.dataStorage.expr.gui.AddColumnPanel;
import fr.unicaen.openmap.exception.UnavailableColumnNameException;
import fr.unicaen.openmap.util.Util;
import java.util.ArrayList;
import java.util.Properties;
import javax.swing.Box;
import javax.swing.JComponent;

public class AddColumnDataStorage extends ExprDataStorage {

    public static final String dataStorageClassPropertyString = "dataStorageClass";
    public static final String addColumnDSPropertyString = "addColumnDS";
    public static final String addedColumnNamePropertyString = "addedColumnName";

    public AddColumnDataStorage(DataStorageListener dsl) {
	this(dsl, null);
    }

    public AddColumnDataStorage(DataStorageListener dsl, DataStorage dataStorage) {     
	super(dsl, dataStorage);
	addedColumnName = i18n.get(this, "newName", "new");
    }

    protected boolean addedColumnIsData;
    /** The added column. 
     * contains String or Number if addedColumnIsData is true
     *                OMDataLocation otherwise
     */
    protected ArrayList<Object> columnToAdd;

    /**
    Precompute the values of each row of the added column
    @return the added column name
     */
    protected String computeColumn() {
        if (treeExpr == null) {
            return null;
        }
        DataStorage ds = getDataStorage();
        int count = ds.getCount();
        columnToAdd = new ArrayList<Object>(count);
        for (int i = 0; i < count; i++) {
            DataObject data = ds.getElementAt(i);
            columnToAdd.add(Function.eval(treeExpr, data));
        }

        addedColumnIsData = !Function.isGeometryType(treeExpr);
        return addedColumnName;

    }

    public void initialize() {
	if (! super.isReady() || ! setColumnNames()) {
	    Debug.message("DataStorage", classNameForDebug + " data not ready *");
	    return;
	}
	Debug.message("DataStorage", classNameForDebug + " data ready ***");

	// We can now (re)build the DataTableModel
	dataTableModel = newMultiDataTableModel();
	if (dtsp == null)
	    dtsp = new DataTableStoragePanel(dataTableModel);
	else
	    ((DataTableStoragePanel)dtsp).setModel(dataTableModel);
	// tell the listener we're ready
	dsl.dataStorageEvent(this);
    }


    public boolean isReady() {
	return super.isReady() && getDataStorage().getDefaultDataTableModel() != null;
    }
    
    String addedColumnName = null;

    protected boolean setColumnNames() {
        if (getDataStorage() == null || getDataStorage().getDefaultDataTableModel() == null)
            return false;

        setAddedColumnName(computeColumn());

        String[] names;
        names = getDataStorage().getDefaultDataTableModel().getDataColumnNames();
        dataColumnNames = new ArrayList<String>(names.length);
        for (String n : names) {
            dataColumnNames.add(n);
        }
        names = getDataStorage().getDefaultDataTableModel().getOMColumnNames();
        omColumnNames = new ArrayList<String>(names.length);
        for (String n : names) {
            omColumnNames.add(n);
        }

        if (addedColumnName != null) {
            // add the new column names
            if (addedColumnIsData)
                dataColumnNames.add(addedColumnName);
            else
                omColumnNames.add(addedColumnName);
        }
        return true;
    }

    public DataObject getElementAt(int index) {
        DataObject data = getDataStorage().getElementAt(index);
        if (columnToAdd == null) {
            computeColumn();
        }
        if (addedColumnName != null) {
            Object o = columnToAdd.get(index);
            if (o instanceof OMDataLocation) {
                data.addOMDataLocation(addedColumnName, (OMDataLocation) o);
            } else {
                data.addData(addedColumnName, o);
            }
        }
        return data;
    }

    public int getCount() {
        return getDataStorage().getCount();
    }

    public void setAddedColumnName(String s) /*throws UnavailableColumnNameException*/ {
        
        if(s == null)
            return;
        addedColumnName = s;
    }

    public String getAddedColumnName() {
        return addedColumnName;
    }

    @Override
    public JComponent getGUI() {
        if (gui == null) {
            gui = new AddColumnPanel(this);
        }
        Box b = Box.createVerticalBox();
        b.add(gui);
        b.add(getDataStorage().getGUI());

        return b;
    }

    @Override
    public void setProperties(String prefix, Properties properties) {
        String prefOld = prefix;
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + addColumnDSPropertyString + ".";
        String dsClassName = properties.getProperty(prefix + dataStorageClassPropertyString, "");
        setAddedColumnName(properties.getProperty(prefix + addedColumnNamePropertyString, "new"));
        if ("".equals(dsClassName)) {
            return;
        }
        
        setDataStorage(Util.loadDataStorage(dsClassName, prefix, properties, this));
        super.setProperties(prefOld, properties);
        initialize();
    }

    @Override
    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + addColumnDSPropertyString + ".";
        properties.put(prefix + addedColumnNamePropertyString, addedColumnName);

        if (getDataStorage() != null) {
            properties.put(prefix + dataStorageClassPropertyString, getDataStorage().getClass().getCanonicalName());
            properties = getDataStorage().getProperties(prefix, properties);
        }
        return properties;
    }
}
