// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
grammar ExprGrammar;
// Grammar for ExprDataStorage
// compile with:  java org.antlr.Tool ExprGrammar.g
// having antlr.jar in your CLASSPATH ( export CLASSPATH=$CLASSPATH:/usr/local/java/antlrworks.jar )
// You wil have the following warnings as we change the $type for the ID token:

options {
    output=AST;
	language = Java ;
    ASTLabelType=CommonTree; 
}

tokens {
	LBRACE		= '{' ;
	RBRACE		= '}' ;
	LPAREN		= '(' ;
	RPAREN		= ')' ;
	DOT		        = '.' ;
	SEMIC		    = ';' ;
	COMMA		= ',' ;
	LT		        = '<' ;
	GT		        = '>' ;
	LTE		        = '<=' ;
	GTE		        = '>=' ;
	EQ		        = '=' ;
	NEQ		        = '<>' ;
    NOTEQ         = '!=';
	ADD		    = '+' ;
	SUB		        = '-' ;
	MUL		        = '*' ;
	MOD		    = '%' ;
	AND		    = '&' ;
	OR		        = '|' ;
	XOR		        = '^' ;
	INV		        = '~' ;
    // ASSIGN		= '=' ;
	DIV		        = '/' ;
    LIKE             = 'like';
    REGEXP       = 'regexp';
    ARGS;
    LIST;
    NEG;
    POS;

    DATACOLUMNNAME;
    OMCOLUMNNAME;
}


// applies only to the parser:
@header {
    package  fr.unicaen.openmap.dataStorage.expr;
}

// applies only to the lexer:
@lexer::header {
    package  fr.unicaen.openmap.dataStorage.expr;
    import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
}
@lexer::members {
    protected DataTableModel dtm;
    public ExprGrammarLexer(CharStream input, DataTableModel dtm) {
            this(input);
            this.dtm = dtm;
    }
    public boolean isDataColumName(String s) {
        if (dtm == null)
            return false;
        for(String name : dtm.getDataColumnNames())
            if (name.equals(s)) 
                return true;
        return false;
    }  
    public boolean isOMColumName(String s) {
        if (dtm == null)
            return false;
        for(String name : dtm.getOMColumnNames())
            if (name.equals(s)) 
                return true;
        return false;
    }
}

arg 
    : value
    ;

args 
    : arg (COMMA! arg)*
    ;

value
    : arithmeticExpression
    | geometry
    | list
    ;
list 
    : LBRACE list_content? RBRACE -> ^(LIST list_content)
    ;
list_content
    : value (COMMA! value)*
    ;

 
primaryExpression
    : TRUE
    | FALSE
    | DATACOLUMNNAME
    | INT | FLOAT
    | arithmeticFunctionCall
    | patternMatch
    | LPAREN! arithmeticExpression RPAREN!
    | stringExpr
    ;

arithmeticFunctionCall
    : FUNCTION_N_TO_N^ LPAREN! arithmeticExpression RPAREN! 
    | FUNCTION_G_TO_N^ LPAREN! geometry RPAREN!
    | FUNCTION_G_G_TO_N^ LPAREN! geometry COMMA! geometry RPAREN!
    ;    

geometry 
    : OMCOLUMNNAME
    | geometryFunctionCall
    ;

geometryFunctionCall
    : FUNCTION_N_N_TO_G^ LPAREN! arithmeticExpression COMMA! arithmeticExpression RPAREN! 
    | FUNCTION_G_N_TO_G^ LPAREN! geometry COMMA! arithmeticExpression RPAREN! 
    | FUNCTION_G_G_TO_G^ LPAREN! geometry COMMA! geometry RPAREN! 
;

patternMatch
    : DATACOLUMNNAME LIKE^ STRING
    | DATACOLUMNNAME REGEXP^ STRING
    ;

stringExpr
    : STRING
    | FUNCTION_S_SPLUS_TO_S^ LPAREN! arithmeticExpression (COMMA! arithmeticExpression)+ RPAREN! 
    | FUNCTION_S_N1OR2_TO_S^ LPAREN! arithmeticExpression COMMA! arithmeticExpression (COMMA! arithmeticExpression)? RPAREN!
    | FUNCTION_S_TO_N^ LPAREN! arithmeticExpression RPAREN! 
    | FUNCTION_S_TO_S^ LPAREN! arithmeticExpression RPAREN! 
    ;
unaryExpression
	: primaryExpression
	| unaryOperator^ unaryExpression
	;
	
unaryOperator
	: NOT
    | op=ADD { $op.setType(POS); }
	| op=SUB { $op.setType(NEG); }
	;

multiplicativeExpression
	: unaryExpression ( ( MUL | DIV | MOD )^ unaryExpression )*
	;

additiveExpression
	: multiplicativeExpression ( ( ADD | SUB )^ multiplicativeExpression )*
	;

relationalExpression
	: additiveExpression ( ( LT | GT | LTE | GTE  )^ additiveExpression )*
	;

equalityExpression
	: relationalExpression ( ( EQ | NEQ )^ relationalExpression )*
	;

logicalANDExpression
	: equalityExpression ( LAND^ equalityExpression )*
	;
	
logicalORExpression
	: logicalANDExpression ( LOR^ logicalANDExpression )*
	;
	
arithmeticExpression /* conditionalExpression */
	: logicalORExpression
    | IF^ LPAREN! logicalORExpression SEMIC! arithmeticExpression SEMIC! arithmeticExpression RPAREN!
	;


// lexer

TRUE : 'TRUE' | 'true';

FALSE : 'FALSE' | 'false';

// We reserve few names inorder to suppress ANTLR warning (see ID rule below)
FUNCTION_N_TO_N : 'COS';
FUNCTION_G_TO_N : 'ST_AREA';
FUNCTION_G_G_TO_N : 'CENTROIDDISTANCE';
FUNCTION_N_N_TO_G : 'MAKEPOINT';
FUNCTION_G_N_TO_G : 'MAKEDISC';
FUNCTION_G_G_TO_G : 'ST_UNION';
FUNCTION_S_TO_N : 'LENGTH';
FUNCTION_S_TO_S : 'TRIM';
FUNCTION_S_SPLUS_TO_S : 'CONCAT';
FUNCTION_S_N1OR2_TO_S : 'SUBSTRING';
OMCOLUMNNAME : 'SHAPE';
DATACOLUMNNAME : 'DATA';
IF : ('if'|'IF');
NOT : ('!'|'not'|'NOT');
LAND : ('&&'|'and'|'AND');
LOR : ('||'|'or'|'OR');

ID  :	('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'0'..'9'|'_'|'/')*
        { 
            /* change $type from ID to the type of the list the token belongs to */
            if (isDataColumName($text)) $type = DATACOLUMNNAME;
            else if (isOMColumName($text)) $type = OMCOLUMNNAME;
            else if (Function.isFunction_n_to_n($text)) $type = FUNCTION_N_TO_N; 
            else if (Function.isFunction_g_to_n($text)) $type = FUNCTION_G_TO_N; 
            else if (Function.isFunction_g_g_to_n($text)) $type = FUNCTION_G_G_TO_N; 
            else if (Function.isFunction_n_n_to_g($text)) $type = FUNCTION_N_N_TO_G; 
            else if (Function.isFunction_g_n_to_g($text)) $type = FUNCTION_G_N_TO_G; 
            else if (Function.isFunction_g_g_to_g($text)) $type = FUNCTION_G_G_TO_G; 
            else if (Function.isFunction_s_to_n($text)) $type = FUNCTION_S_TO_N; 
            else if (Function.isFunction_s_to_s($text)) $type = FUNCTION_S_TO_S; 
            else if (Function.isFunction_s_splus_to_s($text)) $type = FUNCTION_S_SPLUS_TO_S; 
            else if (Function.isFunction_s_n1or2_to_s($text)) $type = FUNCTION_S_N1OR2_TO_S; 
        }
    ;

INT :	'0'..'9'+
    ;

FLOAT
    :   ('0'..'9')+ '.' ('0'..'9')* EXPONENT?
    |   '.' ('0'..'9')+ EXPONENT?
    |   ('0'..'9')+ EXPONENT
    ;

COMMENT
    :   '/*' ( options {greedy=false;} : . )* '*/' {$channel=HIDDEN;}
    ;

WS  :   ( ' '
        | '\t'
        | '\r'
        | '\n'
        ) {$channel=HIDDEN;}
    ;

STRING
    :  '"' ( ESC_SEQ | ~('\\'|'"') )* '"'
    ;

fragment
EXPONENT : ('e'|'E') ('+'|'-')? ('0'..'9')+ ;

fragment
HEX_DIGIT : ('0'..'9'|'a'..'f'|'A'..'F') ;

fragment
ESC_SEQ
    :   '\\' ('b'|'t'|'n'|'f'|'r'|'\"'|'\''|'\\')
    |   UNICODE_ESC
    |   OCTAL_ESC
    ;

fragment
OCTAL_ESC
    :   '\\' ('0'..'3') ('0'..'7') ('0'..'7')
    |   '\\' ('0'..'7') ('0'..'7')
    |   '\\' ('0'..'7')
    ;

fragment
UNICODE_ESC
    :   '\\' 'u' HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT
    ;
