// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.expr.gui;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;

import fr.unicaen.openmap.dataStorage.expr.AddColumnDataStorage;
import fr.unicaen.openmap.dataStorage.expr.Function;
import fr.unicaen.openmap.dataStorage.expr.FunctionDescription;
import fr.unicaen.openmap.exception.UnavailableColumnNameException;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class AddColumnPanel extends JPanel {

    protected I18n i18n = Environment.getI18n();
    protected JList fieldsList;
    protected JList geoList;
    protected ExpressionPanel exprPanel;
    protected JTextField fieldColumnName;
    public AddColumnPanel(final AddColumnDataStorage ds) {
        setLayout(new BorderLayout());
        fieldsList = new JList(ds.getDataStorage().getDefaultDataTableModel().getDataColumnNames());
        fieldsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        geoList = new JList(ds.getDataStorage().getDefaultDataTableModel().getOMColumnNames());
        geoList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        JScrollPane sc = new JScrollPane(fieldsList);
        sc.setPreferredSize(new Dimension(200, 150));
        sc.setMaximumSize(new Dimension(200, 150));
        JScrollPane scGeo = new JScrollPane(geoList);
        scGeo.setPreferredSize(new Dimension(200, 150));
        scGeo.setMaximumSize(new Dimension(200, 150));

        fieldsList.addListSelectionListener(new ListSelectionListener() {

            public void valueChanged(ListSelectionEvent lse) {
                if (lse.getValueIsAdjusting()) {
                   exprPanel.appendText(fieldsList.getSelectedValue().toString());
                    try {
                        fieldsList.setSelectedIndices(new int[]{});
                    } catch (Exception e) {
                    }
                }
            }
        });
        geoList.addListSelectionListener(
                new ListSelectionListener() {

                    public void valueChanged(ListSelectionEvent lse) {
                        if (lse.getValueIsAdjusting()) {
                            exprPanel.appendText(geoList.getSelectedValue().toString());
                            try {
                                geoList.setSelectedIndices(new int[]{});
                            } catch (Exception e) {
                            }
                        }
                    }
                });

        JPanel panButtons = new JPanel(new GridLayout(8, 3));

        for (final FunctionDescription f: Function.getDescriptions()) {
            JButton b = new ExprButton(f.getName());
            b.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent arg0) {
                    exprPanel.appendText(f.getName() + f.getPattern());
                }
            });
            panButtons.add(b);
            ComponentSizer.fixComponentSize(b, new Dimension(90,20));
            b.setToolTipText(f.getDoc());
        }

        JPanel panCenter = new JPanel();
        JPanel wrapFieldsList = new JPanel(new BorderLayout());
        wrapFieldsList.add(new JLabel(i18n.get(this, "dataColumns", "Data columns")), BorderLayout.NORTH);
        wrapFieldsList.add(sc, BorderLayout.CENTER);
        panCenter.add(wrapFieldsList);
        JPanel wrapGeoList = new JPanel(new BorderLayout());
        wrapGeoList.add(new JLabel(i18n.get(this, "geometryColumns", "Geometry columns")), BorderLayout.NORTH);
        wrapGeoList.add(scGeo, BorderLayout.CENTER);
        panCenter.add(wrapGeoList);
        panCenter.add(panButtons);
        JPanel panNewCol = new JPanel();
        panNewCol.add(new JLabel(i18n.get(this, "nameOfTheNewColumn", "Name of the New Column")));
        fieldColumnName =
                new JTextField(10);
        panNewCol.add(fieldColumnName);
        fieldColumnName.setText(ds.getAddedColumnName());
        //add(panNewCol, BorderLayout.NORTH);
        add(panCenter, BorderLayout.CENTER);
        fieldColumnName.addKeyListener(new KeyListener() {

            public void keyTyped(KeyEvent arg0) {
            }

            public void keyPressed(KeyEvent arg0) {
            }

            public void keyReleased(KeyEvent arg0) {
                if(ds.isAnExistingColumnName(fieldColumnName.getText()))
                    JOptionPane.showMessageDialog(null, 
						  i18n.get(this, "alreadyUsedColumnName", "Column name already used"), 
						  fieldColumnName.getText(), JOptionPane.ERROR_MESSAGE);
                else
                    ds.setAddedColumnName(fieldColumnName.getText());
            }
        });

        exprPanel = new ExpressionPanel(ds);
        exprPanel.add(panNewCol, 0);
        add(exprPanel, BorderLayout.SOUTH);
        setBorder(BorderFactory.createTitledBorder(i18n.get(this, "title", "Expression DS")));
    }

}
