// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * SelectionStoragePanel.java
 *
 * Created on 9 novembre 2005, 23:31
 *
 * @author Christophe.Turbout@info.unicaen.fr
 *
 */

package fr.unicaen.openmap.dataStorage.gui;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.dataStorage.SelectionDataStorage;
import fr.unicaen.openmap.dataStorage.SelectionDataStorageListener;
import fr.unicaen.openmap.layer.preLayer.UnicaenLayerListener;
import fr.unicaen.openmap.util.Util;
import java.awt.*;
import java.util.Locale;
import javax.swing.*;


/**
 * This class is used to display the content of a TableModel.
 *
 */
public class SelectionStoragePanel extends JPanel implements SelectionDataStorageListener {
    private JTabbedPane tabs;
    private DataTableSelectionStoragePanel datas = null;
    private SelectionMatrixPanel selections = null;
    private SelectionColorPanel colors = null;
    private UnicaenLayerListener unicaenLayerListener = null;
    private SelectionDataStorage selectionDataStorage = null;
    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     **/
    protected I18n i18n = new BasicI18n(new Locale("fr","FR"));
    
    public  SelectionStoragePanel(SelectionDataStorage selectionDataStorage) {
        super();
        this.selectionDataStorage=selectionDataStorage;
        setDoubleBuffered(true);
        setLayout(new BorderLayout());
        
        // JTabpanned
        tabs = new JTabbedPane();
        add(tabs, BorderLayout.CENTER);
        
        // the components
        try {
            // first one : datas
            datas = new DataTableSelectionStoragePanel(selectionDataStorage);
            // disable the delete menu popup
            datas.setMenuUsage(false);
            // select only one row 
            datas.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            // set the listener to update other views
            datas.setTableModeListener();
            // add datas to tabbedPane
            tabs.addTab(i18n.get(this,"Data","Data"),null,datas);
            // second one : selections
            selections = new SelectionMatrixPanel(selectionDataStorage);
            if (selections !=null) tabs.addTab(i18n.get(this,"Selections","Selections"),null,selections);
            // third one : colors
            colors = new SelectionColorPanel(selectionDataStorage);
            if (colors!=null)tabs.addTab(i18n.get(this,"Colors","Colors"),null,colors);
        } catch (Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
    }
    /**
     *  Add UnicaenLayerListener to refresh layer();
     */
    public void addUnicaenListener(UnicaenLayerListener ull){
        colors.addUnicaenListener(ull);
        selections.addUnicaenListener(ull);
        unicaenLayerListener=ull;
    }
    
    /**
     *  Remove the UnicaenLayerListener;
     */
    public void removeUnicaenListener(){
        colors.removeUnicaenListener();
        selections.removeUnicaenListener();
        unicaenLayerListener=null;
    }
    
    /**
     * We have to update panel, data structue has changed. This method is used when the structure is not the same which
     * means a column are a row has been added or suppressed.
     *
     **/
    public void fireStructureChanged(){
        datas.fireStructureChanged();
        selections.fireStructureChanged();
        colors.fireStructureChanged();
    }
    
    /**
     * if we have to add an unique column to an unique SelectionMatrix
     * we need to update the good datas.
     **/
    
    public void fireStructureColumnAdd(int selection, int subselection){
        Debug.message(Util.className(this),this+"FireStructureColumnAdd "+ selection+" , colonne "+subselection);
        // update de datas => fireStructureChanged.
        datas.fireStructureChanged();
        // update the selection panel and fireTableStructureChanged it's datas.
        selections.fireColumnAdded(selection, subselection);
        // update the colors
        colors.fireColumnAdded(selection,subselection);
    }
    
    
    /**
     * if we have to remove an unique column to an unique SelectionMatrix
     * we need to update the good datas.
     **/
    
    public void fireStructureColumnRemoved(int selection, int subselection){
        Debug.message(Util.className(this),this+"FireStructureColumnRemove "+ selection+" , colonne "+subselection);
        // update the selection panel and fireTableStructureChanged it's datas.
        selections.fireColumnRemoved(selection, subselection);
        // update the colors
        colors.fireStructureChanged();
        // update de datas => fireStructureChanged.
        datas.fireStructureChanged();
    }
    
    public void fireDataChanged(int selection, int subSelection, int row, boolean state){
        //update de datas => fireTableDataChanged on the current selection.
        datas.fireTableDataChanged();
        // update the the the model in others views.
        selections.fireDataChanged(selection,subSelection,row, state);
        // no need to update colors.
    }
    
    public void fireSelectionNameChanged(int selection, String name){
        selections.renameTab(selection,name);
        selectionDataStorage.getDataAndSelectionModel().updateMatrixName(selectionDataStorage.getCurrentSelectionName());
        datas.fireStructureChanged();
        // strange to have to get the current Selection and not take selection parameter ! 
        //colors.fireStructureChanged(selectionDataStorage.getCurrentSelection());
        colors.fireStructureChanged(selection);
    }

    /*
     * the name of a subselection in selection has changed. We need to notify it.
     */
    public void fireSelectionSubNameChanged(int selection){
        //selections.renameTab(selection,name);
        //selectionDataStorage.getDataAndSelectionModel().updateMatrixName(selectionDataStorage.getCurrentSelectionName());
        datas.fireStructureChanged();
        colors.fireStructureChanged(selection);
    }
    
    public DataTableSelectionStoragePanel getDataTableSelectionStoragePanel(){
        return datas;
    }
    
    public void setActiveSelection(int selection){
        selections.setActiveSelection(selection);
        colors.setActiveSelection(selection);
    }
    
    public void fireUpdateColors(){
        datas.fireUpdateColors();
        selections.fireUpdateColors();
    }

    public SelectionMatrixPanel getSelections() {
        return selections;
    }

    public UnicaenLayerListener getUnicaenLayerListener() {
        return unicaenLayerListener;
    }
    
    public int getCurrentSelectionSize() {
        return selectionDataStorage.getCurrentSelectionSize();
    }
}
