// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.kernel;

import java.util.ArrayList;
import fr.unicaen.openmap.dataStorage.BoundingBox;


public class Gaussian extends Kernel {

    protected ArrayList<Integer> cellIndices;

    public Gaussian() {
	super();
    }

    public Gaussian(double radius) {
	super(radius);
    }

    public double k(double d) {
	double u = d/radius;
	return Math.exp(u*u/-2);
    }

    public void prepare(float longitudeResolution, float latitudeResolution, BoundingBox bb) {
	cellIndices = new ArrayList<Integer>();

	int i, j;
	int iMin = (int) Math.floor(bb.getLowerRightLat()/latitudeResolution);
	int jMin = (int) Math.floor(bb.getUpperLeftLon()/longitudeResolution);
	int iMax = (int) Math.floor(bb.getUpperLeftLat()/latitudeResolution);
	int jMax = (int) Math.floor(bb.getLowerRightLon()/longitudeResolution);
	
	for (j = jMin; j < jMax; j++) {
	    for (i = iMin; i < iMax; i++) {
		cellIndices.add(new Integer(j));
		cellIndices.add(new Integer(i));
	    }
	}
    }

    public ArrayList<Integer> getSurroundingCellIndices(int lon, int lat) {
	// we don't need neither lon nor lat
	return cellIndices;
    }
}
