// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.kml;

import java.awt.Color;
import java.util.Iterator;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.bbn.openmap.omGraphics.OMArc;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.omGraphics.OMRect;
import com.bbn.openmap.omGraphics.OMText;

public class KMLFactory {

    /**
     * @param omgl
     * @param doc
     */
    private static void createKmlElements(OMGraphicList omgl, KMLDocument doc) {
        Iterator it = omgl.iterator();
        Object o = null;
        while (it.hasNext()) {
            o = it.next();
            if (o instanceof OMGraphicList) {
                createKmlElements((OMGraphicList) omgl, doc);
            } else if (o instanceof OMGraphic) {
                doc.getDocument().appendChild(
                        createKmlElement((OMGraphic) o, doc));
            }
        }
    }

    /**
     * @param omgl
     * @return Build a KML DOM Document from an OMGraphicList
     */
    public static Document createKmlElements(OMGraphicList omgl) {
        KMLDocument document = new KMLDocument();
        createKmlElements(omgl, document);
        return document.getDocument();
    }

    public static Element createKmlElement(OMGraphic omg, KMLDocument doc) {

        if (omg instanceof OMPoly || omg instanceof OMRect) {
            return (new KMLPolygon(omg)).toKml(doc);
        }

        /*
         * if(omg instanceof OMLine) return (new
         * KmlLine((OMLine)omg,doc)).toKml();
         */

        if (omg instanceof OMText) {
            return (new KMLPlacemark((OMText) omg)).toKml(doc);
        }

        if (omg instanceof OMArc) {
            return (new KMLPolygon(omg)).toKml(doc);
        }

        return null;
    }

    public static void exportOmgListWithStax(OMGraphicList omgl, KMLWriter writer) {
        Iterator it = omgl.iterator();
        Object o = null;
        while (it.hasNext()) {
            o = it.next();
            if (o instanceof OMGraphicList) {
                exportOmgListWithStax((OMGraphicList) o, writer);
                return;
            }
        }
        if (o instanceof OMGraphic) {
            exportOmgWithStax((OMGraphic) o, writer);
        }
    }

    public static void exportOmgWithStax(OMGraphic omg, KMLWriter writer) {
        if (omg instanceof OMPoly || omg instanceof OMRect) {
            //System.out.println("exportOmgWithStax: poly ou rect");
            (new KMLPolygon(omg)).exportWithStax(writer);
            return;
        }
        if (omg instanceof OMText) {
            //System.out.println("exportOmgWithStax: Text");
            (new KMLPlacemark((OMText) omg)).exportWithStax(writer);
            return;
        }
        if (omg instanceof OMArc) {
            //System.out.println("exportOmgWithStax: ARC");
            (new KMLPolygon(omg)).exportWithStax(writer);
            return;
        }
        //System.out.println("exportOmgWithStax: INCONNU");
    }

    public static String stringColorFromColor(Color color) {
        StringBuffer c = new StringBuffer();
        // System.out.println();
        String tmp = Integer.toHexString(color.getAlpha());
        if (tmp.equals("0")) {
            tmp = "00";
        }
        c.append(tmp);
        tmp = Integer.toHexString(color.getBlue());
        if (tmp.equals("0")) {
            tmp = "00";
        }
        c.append(tmp);
        tmp = Integer.toHexString(color.getGreen());
        if (tmp.equals("0")) {
            tmp = "00";
        }
        c.append(tmp);
        tmp = Integer.toHexString(color.getRed());
        if (tmp.equals("0")) {
            tmp = "00";
        }
        c.append(tmp);


        return c.toString();

    }
}
