// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.kml;

import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.PathIterator;

import javax.xml.stream.XMLStreamWriter;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.geo.Geo;
import com.bbn.openmap.omGraphics.OMArc;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.omGraphics.OMRect;

public class KMLPolygon implements KMLElement {

	private OMGraphic omGraphic;

	public KMLPolygon(OMGraphic omGraphic) {
		this.omGraphic = omGraphic;
	}

	public Element toKml(KMLDocument doc) {
		if (omGraphic instanceof OMPoly)
			return buildDom(doc, buildPolyCoords((OMPoly) omGraphic));

		if (omGraphic instanceof OMRect)
			return buildDom(doc, buildRectCoords((OMRect) omGraphic));
		
		if (omGraphic instanceof OMArc)
			return buildDom(doc, buildArcCoords((OMArc) omGraphic));
		return null;
	}

	private String buildRectCoords(OMRect omrect) {
		int type = omrect.getRenderType();
		switch (type) {
		case OMGraphic.RENDERTYPE_OFFSET:
			Shape s = omrect.getShape();
			Rectangle rect = s.getBounds();

			int x1 = (int) rect.getX();
			int y1 = (int) rect.getY();
			int x2 = x1 + (int) rect.getWidth();
			int y2 = y1;
			int x3 = x2;
			int y3 = y1 + (int) rect.getHeight();
			int x4 = x1;
			int y4 = y3;

			LatLonPoint p1 = KMLExporter.CURRENT_PROJECTION.inverse(x1, y1);
			LatLonPoint p2 = KMLExporter.CURRENT_PROJECTION.inverse(x2, y2);
			LatLonPoint p3 = KMLExporter.CURRENT_PROJECTION.inverse(x3, y3);
			LatLonPoint p4 = KMLExporter.CURRENT_PROJECTION.inverse(x4, y4);

			StringBuffer coords = new StringBuffer();
			coords.append(p1.getLongitude() + "," + p1.getLatitude() + "\n");
			coords.append(p2.getLongitude() + "," + p2.getLatitude() + "\n");
			coords.append(p3.getLongitude() + "," + p3.getLatitude() + "\n");
			coords.append(p4.getLongitude() + "," + p4.getLatitude() + "\n");
			coords.append(p1.getLongitude() + "," + p1.getLatitude() + "\n");

			return coords.toString();
		}
		return null;
	}

	private String buildArcCoords(OMArc omarc){
		double rad = 120000;/*omarc.getRadius();*/
		int angle = (int)omarc.getExtentAngle();
		double start = omarc.getStartAngle();
		LatLonPoint center = omarc.getCenter();
		//System.out.println("rayon = "+rad+";angle = "+angle+";start= "+start
				//+";centre = "+center.getLatitude()+","+center.getLongitude());
		return KMLCircle.genCircle(
					center.getLatitude(),
					center.getLongitude(),
					angle,
					rad,
					start);
	}
	
	private String buildPolyCoords(OMPoly ompoly) {

		double[] coords = ompoly.getLatLonArray();
		StringBuffer ch = new StringBuffer();
		for (int i = 0; i < coords.length; i = i + 2) {

			ch.append(Geo.degrees(coords[i + 1]) + "," + Geo.degrees(coords[i])
					+ "\n");
		}
		return ch.toString();
	}

	private Element buildDom(KMLDocument document, String coords) {
		KMLExporter.NB_FIG++;
		Document doc = document.getDocument();
		Element el = doc.createElement(KMLConstants.KML_POLY);
		Element obIs = doc.createElement(KMLConstants.KML_OUTER_BOUNDARY);
		Element lr = doc.createElement(KMLConstants.KML_LINEAR_RING);
		Node txt = doc.createTextNode(coords);
		Element c = doc.createElement(KMLConstants.KML_COORDINATES);
		c.appendChild(txt);
		lr.appendChild(c);
		obIs.appendChild(lr);
		el.appendChild(obIs);
		Element pl = doc.createElement(KMLConstants.KML_PLACEMARK);
		Element name = doc.createElement(KMLConstants.KML_NAME);
		name.appendChild(doc.createTextNode(String
				.valueOf(KMLExporter.NB_FIG)));
		/*Element st = document.createElement(KMLConstants.KML_STYLE_URL);
		st.appendChild(document.createTextNode(style()));*/
		pl.appendChild(name);
		pl.appendChild(buildStyle(document));
		pl.appendChild(el);
		//System.out.println("couleur :"+omGraphic.getDisplayColor().toString());
		return pl;
	}
	
	private Element buildStyle(KMLDocument document){
		Document doc = document.getDocument();
		Element st = doc.createElement(KMLConstants.KML_STYLE_URL);
		st.appendChild(
				doc.createTextNode(document.
						addStyle(KMLFactory.stringColorFromColor(omGraphic.getFillColor()))));
		return st;
	}
	
	public void exportWithStax(KMLWriter writer){
		
		String color = KMLFactory.stringColorFromColor(omGraphic.getFillColor());
		KMLExporter.NB_FIG++;
		String coords = "";
		if (omGraphic instanceof OMPoly)
			coords = buildPolyCoords((OMPoly) omGraphic);
		else{
		if (omGraphic instanceof OMRect)
			coords = buildRectCoords((OMRect) omGraphic);
		else{
		if (omGraphic instanceof OMArc)
			coords = buildArcCoords((OMArc) omGraphic);}
		}
		
		writer.writePolygon(String.valueOf(KMLExporter.NB_FIG), "#style"+color, coords);
		writer.writeStyle("style"+color, color);
	}
}
