// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * DbfDataStorage.java
 *
 * Created on 19 march 2006
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 *
 */
package fr.unicaen.openmap.dataStorage.shapefile;


import fr.unicaen.openmap.dataStorage.*;

import com.bbn.openmap.util.PropUtils;
import com.bbn.openmap.util.FileUtils;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.dataAccess.shape.input.DbfInputStream;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.dataStorage.gui.DataTableStoragePanel;

import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.LonLatNames;
import fr.unicaen.openmap.util.propertyEditor.DataStoragePropertyEditor;
import fr.unicaen.openmap.util.propertyEditor.FileTypeDescription;

import javax.swing.*;
import java.awt.event.*;
import java.util.Iterator;
import java.util.ArrayList;
import java.net.URL;
import java.util.Arrays;
import java.util.Properties;


/**
 * DbfDataStorage loads its data from a dbf file
 * <p>
 * @author Jacques.Madelaine@info.unicaen.fr
 * <p>
 * @see DbfInputStream
 **/
public class DbfDataStorage extends InformationDataStorage {


    public static final String PARAM_DBF = "dbf";

    public final static String DBF_BY_INFO = "dbf.info";

   /**
     * Name of the dbf file
     */
    private String dbfFile = "";
    
    
    public DbfDataStorage(DataStorageListener dsl) {
	super(dsl);
    }

    protected boolean initialized = false;

    public String getName() {
        return "".equals(dbfFile) ? name : stripFileNameWithClassPathDirectories(dbfFile);
    }

    /**
     * Tells if the dataStorage has it's data.
     */
    public boolean isReady() {
	return initialized;
    }
        
    public JComponent getGUI() {
	JPanel panel = new JPanel();
	/*
	final JLabel dbfLabel = new JLabel("Dbf file: "+dbfFile);
	JButton dbfButton = new JButton("Change Dbf File");
	dbfButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    String _dbfFile = FileUtils.getFilePathToOpenFromUser("Dbf File");
		    if (_dbfFile == null || "".equals(_dbfFile)) {
			System.err.println("No dbf file choosen");
			return;
		    }
		    dbfFile = _dbfFile;
		    dbfLabel.setText("Dbf file: "+dbfFile);
		    initialize();
		}
	    });
	panel.add(dbfLabel); 
	panel.add(dbfButton);
	*/
	panel.add(DataStoragePropertyEditor.provideCustomPanel(this));

	return panel; 
    }
    
 
    /**
     * most of this code is borrowed from com.bbn.openmap.plugin.esri.EsriLayer.setProperties
     */
    public void setProperties(String prefix, Properties properties) {        
	super.setProperties(prefix, properties);

        prefix = PropUtils.getScopedPropertyPrefix(prefix);

	dbfFile = properties.getProperty(prefix + PARAM_DBF, 
					 properties.getProperty(prefix + DBF_BY_INFO, ""));

	Debug.message("DataStorage", "[DbfDataStorage] using DBF file \""+ dbfFile +"\"");
    
	initialize();
    }
   
    public Properties getProperties(String prefix, Properties properties) {
	properties = super.getProperties(prefix, properties);
	prefix = PropUtils.getScopedPropertyPrefix(prefix);

	properties.put(prefix + PARAM_DBF, stripFileNameWithClassPathDirectories(dbfFile));

	//Used with getPropertyInfo	    
	properties.put(prefix + DBF_BY_INFO,
		       new FileTypeDescription(FileTypeDescription.DBF_DESCRIPTION, dbfFile, FileTypeDescription.DBF_PATTERN));

	return properties;
    }

    public Properties getPropertyInfo(Properties properties) {
	//useless
	properties = super.getPropertyInfo(properties);
	/*
	PropUtils.setI18NPropertyInfo(i18n,
				      properties,
				      DbfDataStorage.class,
				      PARAM_DBF,
				      PARAM_DBF,
				      PARAM_DBF,
				      "com.bbn.openmap.util.propertyEditor.FilePropertyEditor"
				      );
	*/
	PropUtils.setI18NPropertyInfo(i18n,
				      properties,
				      DbfDataStorage.class,
				      DBF_BY_INFO,
				      "File",
				      "Select a DBF file.",
				      "fr.unicaen.openmap.util.propertyEditor.FileTypePropertyEditor"
				      );

	properties.put(initPropertiesProperty, 
		       //		       PARAM_DBF
		       DBF_BY_INFO
		       );

	return properties;
    }

    private ArrayList<String> typeNames;      // string, lon or lat
    private LonLatNames lonLatNames;    // suffixes of column names starting with longitude or latitude

    private String getTypeName(int i) {
    	if (i >= typeNames.size())
    	    typeNames.add("string");
    	return typeNames.get(i);
    }
    


    protected void initialize() {
	ArrayList<String> columnNames = new ArrayList<String>(); // column names from the dbf file
	typeNames = new ArrayList<String>();
	lonLatNames = new LonLatNames();

	try {
	    URL dbf = PropUtils.getResourceOrFileOrURL(dbfFile);
	    DbfInputStream dis = new DbfInputStream(dbf.openStream());
            
	    Iterator dbfIterator = dis.getRecords().iterator();
            
	    String[] dbfNames = dis.getColumnNames();
	    for (int i = 0 ; i < dbfNames.length; i++) {
		columnNames.add(dbfNames[i]);
		String type = lonLatNames.getLonLatType(dbfNames[i]);
		if (type != null) // we have a lon or lat column name
		    typeNames.add(type);
		else
		    typeNames.add("string");
	    }

	    ArrayList<DataObject> dataObjects = new ArrayList<DataObject>() ;
	    if (dbfIterator!= null) {
		int lineCount = 0;
		while (dbfIterator.hasNext()) {
		    lineCount++;
		    DataObject tempDataObject = new DataObject();		
		    ArrayList record = (ArrayList)dbfIterator.next();
		    Iterator<String> c = columnNames.iterator();
		    int t = 0; // index for type names
		    for (Iterator i = record.iterator() ; i.hasNext(); t++) {
			// add the attribute only if it is member of the attribute list
			String cname = c.next();
			Object obj = i.next();
			String type = getTypeName(t);   
			if (isDataAttributeNameToLoad(cname)) {
			    tempDataObject.addData(cname, obj);
			    try {
				if (LonLatNames.LON.equals(type))
				    lonLatNames.putLon(t, obj);
				else if (LonLatNames.LAT.equals(type))
				    lonLatNames.putLat(t, obj);
			    } catch(java.lang.ClassCastException e) {
				System.err.println("*** DbfDataStorage bad cast to Double line "+lineCount+" for "+ type + " \"" + obj + "\"");
			    } catch(java.lang.NumberFormatException e) {
				System.err.println("*** DbfDataStorage number format exception line "+lineCount+" for "+ type + " \"" + obj + "\"");
			    }
			}

		    }
		    // gather the centroids
		    lonLatNames.putCentroids(tempDataObject);

		    dataObjects.add(tempDataObject);		
		    // Debug.message("DataStorage", "[DbfDataStorage] adding " + tempDataObject);
		}

		Debug.message("DataStorage", "[DbfDataStorage] column names: "+ columnNames);
		Debug.message("DataStorage", "[DbfDataStorage] " + dataObjects.size() + " rows loaded");
		
		dataTable.setTable(dataObjects,
				   getDataAttributesNamesToDisplay(columnNames), 
				   lonLatNames.getOMGraphicNames(),
				   getDataAttributesNamesAliases());
		// data are stored, we can create dataTableModel and panel to view them,
		// if they don't already exist
		if (dataTableModel == null)
		    dataTableModel = new DataTableModel(dataTable);
		resetDataTableStoragePanel(dataTableModel);

		initialized = true;

	    }       
	    // this Event must be used when initialize() will use thread ...
	    Debug.message("DataStorage", "[DbfDataStorage] Lancement du DataStorageEvent vers " + dsl);
	    dsl.dataStorageEvent(this);

	} catch (java.io.IOException e) {
	    System.err.println("DbfDataStorage: IOException for dbfFile " + dbfFile);
	    initialized = false;
	} catch (java.lang.Exception e) {
	    System.err.println("DbfDataStorage: Exception for dbfFile " + dbfFile);
	    e.printStackTrace();
	}

	
    }
}
