// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * ShapeFileIndexedDataStorage.java
 *
 * Created on april 2006
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 *
 */
package fr.unicaen.openmap.dataStorage.shapefile;


import fr.unicaen.openmap.dataStorage.*;

import com.bbn.openmap.util.PropUtils;
import com.bbn.openmap.util.Debug;

import com.bbn.openmap.layer.shape.SpatialIndex;
// import com.bbn.openmap.omGraphics.DrawingAttributes;
// import com.bbn.openmap.layer.shape.ESRIBoundingBox;
import com.bbn.openmap.layer.shape.ESRIRecord;
// import com.bbn.openmap.layer.shape.ShapeFile;
// import com.bbn.openmap.LatLonPoint;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.dataAccess.shape.input.DbfInputStream;
import com.bbn.openmap.MoreMath;

// import fr.unicaen.openmap.dataStorage.postgis.db.DBTableRow;
// import fr.unicaen.openmap.dataStorage.postgis.db.DBTable;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.dataStorage.gui.DataTableStoragePanel;

// import fr.unicaen.openmap.util.Util;

import javax.swing.JComponent;
// import javax.swing.JLabel;
import java.util.Vector;
import java.util.Iterator;
import java.util.ArrayList;
import java.net.URL;
import java.util.List;


/**
 * ### Warning : do not use, this class is not finished yet !<p>
 * DataStorage getting its data from a shape file and a dbf companion file 
 * using a shape spatial index file (ssx).
 * <p>ssx files differ from standard Esri shx file as the index entry contains the shape bounding box
 * (@see SpatialIndex). <code>ssx</code> files may be created from <code>shp</code> file with : <br/>
 * java com.bbn.openmap.layer.shape.SpatialIndex -c file.ssx file.shp
 * <p>
 * Exemple of properties :
 * <pre>
 * exemple.class=fr.unicaen.openmap.layer.dataDisplayLayer.GeometryDataDisplayLayer
 * exemple.prettyName= Exemple
 * exemple.lineColor=FF000000
 * exemple.fillColor=FFFFFFFF
 * exemple.selectedLineColor=FF990099
 * exemple.selectedFillColor=C900FF55
 * exemple.fill=no
 * exemple.geometryColumnName=shape
 * exemple.projectionChangePolicy=pcp
 * exemple.removable=true
 * exemple.renderPolicy=rp
 * exemple.rp.class=com.bbn.openmap.layer.policy.BufferedImageRenderPolicy
 * # properties specific to the DataStorage
 * exemple.dataStorageClass=fr.unicaen.openmap.dataStorage.shapefile.ShapeFileDataStorage
 * exemple.shp=/home/jacques/shape/frdptsdd.shp
 * exemple.dbf=/home/jacques/shape/frdptsdd.dbf
 * # if the ssx property is not set, the ssx filename is deduced from the shape file name 
 * exemple.ssx=/home/jacques/shape/frdptsdd.ssx
 * </pre>
 *
 * The default name for the geometry in the DataStorage is geometryColumnName
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 *
 **/
public class ShapeFileIndexedDataStorage extends ShapeFileDataStorage  {
    
    public static final java.lang.String PARAM_SSX = "ssx";
    /** Name of the shape spatial index file */
    protected String ssxFile;
    /** The spatial index of the shape file to be rendered. */
    protected SpatialIndex spatialIndex;
    /** Dbf records */
    //protected ArrayList records = null;
    List<java.util.List<java.lang.Object>> records = null;
    /** Column names from the dbf file    */
    ArrayList<String> columnNames; 
    public ShapeFileIndexedDataStorage(DataStorageListener dsl) {
        super(dsl);
    }
    
    
    /**
     * Default Implementation of getGUI() method.
     * @return null
     */
    public JComponent getGUI() {
        //return new JLabel("No configuration yet for " + Util.className(this));
        return null;
    }
    
    /**
     * 
     */
    public void setProperties(String prefix, java.util.Properties properties) {
        super.setProperties(prefix, properties);
        
        ssxFile = properties.getProperty(prefix + PARAM_SSX);
        if ((ssxFile == null || ssxFile.equals("")) && shpFile != null)
            ssxFile = shpFile.substring(0, shpFile.lastIndexOf('.') + 1) + PARAM_SSX;

        Debug.message("DataStorage", "[ShapeFileIndexedDataStorage] using shapefile \"" 
		      + this.shpFile +"\" shape spatial index file \"" + this.ssxFile + "\""
		      + " and DBF file \""+ dbfFile +"\" => "+ geometryColumnName + " geometry column name");
	//		      + " waiting for ProjectionChanged event...");
	initialize();
    }
    
    
    public java.util.Properties getProperties(String prefix, java.util.Properties properties) {
	properties = super.getProperties(prefix, properties);
        properties.put(prefix+PARAM_SHP, shpFile);
        properties.put(prefix+PARAM_DBF, dbfFile);
        properties.put(prefix+PARAM_GEOMETRYCOLUMNNAME, geometryColumnName);
        return properties;
    }
    
    protected void initialize() {

	    // SpatialIndex is to be derived in SpatialShapeAndDbfIndex
	    spatialIndex = SpatialIndex.locateAndSetShapeData(/*ssxFile, */shpFile);


	columnNames = new ArrayList<String>();
	if (dbfFile != null)
            try {
                URL dbf = PropUtils.getResourceOrFileOrURL(dbfFile);
                DbfInputStream dis = new DbfInputStream(dbf.openStream());
                
                records = dis.getRecords();
                
                String[] dbfNames = dis.getColumnNames();
                for (int i = 0 ; i < dbfNames.length; i++)
                    columnNames.add(dbfNames[i]);
            } catch (java.io.IOException e) {
                System.err.println("ShapeFileDataStorage: IOException for " + dbfFile);
            } catch (java.lang.Exception e) {
                System.err.println("ShapeFileDataStorage: Exception for " + dbfFile);
            }


            Debug.message("DataStorage", "[ShapeFileIndexedDataStorage] initialized using " + shpFile);

            ArrayList<String> omGraphicNames = new ArrayList<String>();
            omGraphicNames.add(geometryColumnName);
	    dataTable.setTable(new ArrayList<DataObject>(), getDataAttributesNamesToDisplay(columnNames), omGraphicNames, getDataAttributesNamesAliases());
            // data are stored, we can create dataTableModel and panel to view them.
            dataTableModel = new DataTableModel(dataTable);
            resetDataTableStoragePanel(dataTableModel);
            initialized = true;

	    // this Event must be used when initialize() will use thread ...
	    dsl.dataStorageEvent(this);
    }

    protected void initialize(float ulLat, float ulLon, float lrLat, float lrLon) {

	//	code borrowed from com.bbn.openmap.layer.shape.ShapeLayer.prepare()

        OMGraphicList list = null;


        // check for dateline anomaly on the screen. we check for
        // ulLon >= lrLon, but we need to be careful of the check for
        // equality because of floating point arguments...
        if ((ulLon > lrLon)
                || MoreMath.approximately_equal(ulLon, lrLon, .001f)) {
            if (Debug.debugging("DataStorage")) {
                Debug.output("[ShapeFileIndexedDataStorage] Dateline is on screen");
            }

            double ymin = (double) Math.min(ulLat, lrLat);
            double ymax = (double) Math.max(ulLat, lrLat);

            try {
                ESRIRecord records1[] = spatialIndex.locateRecords(ulLon,
                        ymin,
                        180.0d,
                        ymax);
                ESRIRecord records2[] = spatialIndex.locateRecords(-180.0d,
                        ymin,
                        lrLon,
                        ymax);
                int nRecords1 = records1.length;
                int nRecords2 = records2.length;


                list = new OMGraphicList(nRecords1 + nRecords2);
                for (int i = 0; i < nRecords1; i++) {
                    records1[i].addOMGraphics(list, drawingAttributes);
                }
                for (int i = 0; i < nRecords2; i++) {
                    records2[i].addOMGraphics(list, drawingAttributes);
                }

            } catch (java.io.InterruptedIOException iioe) {
                // This means that the thread has been interrupted,
                // probably due to a projection change. Not a big
                // deal, just return, don't do any more work, and let
                // the next thread solve all problems.
                list = null;
            } catch (java.io.IOException ex) {
                ex.printStackTrace();
            } catch (com.bbn.openmap.io.FormatException fe) {
                fe.printStackTrace();
            }
        } else {

            double xmin = (double) Math.min(ulLon, lrLon);
            double xmax = (double) Math.max(ulLon, lrLon);
            double ymin = (double) Math.min(ulLat, lrLat);
            double ymax = (double) Math.max(ulLat, lrLat);

            try {
                ESRIRecord records[] = spatialIndex.locateRecords(xmin,
                        ymin,
                        xmax,
                        ymax);
                int nRecords = records.length;
                list = new OMGraphicList(nRecords);
                for (int i = 0; i < nRecords; i++) {
                    records[i].addOMGraphics(list, drawingAttributes);
                }
            } catch (java.io.InterruptedIOException iioe) {
                // This means that the thread has been interrupted,
                // probably due to a projection change. Not a big
                // deal, just return, don't do any more work, and let
                // the next thread solve all problems.
                list = null;
            } catch (java.io.IOException ex) {
                ex.printStackTrace();
            } catch (com.bbn.openmap.io.FormatException fe) {
                fe.printStackTrace();
            }
        }

	
        // this Event must be used when initialize() will use thread ...
        dsl.dataStorageEvent(this);
    }

}
