// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors
 */
// </header>
package fr.unicaen.openmap.gui.menu;

import com.bbn.openmap.gui.menu.MapHandlerMenuItem;
import java.awt.event.*;
import java.awt.*;
import java.io.*;

import com.bbn.openmap.*;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.gui.ThemaMapFileChooserManager;
import fr.unicaen.openmap.util.UnicodePrintStream;
import javax.swing.JFileChooser;


/**
 * A Menu item that generates a Properties object when it is clicked
 * upon. It needs BeanContext object. It iterates through objects in
 * BeanContext and looks for objects that implement PropertyConsumer
 * interface and generates Properties object from them.Wwhen it has
 * found all the Properties, it writes them to a file by providing the
 * user with a file dialog box.
 */
public class SavePropertiesMenuItem extends MapHandlerMenuItem implements
        ActionListener {

    private String titleString;
    private String saveProperties = "Save";

    public SavePropertiesMenuItem(String initString) {
        super(initString);
        titleString = initString;
        addActionListener(this);
    }

    public void actionPerformed(ActionEvent ae) {
        //Collect properties
        if (mapHandler == null) {
            if (Debug.debugging("menu")) {
                Debug.error("SavePropertiesMenuItem: no Map Handler to use");
            }
            return;
        }

        String saveString = ThemaMapFileChooserManager.propertiesSaveDialog(titleString);
        if (saveString==null ||saveString.length()==0) return;
        File file = new File(saveString);
        FileOutputStream fos;
        PrintStream ps = null;
        try {
            fos = new FileOutputStream(file);
        }
        catch (IOException fnfe) {
            System.err.println(fnfe.getMessage());
            return;
        }

        ps = new UnicodePrintStream(fos);

        PropertyHandler.createOpenMapProperties(mapHandler, ps);

        ps.close();
    }
}


