// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.jts;

import java.util.List;
import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.OMDataLocation;
import fr.unicaen.openmap.dataStorage.BoundingBox;
import fr.unicaen.openmap.dataStorage.Centroid;
import fr.unicaen.openmap.dataStorage.expr.SpatialFunction;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMPoint;
import com.bbn.openmap.omGraphics.OMLine;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.geo.Geo;


import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryCollection;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.PrecisionModel;
import com.vividsolutions.jts.geom.LinearRing;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Envelope;

import fr.unicaen.openmap.dataStorage.OMDataLocation;

/** 
 * JTS Geometry to OMGraphic
 */
public class JTSFactory {
    static public GeometryFactory defaultFactory = 
	new GeometryFactory(new PrecisionModel(), 4326);

    /** From OpenMap structures to JTS structures */
    static public Geometry OMGraphicToJTSGeometry(OMGraphic omg) {
	    return OMGraphicToJTSGeometry(omg, defaultFactory);
    }
    static public Geometry OMGraphicToJTSGeometry(OMGraphic omg, GeometryFactory factory) {
	if (omg instanceof OMGraphicList) 
	    return OMGraphicToJTSGeometry((OMGraphicList)omg, factory);
	if (omg instanceof OMPoint)
	    return OMGraphicToJTSGeometry((OMPoint)omg, factory);
	if (omg instanceof OMLine)
	    return OMGraphicToJTSGeometry((OMLine)omg, factory);
	if (omg instanceof OMPoly)
	    return OMGraphicToJTSGeometry((OMPoly)omg, factory);
	    
	String errmsg = "fr.unicaen.openmap.jts.JTSFactory.OMGraphicToJTSGeometry: unsupported class for "+omg;
	assert false : errmsg;
	System.err.println(errmsg);
	return null;
    }

	static public Geometry OMGraphicToJTSGeometry(OMGraphicList omglist, GeometryFactory factory) {
	int size = omglist.size();
	Geometry geometries[] = new Geometry[size];
	for(int i = 0; i < size; i++)
	    geometries[i] = OMGraphicToJTSGeometry(omglist.get(i));
	return factory.createGeometryCollection(geometries);
    }

    static public Geometry OMGraphicToJTSGeometry(OMPoint omp, GeometryFactory factory) {
	return factory.createPoint(new Coordinate(omp.getLon(), omp.getLat()));
    }

    static public Geometry OMGraphicToJTSGeometry(OMLine oml, GeometryFactory factory) {
	return factory.createLineString(extractCoordinates(oml));
    }

    static public Geometry OMGraphicToJTSGeometry(OMPoly omp, GeometryFactory factory) {
	return factory.createLinearRing(extractCoordinates(omp));
    }

    /** extract Coordinates from OMGraphic */
    static public Coordinate[] extractCoordinates(OMGraphic omg) {
	if (omg instanceof OMGraphicList) 
	    return extractCoordinates((OMGraphicList)omg);
	if (omg instanceof OMPoint)
	    return extractCoordinates((OMPoint)omg);
	if (omg instanceof OMLine)
	    return extractCoordinates((OMLine)omg);
	if (omg instanceof OMPoly)
	    return extractCoordinates((OMPoly)omg);
	    
	String errmsg = "fr.unicaen.openmap.jts.JTSFactory.extractCoordinates: unsupported class for "+omg;
	assert false : errmsg;
	System.err.println(errmsg);
	return null;
    }

    static public Coordinate[] extractCoordinates(OMGraphicList omglist) {
	int size = omglist.size();
	Coordinate[][] coords = new Coordinate[size][];
	int nbCoordinates = 0;
	for (int i = 0; i < size; i++) {
	    coords[i] = extractCoordinates(omglist.get(i));
	    nbCoordinates += coords[i].length;
	}
	// merge the coords elements into the result
	Coordinate[] coordinates = new Coordinate[nbCoordinates];
	int j = 0;
	for (int i = 0; i < size; i++)
	    for (Coordinate c : coords[i])
		coordinates[j++] = c;
	return coordinates;
    }

    static public Coordinate[] extractCoordinates(OMPoint omp) {
	Coordinate[] coords = new Coordinate[1];
	coords[0] = new Coordinate(omp.getLon(), omp.getLat());
	return coords;
    }

    static public Coordinate[] extractCoordinates(OMLine oml) {
	double lls[] = oml.getLL();
	int nbPoints = lls.length/2;
	Coordinate coords[] = new Coordinate[nbPoints];
	for(int i = 0; i < nbPoints; i++)
	    coords[i] = new Coordinate(lls[2*i+1], lls[2*i]);
	return coords;
    }

    static public Coordinate[] extractCoordinates(OMPoly omp) {
	double rawllpts[] = omp.getRawllpts();
	double llpts[] = new double[rawllpts.length];
	
	for(int i = 0; i < rawllpts.length; i++)
	    llpts[i] = Geo.degrees(rawllpts[i]);
	int nbPoints = llpts.length/2;
	Coordinate coords[] = new Coordinate[nbPoints];
	for(int i = 0; i < nbPoints; i++)
	    coords[i] = new Coordinate(llpts[2*i+1], llpts[2*i]);
	
	return coords;
    }

    /** From JTS structures to OpenMap structures */
    static public OMGraphic JTSGeometryToOMGraphic(Geometry geom) {
	if (geom instanceof GeometryCollection)
	    return JTSGeometryToOMGraphic((GeometryCollection)geom);
	if (geom instanceof LinearRing)
	    return JTSGeometryToOMGraphic((LinearRing)geom);
	if (geom instanceof LineString)
	    return JTSGeometryToOMGraphic((LineString)geom);
	if (geom instanceof Point)
	    return JTSGeometryToOMGraphic((Point)geom);
	if (geom instanceof Polygon)
	    return JTSGeometryToOMGraphic((Polygon)geom);
	String errmsg = "fr.unicaen.openmap.jts.JTSFactory.JTSGeometryToOMGraphic: unsupported class for "+geom;
	// assert false : errmsg;
	System.err.println(errmsg);
	return null;
    } 

    static public OMGraphic JTSGeometryToOMGraphic(GeometryCollection geom) {
	OMGraphicList omgl = new OMGraphicList();
	for (int i = 0; i < geom.getNumGeometries(); i++) {
	    omgl.add(JTSGeometryToOMGraphic(geom.getGeometryN(i)));
	}
	return omgl;
    }

    static public OMGraphic JTSGeometryToOMGraphic(LinearRing geom) {
 	Coordinate coords[] = geom.getCoordinates();
	double rawllpts[] = new double[coords.length *2];
	for (int i = 0; i < coords.length; i++) {
	    rawllpts[i*2] = coords[i].y;
	    rawllpts[i*2+1] = coords[i].x;
	}
	return new OMPoly(rawllpts, OMGraphic.DECIMAL_DEGREES, OMGraphic.LINETYPE_STRAIGHT);   
    }

    static public OMGraphic JTSGeometryToOMGraphic(LineString geom) {
	Coordinate coords[] = geom.getCoordinates();
	if (coords.length == 2) 
	    return new OMLine(coords[0].y, coords[0].x, coords[1].y, coords[1].x, OMGraphic.LINETYPE_STRAIGHT);
	double rawllpts[] = new double[coords.length *2];
	for (int i = 0; i < coords.length; i++) {
	    rawllpts[i*2] = coords[i].y;
	    rawllpts[i*2+1] = coords[i].x;
	}
	return new OMPoly(rawllpts, OMGraphic.DECIMAL_DEGREES, OMGraphic.LINETYPE_STRAIGHT);
    }

    static public OMGraphic JTSGeometryToOMGraphic(Point geom) {
	return new OMPoint(geom.getY(), geom.getX());
    }

    static public OMGraphic JTSGeometryToOMGraphic(Polygon geom) {
	return JTSGeometryToOMGraphic(geom.getExteriorRing());
    }


    static public OMDataLocation JTSGeometryToOMDataLocation(Geometry geom) {
	if (geom == null)
	    return null;
	Envelope env = geom.getEnvelopeInternal();
	Centroid centroid = null;
	BoundingBox bb = null;
	OMGraphic omg = JTSFactory.JTSGeometryToOMGraphic(geom);
	Point geomCentroid = geom.getCentroid();
	if (geomCentroid != null) {
	    Coordinate c = geom.getCentroid().getCoordinate();
	    centroid = new Centroid(c.x, c.y);
	    bb = new BoundingBox(env.getMinX(), env.getMaxY(), env.getMaxX(), env.getMinY());
	} 
	
	return new OMDataLocation(JTSGeometryToOMGraphic(geom),
				  centroid,
				  bb);
    }
}
