/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.unicaen.openmap.layer;

import com.bbn.openmap.MapBean;
import com.bbn.openmap.MoreMath;
import com.bbn.openmap.omGraphics.OMColorChooser;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.omGraphics.OMText;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.gui.ThemaMapColorChooser;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMLabel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableGraphicList;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Offseter;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import java.beans.beancontext.BeanContext;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 *
 * @author jchauvea
 */
public class GraticuleLayer extends com.bbn.openmap.layer.GraticuleLayer {

    public final static String equatorCmd = "equatorCmd";
    public final static String graticuleCmd = "graticuleCmd";
    private int latLineStep = 10;
    private int lonLineStep = 10;
    private int latLabStep = 1;
    private int lonLabStep = 1;
    private OMResizeableGraphicList legendList;
    //private OMResizeableFrame graticuleFrame;
    //private LegendLayer legend;
    //private OMGraphicList legendLabels = new OMGraphicList();
    //private HashMap<String, OMLabel> legendLabels = new HashMap<String, OMLabel>();
    //private Offseter offseter = new Offseter(2,-2);

    public GraticuleLayer() {
        super();
    }

    @Override
    /** Create the ten degree lines. */
    protected OMGraphicList constructTenDegreeLines() {

        OMGraphicList lines = new OMGraphicList(3);
        OMPoly currentLine;

        // generate other parallels of latitude by creating series
        // of polylines
        for (int i = 1; i <= 80 / latLineStep; i++) {
            for (int j = -1; j < 2; j += 2) {
                float lat = (float) (latLineStep * i * j);
                // generate parallel of latitude North/South of the
                // equator
                double[] llp = {lat, -180f, lat, -90f, lat, 0f, lat, 90f, lat,
                    180f};
                currentLine = new OMPoly(llp, OMGraphic.DECIMAL_DEGREES, boxy ? OMGraphic.LINETYPE_STRAIGHT
                        : OMGraphic.LINETYPE_RHUMB);
                currentLine.setLinePaint(tenDegreeColor);
                lines.add(currentLine);
            }
        }

        // generate lines of longitude
        for (int i = 1; i < 180 / lonLineStep; i++) {

            for (int j = -1; j < 2; j += 2) {
                float lon = (float) (lonLineStep * i * j);
                // not quite 90.0 for beautification reasons.
                double[] llp = {80f, lon, 0f, lon, -80f, lon};
                if (MoreMath.approximately_equal(Math.abs(lon), 90f, 0.001f)) {
                    llp[0] = 90f;
                    llp[4] = -90f;
                }
                currentLine = new OMPoly(llp, OMGraphic.DECIMAL_DEGREES, boxy ? OMGraphic.LINETYPE_STRAIGHT
                        : OMGraphic.LINETYPE_GREATCIRCLE);
                currentLine.setLinePaint(tenDegreeColor);
                lines.add(currentLine);
            }
        }

        if (Debug.debugging("graticule")) {
            Debug.output("GraticuleLayer.constructTenDegreeLines(): " + "constructed " + lines.size() + " graticule lines");
        }
        lines.generate(getProjection());
        return lines;
    }

    @Override
    public Component getGUI() {
        JPanel gui = new JPanel(new BorderLayout());
        Box boxConf = Box.createHorizontalBox();
        ThematicConfPanel tcpLine = ThematicConfPanel.createVerticalThematicConfPanel("Lines preferences");
        ThematicConfPanel tcpLabel = ThematicConfPanel.createVerticalThematicConfPanel("Labels preferences");

        Box boxLine = Box.createVerticalBox();

        JSpinner latSpinner = new JSpinner(new SpinnerNumberModel(
                latLineStep,
                1,
                90,
                1));
        latSpinner.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent ce) {
                JSpinner sp = (JSpinner) ce.getSource();
                latLineStep = Integer.parseInt(sp.getValue().toString());
                refreshGraphics();
            }
        });
        //lon step
        JSpinner lonSpinner = new JSpinner(new SpinnerNumberModel(
                lonLineStep,
                1,
                180,
                1));
        lonSpinner.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent ce) {
                JSpinner sp = (JSpinner) ce.getSource();
                lonLineStep = Integer.parseInt(sp.getValue().toString());
                refreshGraphics();
            }
        });
        JPanel spinnersLinePanel = new JPanel(new GridLayout(2, 2));
        JPanel wrap = new JPanel();
        wrap.add(new JLabel("Latitude step"));
        spinnersLinePanel.add(wrap);
        wrap = new JPanel();
        wrap.add(latSpinner);
        spinnersLinePanel.add(wrap);
        wrap = new JPanel();
        wrap.add(new JLabel("Longitude step"));
        spinnersLinePanel.add(wrap);
        wrap = new JPanel();
        wrap.add(lonSpinner);
        spinnersLinePanel.add(wrap);
        boxLine.add(spinnersLinePanel);

        //color buttons
        JPanel colButtons = new JPanel();
        JPanel eqPanCol = new JPanel();
        PropertyColorButton equatorButton = new PropertyColorButton(equatorColor);
        equatorButton.setActionCommand(equatorCmd);
        equatorButton.addActionListener(this);
        eqPanCol.add(new JLabel("Equator color"));
        eqPanCol.add(equatorButton);
        colButtons.add(eqPanCol);

        JPanel gratPanCol = new JPanel();
        PropertyColorButton graticuleButton = new PropertyColorButton(tenDegreeColor);
        graticuleButton.setActionCommand(graticuleCmd);
        graticuleButton.addActionListener(this);
        gratPanCol.add(new JLabel("Graticule color"));
        gratPanCol.add(graticuleButton);
        colButtons.add(gratPanCol);

        boxLine.add(colButtons);

        //redraw button
//        JButton bDraw = new JButton("Apply");
//        bDraw.addActionListener(new ActionListener() {
//
//            public void actionPerformed(ActionEvent arg0) {
//                refreshGraphics();
//            }
//        });
        //boxLine.add(bDraw);
        tcpLine.setContent(boxLine);
        boxConf.add(tcpLine);

        //labels
        Box boxLabel = Box.createVerticalBox();
        JSpinner latLabSpinner = new JSpinner(new SpinnerNumberModel(
                latLabStep,
                1,
                90,
                1));
        latLabSpinner.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent ce) {
                JSpinner sp = (JSpinner) ce.getSource();
                latLabStep = Integer.parseInt(sp.getValue().toString());
                refreshGraphics();
            }
        });


        JSpinner lonLabSpinner = new JSpinner(new SpinnerNumberModel(
                lonLabStep,
                1,
                90,
                1));
        lonLabSpinner.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent ce) {
                JSpinner sp = (JSpinner) ce.getSource();
                lonLabStep = Integer.parseInt(sp.getValue().toString());
                refreshGraphics();
            }
        });
        JPanel spinnerLabelPanel = new JPanel(new GridLayout(2, 2));
        wrap = new JPanel();
        wrap.add(new JLabel("Latitude step"));
        spinnerLabelPanel.add(wrap);
        wrap = new JPanel();
        wrap.add(latLabSpinner);
        spinnerLabelPanel.add(wrap);
        wrap = new JPanel();
        wrap.add(new JLabel("Longitude step"));
        spinnerLabelPanel.add(wrap);
        wrap = new JPanel();
        wrap.add(lonLabSpinner);
        spinnerLabelPanel.add(wrap);
        boxLabel.add(spinnerLabelPanel);

        tcpLabel.setContent(boxLabel);
        boxConf.add(tcpLabel);

        gui.add(boxConf, BorderLayout.WEST);
        return gui;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        super.actionPerformed(e);
        if (e.getActionCommand().equals(equatorCmd)) {
            JButton source = (JButton) e.getSource();
            equatorColor = ThemaMapColorChooser.showDialog(
                    source,
                    "Choose Color",
                    equatorColor);
            source.setBackground(equatorColor);
            refreshGraphics();
            return;
        }

        if (e.getActionCommand().equals(graticuleCmd)) {
            JButton source = (JButton) e.getSource();
            tenDegreeColor = ThemaMapColorChooser.showDialog(
                    source,
                    "Choose Color",
                    tenDegreeColor);
            source.setBackground(tenDegreeColor);
            refreshGraphics();
            return;
        }
    }

    @Override
    /** Constructs the Dateline and Prime Meridian lines. */
    protected OMGraphicList constructMarkerLines() {
        OMGraphicList lines = new OMGraphicList(3);
        OMPoly currentLine;

        // generate Prime Meridian and Dateline
        for (int j = 0; j < 360; j += 180) {
            float lon = (float) j;
            double[] llp = {90f, lon, 0f, lon, -90f, lon};
            currentLine = new OMPoly(llp, OMGraphic.DECIMAL_DEGREES, boxy ? OMGraphic.LINETYPE_STRAIGHT
                    : OMGraphic.LINETYPE_GREATCIRCLE);
            currentLine.setLinePaint(Color.ORANGE);
            lines.add(currentLine);
        }

        // equator
        double[] llp = {0f, -180f, 0f, -90f, 0f, 0f, 0f, 90f, 0f, 180f};
        // polyline
        currentLine = new OMPoly(llp, OMGraphic.DECIMAL_DEGREES, boxy ? OMGraphic.LINETYPE_STRAIGHT
                : OMGraphic.LINETYPE_GREATCIRCLE);
        currentLine.setLinePaint(equatorColor);
        lines.add(currentLine);

        if (Debug.debugging("graticule")) {
            Debug.output("GraticuleLayer.constructMarkerLines(): " + "constructed " + lines.size() + " graticule lines");
        }
        lines.generate(getProjection());
        return lines;
    }

    protected OMGraphicList constructGraticuleLines() {

        OMGraphicList newgraphics = new OMGraphicList(20);
        // Lets figure out which lines should be painted...
        Projection projection = getProjection();

        if (projection == null) {
            return newgraphics;
        }
        tenDegreeLines = null;

        if (showOneAndFiveLines || showRuler || showBelowOneLines) {

            Point2D ul = projection.getUpperLeft();
            Point2D lr = projection.getLowerRight();

            float left = (float) ul.getX();
            float right = (float) lr.getX();
            float up = (float) ul.getY();
            float down = (float) lr.getY();

            if (up > 80.0f)
                up = 80.0f;
            if (down > 80.0f)
                down = 80f; // unlikely
            if (up < -80.0f)
                up = -80.0f; // unlikely
            if (down < -80)
                down = -80.0f;

            //int showWhichLines = SHOW_ONES;//evaluateSpacing(up, down, left, right);

            // Find out whether we need to do one or two queries,
            // depending on if we're straddling the dateline.
            if ((left > 0 && right < 0) || (left > right) || (Math.abs(left - right) < 1)) {
                // Test to draw the ones and fives, which will also do
                // the labels.

                //if (showWhichLines != SHOW_TENS) {
                newgraphics.add(constructGraticuleLines(up,
                        down,
                        left,
                        180.0f/*,
                        showWhichLines*/));
                newgraphics.add(constructGraticuleLines(up,
                        down,
                        -180.0f,
                        right/*,
                        showWhichLines*/));
//                } else if (showRuler) { // Just do the labels for the
//                    // tens lines
//                    newgraphics.add(constructTensLabels(up,
//                            down,
//                            left,
//                            180.0f,
//                            true));
//                    newgraphics.add(constructTensLabels(up,
//                            down,
//                            -180.0f,
//                            right,
//                            false));
//                }
            } else {
                // Test to draw the ones and fives, which will also do
                // the labels.
                //if (showWhichLines != SHOW_TENS) {
                newgraphics = constructGraticuleLines(up,
                        down,
                        left,
                        right/*,
                        showWhichLines*/);
                /*} else if (showRuler) { // Just do the labels for the
                // tens lines
                newgraphics.add(constructTensLabels(up,
                down,
                left,
                right,
                true));
                }*/
            }
        }

        OMGraphicList list;
        if (tenDegreeLines == null) {
            list = constructTenDegreeLines();
            tenDegreeLines = list;
        } else {
            synchronized (tenDegreeLines) {
                setLineTypeAndProject(tenDegreeLines,
                        boxy ? OMGraphic.LINETYPE_STRAIGHT
                        : OMGraphic.LINETYPE_RHUMB);
            }
        }
        if (markerLines == null) {
            list = constructMarkerLines();
            markerLines = list;
        } else {
            synchronized (markerLines) {
                setLineTypeAndProject(markerLines,
                        boxy ? OMGraphic.LINETYPE_STRAIGHT
                        : OMGraphic.LINETYPE_RHUMB);
            }
        }

        newgraphics.add(markerLines);
        newgraphics.add(tenDegreeLines);

        if (Debug.debugging("graticule")) {
            Debug.output("GraticuleLayer.constructGraticuleLines(): " + "constructed " + newgraphics.size() + " graticule lines");
        }

        return newgraphics;
    }

    protected OMGraphicList constructGraticuleLines(float up, float down,
            float left, float right/*,
            int showWhichLines*/) {
        OMGraphicList lines = new OMGraphicList();

        // Set the line limits for the lat/lon lines...
        int north = (int) Math.ceil(up);
        if (north > 80)
            north = 80;

        int south = (int) Math.floor(down);
        south -= (south % 10); // Push down to the lowest 10 degree
        // line.
        // for neg numbers, Mod raised it, lower it again. Also
        // handle straddling the equator.
        if ((south < 0 && south > -80) || south == 0)
            south -= 10;

        int west = (int) Math.floor(left);
        west -= (west % 10);
        // for neg numbers, Mod raised it, lower it again. Also
        // handle straddling the prime meridian.
        if ((west < 0 && west > -180) || west == 0)
            west -= 10;

        int east = (int) Math.ceil(right);
        if (east > 180)
            east = 180;

        int stepSize;
        // Choose how far apart the lines will be.
        stepSize = latLineStep;//((showWhichLines == SHOW_ONES) ? 1 : 5);
        double[] llp;
        OMPoly currentLine;
        OMText currentText;

        // For calculating text locations
        Point point = new Point();
        LatLonPoint llpoint;

        Projection projection = getProjection();

        // generate other parallels of latitude be creating series
        // of polylines
        for (int i = south; i < north; i += stepSize) {
            float lat = (float) i;
            // generate parallel of latitude North/South of the
            // equator
            if (west < 0 && east > 0) {
                llp = new double[6];
                llp[2] = lat;
                llp[3] = 0f;
                llp[4] = lat;
                llp[5] = east;
            } else {
                llp = new double[4];
                llp[2] = lat;
                llp[3] = east;
            }
            llp[0] = lat;
            llp[1] = west;

            // Do not duplicate the 10 degree line.
            if ((lat % 10) != 0) {
                currentLine = new OMPoly(llp, OMGraphic.DECIMAL_DEGREES, boxy ? OMGraphic.LINETYPE_STRAIGHT
                        : OMGraphic.LINETYPE_RHUMB);
                if ((lat % 5) == 0) {
                    currentLine.setLinePaint(fiveDegreeColor);
                } else {
                    currentLine.setLinePaint(oneDegreeColor);
                }
                lines.add(currentLine);
            }

            if (showRuler && (lat % latLabStep == 0)) {
                if (boxy) {
                    projection.forward(lat, west, point);
                    point.x = 0;
                    llpoint = projection.inverse(point.x,
                            point.y,
                            new LatLonPoint.Double());
                } else {
                    llpoint = new LatLonPoint.Double(lat, west);
                    while (projection.forward(llpoint).getX() < 0) {
                        llpoint.setLongitude(llpoint.getX() + stepSize);
                    }
                }

                currentText = new OMText(llpoint.getY(), llpoint.getX(),
                        // Move them up a little
                        (int) 2, (int) -2, Integer.toString((int) lat), font, OMText.JUSTIFY_LEFT);
                currentText.setLinePaint(textColor);
                lines.add(currentText);
            }
        }
        stepSize = lonLineStep;
        // generate lines of longitude
        for (int i = west; i < east; i += stepSize) {
            float lon = (float) i;

            if (north < 0 && south > 0) {
                llp = new double[6];
                llp[2] = 0f;
                llp[3] = lon;
                llp[4] = south;
                llp[5] = lon;
            } else {
                llp = new double[4];
                llp[2] = south;
                llp[3] = lon;
            }
            llp[0] = north;
            llp[1] = lon;

            if ((lon % 10) != 0) {
                currentLine = new OMPoly(llp, OMGraphic.DECIMAL_DEGREES, boxy ? OMGraphic.LINETYPE_STRAIGHT
                        : OMGraphic.LINETYPE_GREATCIRCLE);
                if ((lon % 5) == 0) {
                    currentLine.setLinePaint(fiveDegreeColor);
                } else {
                    currentLine.setLinePaint(oneDegreeColor);
                }
                lines.add(currentLine);
            }

            if (showRuler && (lon % lonLabStep == 0)) {
                if (boxy) {
                    projection.forward(south, lon, point);
                    point.y = projection.getHeight();
                    llpoint = projection.inverse(point.x,
                            point.y,
                            new LatLonPoint.Double());
                } else {
                    llpoint = new LatLonPoint.Double(south, lon);
                    while (projection.forward(llpoint).getY() > projection.getHeight()) {
                        llpoint.setLatitude(llpoint.getY() + stepSize);
                    }
                }

                currentText = new OMText(llpoint.getY(), llpoint.getX(),
                        // Move them up a little
                        (int) 2, (int) -5, Integer.toString((int) lon), font, OMText.JUSTIFY_CENTER);
                currentText.setLinePaint(textColor);
                lines.add(currentText);

            }
        }

        if (Debug.debugging("graticule")) {
            Debug.output("GraticuleLayer.constructTenDegreeLines(): " + "constructed " + lines.size() + " graticule lines");
        }
        lines.generate(projection);
        return lines;
    }

    /**
     * construct labels (not always every ten degrees)
     * @param up
     * @param down
     * @param left
     * @param right
     * @param doLats
     * @return
     */
//    @Override
//    protected OMGraphicList constructTensLabels(float up, float down,
//            float left, float right,
//            boolean doLats) {
//
//        OMGraphicList labels = new OMGraphicList();
//        // Set the line limits for the lat/lon lines...
//        int north = (int) Math.ceil(up);
//        if (north > 80) {
//            north = 80;
//        }
//
//        int south = (int) Math.floor(down);
//        south -= (south % 10); // Push down to the lowest 10 degree
//        // line.
//        // for neg numbers, Mod raised it, lower it again
//        if ((south < 0 && south > -70) || south == 0) {
//            south -= 10;
//        }
//
//        int west = (int) Math.floor(left);
//        west -= (west % 10);
//        // for neg numbers, Mod raised it, lower it again
//        if ((west < 0 && west > -170) || west == 0) {
//            west -= 10;
//        }
//
//        int east = (int) Math.ceil(right);
//        if (east > 180) {
//            east = 180;
//        }
//
//        int stepSize = latStep;
//        //OMLabel currentLabel;
//        OMText currentText;
//        // For calculating text locations
//        Point point = new Point();
//        LatLonPoint llpoint;
//        Projection projection = getProjection();
//
//        if (doLats) {
//            ArrayList<Integer> labs = new ArrayList<Integer>();
//            for (int i = 0; i <= north; i += stepSize) {
//                labs.add(i);
//            }
//            for (int i = -stepSize; i >= south; i -= stepSize) {
//                labs.add(i);
//            }
//            // generate other parallels of latitude be creating series
//            // of labels
//            for (int i : labs) {
//                float lat = (float) i;
//
//                //if ((lat % 2) == 0) {
//                if (boxy) {
//                    projection.forward(lat, west, point);
//                    point.x = 0;
//                    llpoint = projection.inverse(point.x,
//                            point.y,
//                            new LatLonPoint.Double());
//                } else {
//                    llpoint = new LatLonPoint.Double(lat, west);
//                    while (projection.forward(llpoint).getX() < 0) {
//                        llpoint.setLongitude(llpoint.getX() + stepSize);
//                    }
//                }
//
//                currentText = new OMText(llpoint.getY(), llpoint.getX(), (int) 2, (int) -2, // Move
//                        // them
//                        // up a
//                        // little
//                        Integer.toString((int) lat), font, OMText.JUSTIFY_LEFT);
//                String id = "LAT" + String.valueOf(i);
////                currentLabel = new OMLabel(null, id,
////                        (float) llpoint.getY(),
////                        (float) llpoint.getX(),
////                        (int) 2, (int) -2,
////                        Integer.toString((int) lat),
////                        font, OMText.JUSTIFY_LEFT, true);
//                currentText.setLinePaint(textColor);
//                //legendLabels.put(id, currentLabel);
//                //offseter.insert(id/*,(float) llpoint.getY() , (float) llpoint.getX()*/);
//                labels.add(currentText);
//
//            }
//            //  }
//        }
//        System.out.println("------------------------------------------------------");
//        stepSize = lonStep;
//        // generate labels of longitude
//        ArrayList<Integer> labs = new ArrayList<Integer>();
//        for (int i = 0; i <= east; i += stepSize) {
//            labs.add(i);
//        }
//        for (int i = -stepSize; i >= west; i -= stepSize) {
//            labs.add(i);
//        }
//        for (int i : labs) {
//            float lon = (float) i;
//
//            //if ((lon % 2) == 0) {
//            if (boxy) {
//                projection.forward(south, lon, point);
//                point.y = projection.getHeight();
//                llpoint = projection.inverse(point.x,
//                        point.y,
//                        new LatLonPoint.Double());
//            } else {
//                llpoint = new LatLonPoint.Double(south, lon);
//                while (projection.forward(llpoint).getY() > projection.getHeight()) {
//                    llpoint.setLatitude(llpoint.getY() + stepSize);
//                }
//            }
//
//            currentText = new OMText(llpoint.getY(), llpoint.getX(),
//                    // Move them up a little
//                    (int) 2, (int) -5, Integer.toString((int) lon), font, OMText.JUSTIFY_CENTER);
//            if ((int) lon == -180)
//                continue; // -180 label = +180 label; one is enough
//            System.out.println("LON ->" + String.valueOf(i));
////            currentLabel = new OMLabel(null, id,
////                    (float) llpoint.getY(),
////                    (float) llpoint.getX(),
////                    (int) 2, (int) -2,
////                    Integer.toString((int) lon),
////                    font, OMText.JUSTIFY_LEFT, true);
//            currentText.setLinePaint(textColor);
//            labels.add(currentText);
//            //legendLabels.put(id, currentLabel);
//            //offseter.insert(id/*,(float) llpoint.getY() , (float) llpoint.getX()*/);
//
//            // }
//        }
//
//        if (Debug.debugging("graticule")) {
//            Debug.output("GraticuleLayer.constructTensLabels(): " + "constructed " + labels.size() + " graticule labels");
//        }
//        labels.generate(projection);
//        return labels;
//        //return new OMGraphicList();//labels are displayed in a legend layer
//    }
//    public OMGraphicList getLegend() {
//        OMGraphicList legendList = new OMGraphicList();
//        for (OMLabel lab : legendLabels.values())
//            legendList.add(lab);
//        return legendList;
//    }
    public OMGraphicList getLegendControlPoints() {
        OMGraphicList controlPoints = new OMGraphicList();
//        for(OMLabel lab : legendLabels.values()){
//            lab.addOffsetableListener(offseter);
//            controlPoints.add(lab.controlPoints);
//        }
        return controlPoints;
    }
//    public OMGraphicList getLegend(LegendLayer legend) {
//        this.legend = legend;
    ///legendList = new OMResizeableGraphicList(this, legend.getMouseControlPointsListener(), getName());
//        for(OMGraphic omg : legendLabels){
//            OMLabel lab = (OMLabel)omg;
//            legendList.add(OMResizeableText.contructOMResiezableText(
//                    lab,
//                    this,
//                    legend.getMouseControlPointsListener(), getProjection()));
//
//
//        }
    //        graticuleFrame = new OMResizeableFrame(this,
//                legend.getMouseControlPointsListener(),
//                0, 0,
//                getMapBean().getWidth(), getMapBean().getHeight(), "Graticule");
//        graticuleFrame = new OMResizeableFrame(this,
//                legend.getMouseControlPointsListener(),
//                0, 0,
//                getMapBean().getWidth(), getMapBean().getHeight(), "Graticule");
//        graticuleFrame.setBorderLineStroke(new BasicStroke(5));
//        //getMapBean().addPropertyChangeListener(this);//listen map resizing
//        legendList.add(graticuleFrame);
//        System.out.println("NOMBRE DE LABEL DE GRATICULE:" + labels.size());
//        for(OMGraphic oml : labels){
//            legendList.add(OMResizeableText.contructOMResiezableText(
//                    (OMLabel)oml,
//                    this,
//                    legend.getMouseControlPointsListener()));
//        }
////        return legendLabels;
///    }
    public static String graticuleLegendPropertyString = "graticuleLegend";

    public void setLegendProperties(String prefix, Properties properties) {
        legendList.setProperties(prefix + "." + graticuleLegendPropertyString, properties);
    }

    public Properties getLegendProperties(String prefix, Properties properties) {
        return legendList.getProperties(prefix + "." + graticuleLegendPropertyString, properties);

    }

    public void refreshGraphics() {
        markerLines = null;
        if (isVisible()) {
            doPrepare();
        }
    }

    public void refreshInteractiveGraphics() {
        refreshGraphics();
    }

//    public Offseter getOffseter(){return offseter;}

    /*
     * Get the MapBean through the BeanContext. Set the layer property
     * "addToBeanContext" to true in order to be able to get it.
     */
    public MapBean getMapBean() {
        BeanContext mapHandler = getBeanContext();
        if (mapHandler == null) {
            return null;
        }

        Iterator it = mapHandler.iterator();
        while (it.hasNext()) {
            Object o = it.next();
            if (o instanceof MapBean) {
                return (MapBean) o;
            }
        }
        return null;
    }

    class PropertyColorButton extends JButton {

        public PropertyColorButton(Color c) {
            super();
            ComponentSizer.fixComponentSize(this, new Dimension(15, 15));
            setBackground(c);
        }
    }
}
