// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

/*
 * @author davy@info.unicaen.fr
 * code adapted from : Jacques Madelaine GREYC / Nicolas Antoine IRD
 */

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.basic.BasicButtonUI;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import com.bbn.openmap.omGraphics.OMColorChooser;

import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.classification.Classification;
import fr.unicaen.openmap.util.classification.ClassificationOfString;
import fr.unicaen.openmap.util.classification.ClassificationOfDouble;
import fr.unicaen.openmap.util.classification.ClassificationListener;
import fr.unicaen.openmap.util.classification.CustomizedClassification;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;

/**
 * This panel helps configuring color scale and custom classification.
 * Every action method must keep colorScale (actually used) and customColorScale when displayed
 * in sync. Every action could also change the colorScale in Colorizer if not actually the same.
 * <b>Exception</b> : when shading, only actually used color scale is modified.
 */
public class ColorizerColorScalePanel extends JPanel implements ChangeListener, ItemListener, ClassificationListener {
    
    protected Colorizer colorizer = null;
    
    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     */
    public static I18n i18n = new BasicI18n();

    // Some predifined ColorScale.
    private static final ColorScale crayola8 = new ColorScale
	(ColorScalePanel2.crayola8InHex,
	 i18n.get(ColorScalePanel2.class, "crayola8", "Crayola 8"));
    private static final ColorScale crayola10 = new ColorScale
	(ColorScalePanel2.crayola10InHex,
	 i18n.get(ColorScalePanel2.class, "crayola10", "Crayola 10"));
    //	 "Crayola 10"); // Crayola 10 does not exists : it is a subjective subset of Crayola 16
    private static final ColorScale crayolaRed = new ColorScale
	(ColorScalePanel2.crayolaRedInHex,
	 i18n.get(ColorScalePanel2.class,"crayola16redsubset","<html>Crayola 16 <i>(Red Subset)</i></html>"));
    private static final ColorScale crayolaBlue = new ColorScale
	(ColorScalePanel2.crayolaBlueInHex,
	 i18n.get(ColorScalePanel2.class, "crayola16bluesubset", "<html>Crayola 16 <i>(Blue Subset)</i></html>"));

    /*    private static final ColorScale pastelColorScale = new ColorScale
	("dd8b7d6b  ddffe7ba  dda52a2a  dd98f5ff  dd7fff00  ddfff8dc  ddb8860b  dd6e8b3d  ddff1493",
	 i18n.get(ColorScalePanel.class, ColorScalePanel.PASTEL_PROPERTY_NAME, ColorScalePanel. PASTEL_PROPERTY_NAME));
	 //  dd00bfff  dddcdcdc  dd141414", ; // FF69B4 FAF0E6 8B1C62 7D26CD CD9B9B CDB5CD 00C5CD";*/
    private static final ColorScale waterColorScale = new ColorScale
	(ColorScalePanel2.waterColorScaleInHex,
	 i18n.get(ColorScalePanel2.class, ColorScalePanel2.WATER_PROPERTY_NAME, ColorScalePanel2.WATER_PROPERTY_NAME ));
    private static final ColorScale sandColorScale = new ColorScale
	(ColorScalePanel2.sandColorScaleInHex,
	 i18n.get(ColorScalePanel2.class, ColorScalePanel2.SAND_PROPERTY_NAME, ColorScalePanel2.SAND_PROPERTY_NAME));
    private static final ColorScale pastelColorScale = new ColorScale
	(ColorScalePanel2.pastelColorScaleInHex,
	 i18n.get(ColorScalePanel2.class, ColorScalePanel2.PASTEL_PROPERTY_NAME, ColorScalePanel2.PASTEL_PROPERTY_NAME));

    //Others colormap.
    private static final ColorScale camouflageColorScale = new ColorScale
	(ColorScalePanel2.camouflageColorScaleInHex,
	 i18n.get(ColorScalePanel2.class, ColorScalePanel2.CAMOUFLAGE_PROPERTY_NAME, "camouflage"));
    private static final ColorScale warmColorScale = new ColorScale
	(ColorScalePanel2.warmColorScaleInHex, "<html>warm</html>");
	 //i18n.get(ColorScalePanel.class, ColorScalePanel.WARM_PROPERTY_NAME, "<html>warm<i>(test)</i></html>"));
    private static final ColorScale desert1ColorScale = new ColorScale
	(ColorScalePanel2.desert1ColorScaleInHex, "<html>desert1</html>");
	// i18n.get(ColorScalePanel.class, ColorScalePanel.DESERT1_PROPERTY_NAME, "desert1"));
    private static final ColorScale desert2ColorScale = new ColorScale
	(ColorScalePanel2.desert2ColorScaleInHex, "<html>desert2</html>");
	// i18n.get(ColorScalePanel.class, ColorScalePanel.DESERT2_PROPERTY_NAME, "desert2"));
    private static final ColorScale forestColorScale = new ColorScale
	(ColorScalePanel2.forestColorScaleInHex,
	 i18n.get(ColorScalePanel2.class, ColorScalePanel2.FOREST_PROPERTY_NAME, "forest"));
    private static final ColorScale coldColorScale = new ColorScale
	(ColorScalePanel2.coldColorScaleInHex, "<html>cold</html>");
	// i18n.get(ColorScalePanel.class, ColorScalePanel.COLD_PROPERTY_NAME, "cold"));
    private static final ColorScale fruitColorScale = new ColorScale
	(ColorScalePanel2.fruitColorScaleInHex,"<html>fruit</html>");
	// i18n.get(ColorScalePanel.class, ColorScalePanel.FRUIT_PROPERTY_NAME, "fruit"));
    private static final ColorScale blueGreenColorScale = new ColorScale
	(ColorScalePanel2.blueGreenColorScaleInHex,"<html>blue-green</html>");
	// i18n.get(ColorScalePanel.class, ColorScalePanel.BLUE_GREEN_PROPERTY_NAME, "blue-green"));
    private static final ColorScale fleshColorScale = new ColorScale
	(ColorScalePanel2.fleshColorScaleInHex, "<html>flesh</html>");
	// i18n.get(ColorScalePanel.class, ColorScalePanel.FLESH_PROPERTY_NAME, "flesh"));
    private static final ColorScale mineralColorScale = new ColorScale
	(ColorScalePanel2.mineralColorScaleInHex,
	 i18n.get(ColorScalePanel2.class, ColorScalePanel2.MINERAL_PROPERTY_NAME, "mineral"));

    /**
     * Used to keep user change to the color scale in order to re-use or save it.
     */
    private ColorScale customColorScale = new ColorScale
	("ddffffff ddffffff ddffffff ddffffff ddffffff ddffffff ddffffff ddffffff ddffffff ddffffff",
	 i18n.get(this, "custom", "<html><i>custom</i></html>"));

    
    private ColorScale colorScales[] =
    { customColorScale, 
      // crayola8, 
      // crayola10, 
      crayolaRed, 
      crayolaBlue, 
      // pastelColorScale, 
      // waterColorScale, 
      // sandColorScale,
      // camouflageColorScale, 
      warmColorScale, 
      desert1ColorScale, 
      desert2ColorScale, 
      // forestColorScale, 
      coldColorScale, 
      fruitColorScale, 
      blueGreenColorScale, 
      fleshColorScale, 
      // mineralColorScale,
    };
    
    
    /**
     * Actually used Color Scale.
     */
    protected final ColorScale colorScale = new ColorScale
	("ddffffff ddffffff ddffffff ddffffff ddffffff ddffffff ddffffff ddffffff ddffffff ddffffff", "current");

    protected String stringNumberFormat = "%5.2f";

    public CSJButton colorButtons[];
    protected JSpinner colorNumber;
    protected JCheckBox shadedCheckBox;
    protected JComboBox colorMapCombo;
    protected JCheckBox checkReverseColorMap;
    protected JComponent scaleParametersPan;
    
    public ColorizerColorScalePanel(ColorScale cScale, Colorizer colorizer) {
	this.colorScale.copy(cScale);
        this.colorizer = colorizer;
	customColorScale.copy(colorScale);
	//setPreferredSize(new Dimension(400, 400));
	//	setBorder(BorderFactory.createTitledBorder(i18n.get(this,"title","Color Scale")));

	// for(int i = 0 ; i<colorScales.length; i++){
	// 	System.out.println("COLOR "+i+"="+((ColorScale)colorScales[i]).toString()+"***");	
	// }


	colorMapCombo = new JComboBox(colorScales);
	colorMapCombo.setSelectedItem(customColorScale);
	colorMapCombo.addItemListener(this);
	colorMapCombo.setToolTipText(i18n.get(this,"tooltipColorMap","Standard Color Maps"));
	DefaultListCellRenderer renderer = new DefaultListCellRenderer();
	renderer.setHorizontalAlignment(SwingConstants.RIGHT);
	colorMapCombo.setRenderer(renderer);
	colorNumber = new JSpinner(new SpinnerNumberModel(colorScale.getColorNumber(), 1, ClassificationOfDouble.NbClassesMax, 1));
	colorNumber.addChangeListener(this);
	colorNumber.setToolTipText(i18n.get(this, "tooltipColorNb", "Colors Number"));
	shadedCheckBox = new JCheckBox(i18n.get(this, "shadedButton", "Shaded"), false);
	shadedCheckBox.setToolTipText(i18n.get(this, "shadedButton", I18n.TOOLTIP, "Make a shaded colormap"));
	shadedCheckBox.addActionListener(new ActionListener () {
	        public void actionPerformed(ActionEvent e) {
		    if (shadedCheckBox.isSelected()) {
			ColorizerColorScalePanel.this.colorScale.doShade();
			shadedCheckBox.setToolTipText(i18n.get(ColorizerColorScalePanel.class,"tooltipShadedOn", "disable"));
		    } else {
			ColorizerColorScalePanel.this.colorScale.copy((ColorScale) colorMapCombo.getSelectedItem());
			shadedCheckBox.setToolTipText(i18n.get(ColorizerColorScalePanel.class, "tooltipShadedOff", "enable"));
		    }
		    refreshPanel();
		}
	    });

	//bReverseColorMap = new JButton(Util.loadIcon("reverse.png"));
	checkReverseColorMap = new JCheckBox(i18n.get(ColorizerColorScalePanel.class,"reverse", "Reversed map"));
	checkReverseColorMap.addActionListener(new ActionListener () {
	        public void actionPerformed(ActionEvent e) {
		    ColorScale c = (ColorScale) colorMapCombo.getSelectedItem();
		    colorScale.setColorNumber(((Integer) colorNumber.getValue()).intValue());
		    colorScale.setReverse(checkReverseColorMap.isSelected());
		    colorScale.copy(c);
		    shadedCheckBox.setSelected(false);
		    refreshPanel();
		}
	    });
	checkReverseColorMap.setBorder(null);
	checkReverseColorMap.setBackground(Color.GRAY);
	checkReverseColorMap.setOpaque(false);
	colorizer.addClassificationListener(this);
	customClassif = new CustomizedClassification(null, colorScale.getColorNumber());
        scaleParametersPan = Box.createVerticalBox();
        scaleParametersPan.add(colorMapCombo);
        scaleParametersPan.add(colorNumber);
        JPanel pShadeReverse = new JPanel();
        pShadeReverse.add(shadedCheckBox);
        pShadeReverse.add(checkReverseColorMap);
        scaleParametersPan.add(pShadeReverse);
	refreshPanel();
    }

    public JComponent getScaleParametersPan(){ return scaleParametersPan;}
    
    // public void setColorScale(ColorScale c) { colorScale = c; }
    
    /**
     * Refresh the dynamic components.
     * This method will also transfert the current color scale to
     * the colorizer, thus the histogram panel will be refreshed too.
     * Crude way to refresh: empty the panel and re-create the content! */
    public void refreshPanel() {
	colorizer.setColorScale(colorScale);
	setVisible(false);
	removeAll();
	createOptionPanel();
	setVisible(true);
    }

    public void createOptionPanel() {
	Classification c = colorizer.getClassification();
	if (c instanceof ClassificationOfString)
	    createOptionPanelWithoutSpinners();
	else 
	    createOptionPanelWithSpinners();

    }

    public void createOptionPanelWithoutSpinners() {
	setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
	Box pane = Box.createVerticalBox();
	Box topPane = Box.createHorizontalBox();
	//	topPane.add(colorNumber);
	topPane.add(colorMapCombo);
	topPane.add(shadedCheckBox);
	topPane.add(checkReverseColorMap);
	pane.add(topPane);


	colorButtons = new CSJButton[colorScale.getColorNumber()];
	// System.out.println("### colorButtons.length = "+colorButtons.length + " colorScale: " + colorScale);
	for(int i = 0; i < colorButtons.length ; i++){
	    colorButtons[i] = new CSJButton("button", i);
	    colorButtons[i].setBackground(colorScale.getColor(i));
	    colorButtons[i].setText(getColorButtonText(i));
	    pane.add(colorButtons[i]);
	}
	
	setButtonsEnabled(!shadedCheckBox.isSelected());
	JPanel flowPanel = new JPanel();
	flowPanel.add(pane);
	JScrollPane scrollPane = new JScrollPane(flowPanel);
	add(scrollPane);
    }
    
    public void createOptionPanelWithSpinners() {
	// A java.lang.IllegalStateException occurs sometimes! 
	// The panel is always recreated twice!!
	// System.out.println("#####ColorizerColorScalePanel createOptionPanel classif="+colorizer.getClassification());
	GroupLayout.ParallelGroup bPG;//button parallel group
	GroupLayout.SequentialGroup bSG;
	GroupLayout.ParallelGroup spPG;//spinner parallel group
	GroupLayout.SequentialGroup spSG;

        GroupLayout layout = new GroupLayout(this);
        //setLayout(layout);
        /*layout.setHorizontalGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
			      .addComponent(colorNumber, GroupLayout.PREFERRED_SIZE, 40, GroupLayout.PREFERRED_SIZE)
			      .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
			      .addComponent(colorMapCombo, GroupLayout.PREFERRED_SIZE, 160, GroupLayout.PREFERRED_SIZE)
			      .addComponent(shadedCheckBox, GroupLayout.PREFERRED_SIZE, 20, Short.MAX_VALUE)
			      .addComponent(checkReverseColorMap, GroupLayout.PREFERRED_SIZE, 20, Short.MAX_VALUE)
			      )
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(bPG = layout.createParallelGroup(GroupLayout.Alignment.TRAILING))
                        .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(spPG = layout.createParallelGroup(GroupLayout.Alignment.LEADING))))
		      .addContainerGap()));
	
        layout.setVerticalGroup(
            layout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
			  .addComponent(colorMapCombo, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
                    .addComponent(colorNumber, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
		    .addComponent(shadedCheckBox, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
			 .addComponent(checkReverseColorMap, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
                )
                .addGap(11, 11, 11)
                .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
                    .addGroup(spSG = layout.createSequentialGroup())
                    .addGroup(bSG = layout.createSequentialGroup()))
		      .addContainerGap(32, Short.MAX_VALUE))
        );*/

        Box buttonsBox = Box.createVerticalBox();
        Box spinnersBox = Box.createVerticalBox();
	colorButtons = new CSJButton[colorScale.getColorNumber()];
	spinners = new CSJSpinner[colorScale.getColorNumber() - 1];
	
	
	JLabel labColor = new JLabel(i18n.get(this,"color","Colors"));
	JLabel labBreak = new JLabel(i18n.get(this, "break", "Breaks"));

	/*bPG.addComponent(labColor, GroupLayout.Alignment.CENTER);
	bSG.addComponent(labColor);
	spPG.addComponent(labBreak, GroupLayout.Alignment.CENTER);
	spSG.addComponent(labBreak);*/
	CSJButton lower;
	colorButtons[0] = new CSJButton("button", 0);
	colorButtons[0].setBackground(colorScale.getColor(0));
	colorButtons[0].setText(getColorButtonText(0));
	lower = colorButtons[0];
	/*bPG.addComponent(colorButtons[0], GroupLayout.DEFAULT_SIZE, 202, Short.MAX_VALUE);
	bSG.addComponent(colorButtons[0]);
	bSG.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED);
	spSG.addGap(6, 6, 6);*/
	
	// System.out.println("### colorButtons.length = "+colorButtons.length);
	buttonsBox.add(colorButtons[0]);
        buttonsBox.add(Box.createRigidArea(new Dimension(120,2)));
        spinnersBox.add(Box.createRigidArea(new Dimension(120,6)));
	for(int i = 1; i < colorButtons.length ; i++){
	    colorButtons[i] = new CSJButton("button", i);
	    colorButtons[i].setBackground(colorScale.getColor(i));
	    colorButtons[i].setText(getColorButtonText(i));
	    // System.out.println("#####ColorizerColorScalePanel spinner value["+i+"] ="+getSpinnerValue(i));
	    CSJSpinner sp = new CSJSpinner(lower, colorButtons[i], getSpinnerValue(i)); // , getSpinnerValue(i-1), getSpinnerValue(i+1));
	    spinners[i - 1] = sp;
            buttonsBox.add(colorButtons[i]);
            buttonsBox.add(Box.createRigidArea(new Dimension(120,2)));

            spinnersBox.add(sp);
            spinnersBox.add(Box.createRigidArea(new Dimension(120,2)));
	    /*bPG.addComponent(colorButtons[i], GroupLayout.DEFAULT_SIZE, 80, Short.MAX_VALUE);
	    bSG.addComponent(colorButtons[i], GroupLayout.PREFERRED_SIZE, 13, GroupLayout.PREFERRED_SIZE);
	    bSG.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED);

	    spPG.addComponent(sp, GroupLayout.PREFERRED_SIZE, 80, 80);
	    spSG.addGap(5, 5, 5);
	    spSG.addComponent(sp, GroupLayout.PREFERRED_SIZE, 13, GroupLayout.PREFERRED_SIZE);*/

	    lower = colorButtons[i];
	}
        add(buttonsBox);
        add(spinnersBox);
	setButtonsEnabled(!shadedCheckBox.isSelected());
    }

    public void setColorNumberModel(SpinnerNumberModel spm) {
	colorNumber.setModel(spm);
	int nb = spm.getNumber().intValue();
	// System.out.println("### color number = " + nb);
	colorScale.setColorNumber(nb);

	refreshPanel();
    }
    
    protected String getColorButtonText(int i) {
	return i < colorButtons.length && colorizer != null ? colorizer.getColorText(i) : "";
    }

    protected double getSpinnerValue(int i) {
        if (colorizer != null && colorizer.getClassification() != null && i < colorButtons.length)
	    return colorizer.getClassification().getNthBreak(i);
	return 0;
    }
    
    private void setButtonsEnabled(boolean enabled) {
	for(int i = 1; i < colorButtons.length -1 ; i++){
	    colorButtons[i].setEnabled(enabled);
	}
    }

    /**
     * This method is run when the colorMapCombo changes
     */
    public void itemStateChanged(ItemEvent ie) {
	if (ie.getSource() == colorMapCombo) {
	    try {
		ColorScale c = (ColorScale) colorMapCombo.getSelectedItem();
		// 2 ways of thinking the change: 
		// colorNumber.setValue(c.getColorNumber());
		// or:
		c.setColorNumber(((Integer) colorNumber.getValue()).intValue());
		colorScale.copy(c);
		shadedCheckBox.setSelected(false);
		refreshPanel();
	    } catch (Exception e) { }
	}
    }
    
    /**
     * Called when colorNumber changes.
     * Interacts with internal last color of the shadedColorScale.
     * Must change colorNumber in the actually used colorScale and in the
     * selected combo box colorScale.
     *
     * BUG ? Once, using shaded on and standardization (i.e. spinner model with
     * odd numbers (adding two each time), i got null pointer in doShade method.
     */
    public void stateChanged(ChangeEvent e) {
	if(e.getSource() == colorNumber) { // We set the number of colors
	    int n = ((Integer) colorNumber.getValue()).intValue();
	    setColorNumber(n);
	    return;
	}
    }

    private void setColorNumber(int n) {
	int m = colorScale.getColorNumber();
	// System.out.println("################  " +n);
	colorScale.setColorNumber(n);
	// In shaded mode, we automagically propagate the last color
	// Does not work for the moment
	if (shadedCheckBox.isSelected()) {
	    if ((n == m+1))
		colorScale.setColor(n, colorScale.getColor(m));
	    else
		colorScale.setColor(n-1, colorScale.getColor(n));
	    colorScale.doShade();
	}
	if (colorizer != null && colorizer.getClassification() != null)
	    colorizer.getClassification().setNumberOfClasses(n);
	// Apply the same change to the selected item
	// do not use copy here !
	ColorScale selected = (ColorScale) colorMapCombo.getSelectedItem();
	selected.setColorNumber(n);
	refreshPanel();
    }

    //MODE CUSTOM
    private CSJSpinner[] spinners;
 
     //current custom classif
    private CustomizedClassification customClassif;

    //INCORRECT ISSUE (?!) : hack to correctly repaint the GUI >> *CLIPPING* !! A java.lang.IllegalStateException occurs sometimes, 'cause some components are not attached to groups ?\ 
    public void classificationChanged(Object source) {
	// System.out.println("########## ColorizerColorScalePanel.classifChanged");
	setVisible(false); invalidate();
	removeAll();
	// reset ColorScale color number
	if (colorizer.getClassification() != null)
	    colorizer.getColorScale().setColorNumber(colorizer.getClassification().getNumberOfClasses());
	createOptionPanel(); validate();
	setVisible(true);
    }

    private class CSJSpinner extends JSpinner implements ChangeListener {
	private JButton upper;
	private JButton lower;
	public CSJSpinner (CSJButton lower, CSJButton upper, double value) { // , double min, double max) {
	    super(new SpinnerNumberModel((value < 0) ? 0 : value, 0, Double.MAX_VALUE, 1));
	    // super(new SpinnerNumberModel(colorScale.getColorNumber(), 1, 10, 1));
            setPreferredSize(new Dimension(70, 13));
	    setMinimumSize(new Dimension(70, 13));
            setMaximumSize(new Dimension(70, 13));
            setFont(new Font("Arial", Font.PLAIN, 8));
	    this.upper = upper;
	    this.lower = lower;
	    addChangeListener(this);
	}

	//Enables CUSTOM CLASSIFICATION MODE -- change breaks in live
	public void stateChanged(ChangeEvent e) {
	    double value = ((Double) this.getValue()).doubleValue(); 

	    if (colorizer.getClassification() != null
		&& (! (colorizer.getClassification() instanceof CustomizedClassification)
		    || colorizer.getClassification() != customClassif) ) {
		customClassif.reInit(colorizer.getClassification());
		colorizer.setClassification(customClassif);
		customClassif.setNthBreak(((CSJButton) upper).getPosition(), value);
		return;
	    }
	    if (colorizer.getClassification() == null) {
		colorizer.setClassification(customClassif);
	    }
 	    customClassif.setNthBreak(((CSJButton) upper).getPosition(), value);
	    // System.out.println("####ColorizerColorScalePanel: " + colorizer.getClassification());
	    return;
	}
    }

    /**
     * Customized JButton to choose color.
     * When customizing color, the selected color scale automaticaly switches
     * to custom to avoid changing predefined color scales. We must use copy
     * on colorScale too.
     */
    private class CSJButton extends JButton implements ActionListener {
	private int position;
	public CSJButton (String s, int position) {
	    super(s);
	    this.position = position;
	    setPreferredSize(new Dimension(120,13));
            setMinimumSize(new Dimension(120,13));
            setMaximumSize(new Dimension(120,13));
	    setFocusable(false);
            setFont(new Font("Arial", Font.PLAIN, 8));
	    setUI(new BasicButtonUI());
	    addActionListener(this);
            
	}

	public void actionPerformed(ActionEvent event) {

	    Color newButtonColor = OMColorChooser.showDialog(this, i18n.get(ColorizerColorScalePanel.class, "colorChooser", "Choose a color..."), getBackground());
	    if(newButtonColor != null){
		setBackground(newButtonColor);
		ColorScale selected = (ColorScale) colorMapCombo.getSelectedItem();
		if (selected != customColorScale) {
		    colorMapCombo.setSelectedItem(customColorScale);
		    customColorScale.copy(selected);
		    customColorScale.setColor(position, newButtonColor);
		    colorScale.copy(customColorScale);
		} else {
		    customColorScale.setColor(position, newButtonColor);
		    colorScale.setColor(position, newButtonColor);
		}
		refreshPanel();	    
	    }
	    // Must redo shade after changing a color.
	    if (shadedCheckBox.isSelected()) {
		colorScale.doShade();
		refreshPanel();
	    }
	}

	public int getPosition() {
	    return position;
	}
    }
}