// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import fr.unicaen.openmap.util.Util;
import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.SwingConstants;
import org.jdesktop.swingx.JXCollapsiblePane;

/**
 * Unrollable Thematic configurationPanel thanks to a JToggleButton
 * 
 * @author jchauvea
 *
 */
public class ThematicConfigurationPanel extends JPanel {

    protected JToggleButton button;
    protected final JXCollapsiblePane configPanel;
    private String name;
    private boolean vertical = true;
    public final static int V_HEIGHT = 170;
    public final static int H_HEIGHT = 25;
    public JXCollapsiblePane.Direction direction;
    private int parentWidth;
    //private int expandWidth = 500;

    public ThematicConfigurationPanel(String name, final JXCollapsiblePane cp) {
        button = new VerticalJButton(name, Util.loadIcon("expand.png"));
        this.name = name;
        direction = cp.getDirection();
        button.setContentAreaFilled(false);
        button.setBorderPainted(false);
        this.configPanel = cp;
        setLayout(new BorderLayout());
        add(configPanel, BorderLayout.CENTER);
        collapse();
        configPanel.setCollapsed(true);
        //listener
        button.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                if (configPanel.isCollapsed()) {
                    if (direction == JXCollapsiblePane.Direction.LEFT) {
                        vertical = false;
                        expand();
                    }
                    configPanel.setCollapsed(false);


                } else {
                    //button.setHorizontalAlignment(SwingConstants.CENTER);
                    if (direction == JXCollapsiblePane.Direction.LEFT) {
                        vertical = true;
                        collapse();
                        //button.setIcon(Util.loadIcon("expand.png"));
                    }
                    configPanel.setCollapsed(true);

                }
            }
        });
    }

    public void setDirection(JXCollapsiblePane.Direction direction){
        this.direction = direction;
        configPanel.setDirection(direction);
        updateUI();
        configPanel.updateUI();
    }

    /*public void parentWidthChanged(int width) {
        parentWidth = width;
        Dimension dim = new Dimension(parentWidth, ThematicConfigurationPanel.this.getHeight());
        ThematicConfigurationPanel.this.setSize(dim);
        ThematicConfigurationPanel.this.setMaximumSize(dim);
        ThematicConfigurationPanel.this.setMinimumSize(dim);
        ThematicConfigurationPanel.this.setPreferredSize(dim);

        //repaint();
        //button.repaint();
    }*/

    /*public ThematicConfigurationPanel(String name, final JXCollapsiblePane cp, final JComponent thematicGUI,int expWidth){
    this(name, cp, thematicGUI);
    this.expandWidth = expWidth;
    }*/
    private void collapse() {
        if (direction == JXCollapsiblePane.Direction.LEFT) {
            add(button, BorderLayout.WEST);
            button.setPreferredSize(new Dimension(H_HEIGHT, V_HEIGHT));
            button.setMinimumSize(new Dimension(H_HEIGHT, V_HEIGHT));
            button.setMaximumSize(new Dimension(H_HEIGHT, V_HEIGHT));
        } else {
            add(button, BorderLayout.NORTH);
            int pWidth = (int) configPanel.getPreferredSize().getWidth();
            button.setPreferredSize(new Dimension(pWidth, H_HEIGHT));
            button.setMinimumSize(new Dimension(pWidth, H_HEIGHT));
            button.setMaximumSize(new Dimension(pWidth, H_HEIGHT));
        }
        //setMaximumSize(new Dimension(H_HEIGHT, V_HEIGHT));
        //setPreferredSize(new Dimension(H_HEIGHT, V_HEIGHT));
        //setMaximumSize(new Dimension(H_HEIGHT, V_HEIGHT));
        //button.setText(verticalName);
    }

    private void expand() {
        button.setPreferredSize(new Dimension(configPanel.getWidth(), H_HEIGHT));
        button.setMinimumSize(new Dimension(configPanel.getWidth(), H_HEIGHT));
        button.setMaximumSize(new Dimension(configPanel.getWidth(), H_HEIGHT));
        //setMaximumSize(new Dimension(expandWidth, V_HEIGHT));
        //setPreferredSize(new Dimension(expandWidth, V_HEIGHT));
        add(button, BorderLayout.NORTH);
        button.setText(name);
    }

    /*private void verticalizeButtonText() {
    verticalName = "<html>";
    for (int i = 0; i < name.length(); i++) {
    verticalName += name.charAt(i) + "<br/>";
    }
    verticalName += "</html>";
    }*/
    public void setButtonVisibility(boolean b) {
        button.setVisible(b);
    }

    public void setConfigPanelVisibility(boolean b) {
        configPanel.setVisible(b);
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        //if (!vertical) {
        Graphics2D g2d = (Graphics2D) g;
        g2d.setColor(/*new Color(0xDD, 0xDD, 0xDD, 0xFF)*/Color.BLACK);
        //g2d.fillRoundRect(0, 0, getWidth(), getHeight(), 12, 12);
        g2d.drawRoundRect(0, 0, getWidth() - 1, getHeight() - 1, 12, 12);
        //}
        //System.out.println("paint parent width "+parentWidth);

    }

    class VerticalJButton extends JToggleButton {

        public VerticalJButton(String name, ImageIcon icon) {
            super(name);
        }

        @Override
        public void paint(Graphics g) {
            setBackground(getParent().getBackground());
            setBorder(null);
            super.paint(g);
            Graphics2D g2d = (Graphics2D) g;

            // FontRenderContext frc = new FontRenderContext(null, false, false);
            //Rectangle2D r = getFont().getStringBounds(getText(), frc);
            if (vertical && direction == JXCollapsiblePane.Direction.LEFT) {
                g2d.setColor(new Color(0xDD, 0xDD, 0xDD, 0xFF));
                g2d.fillRoundRect(0, 0, getWidth(), getHeight(), 12, 12);
                g2d.setColor(Color.BLACK);
                g2d.drawRoundRect(0, 0, getWidth() - 1, getHeight() - 1, 12, 12);
                button.setText("");
                super.paintComponent(g);
                g2d.translate(0, getHeight());
                g2d.rotate(-Math.PI / 2);
                g2d.drawString(name, 5, 15);
                Image image = Util.loadIcon("expand.png").getImage();
                g.drawImage(image, V_HEIGHT - 20, 5, this);
            } else {
                g2d.setColor(new Color(0xDD, 0xDD, 0xDD, 0xFF));
                g2d.fillRoundRect(0, 0, getWidth(), getHeight(), 12, 12);
                button.setText(name);
                super.paintComponent(g);
                Image image = Util.loadIcon("collapse.png").getImage();
                g.drawImage(image, 5, 5, this);
            }
            g2d.dispose();

        }
        //@Override
        // public void paintComponent(Graphics g) {
//            Graphics2D g2d = (Graphics2D) g;
//            //button.setBackground(Color.red);
//            if (vertical) {
//                button.setText("");
//                super.paintComponent(g);
//                g2d.translate(0, getHeight());
//                g2d.rotate(-Math.PI / 2);
//
//                //g2d.rotate(Math.toRadians(-90.0)); // use rotate(double theta, double x, double y) if relevant
//                g2d.drawString(name, 0, (getHeight() - getStringHeight()) / 2);
//            } else {
//                button.setText(name);
//                super.paintComponent(g);
//            }
//            g2d.setColor(new Color(128, 0, 128));
//
//            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
//                    RenderingHints.VALUE_ANTIALIAS_ON);
//            g2d.setStroke(new BasicStroke(1.2f));
//            g2d.draw(new RoundRectangle2D.Double(1, 1, (getWidth() - 3),
//                    (getHeight() - 3), 12, 8));
//            g2d.setStroke(new BasicStroke(1.5f));
//            g2d.drawLine(4, getHeight() - 3, getWidth() - 4, getHeight() - 3);
//
//            g2d.dispose();
        // }
    }
}
