// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import java.util.Properties;
import java.util.regex.Pattern;

public class OMResizeableColorScaleLegend extends OMResizeableFrame {

    public  int rectWidth = 40, rectHeight = 15;
    private ColorScale colorScale;
    private String[] colorNames;
    private String title;
    //private OMResizeableText omTitle = null;

    public OMResizeableColorScaleLegend(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, ColorScale colorScale, String[] colorNames, String title, String nameForGUI) {
        super(r, ml, xAnchor, yAnchor, 0, 0, nameForGUI, false); // we'll set width and height at the end of the ctor.
        init(colorScale, colorNames, title);
    }

    protected void init() {
        init(colorScale, colorNames, title);
    }

    public void init(ColorScale colorScale, String[] colorNames, String title/*, boolean extend*/) {

        clear();
//	omPolys.clear();
//	omGroups.clear();

        this.colorScale = colorScale;
        this.colorNames = colorNames;
        this.title = title;

        //OMResizeableText[] usualNames = new OMResizeableText[colorNames.length];
        Pattern tagHTML = Pattern.compile("<[^>]*>");
        int x0 = getXAnchor() + (int) (0.5 * rectHeight), y0 = getYAnchor() + (int) (0.5 * rectHeight);

        int maxTextWidth = 0;

        OMResizeableText omTitle = new OMResizeableText(refreshable, ml,
                getXAnchor(), y0 + (int) (0.5 * rectHeight),
                title, OMText.JUSTIFY_CENTER, "title");
        omTitle.setFontStyle(java.awt.Font.BOLD);
        //omTitle.setFontStyle(java.awt.Font.BOLD);
        y0 += omTitle.getHeight();
        this.add(omTitle);
//	    customizedTexts.setTitle(omTitle);
        maxTextWidth = omTitle.getWidth();


        for (int i = 0; i < colorNames.length; i++, y0 += 1.5 * rectHeight) {
            // System.out.println("OMResizeableColorScaleLegend x0="+x0+" y0="+y0+" "+rectWidth+"x"+rectHeight);

            OMResizeableGroup group =
                    new OMResizeableGroup(refreshable, ml,
                    x0 - OMControlPointRadius, y0 - OMControlPointRadius, 0, 0, "Elem " + i);
            if (colorScale != null) {
                OMResizeableRect rect =
                        new OMResizeableRect(refreshable, ml,
                        x0, y0, rectWidth, rectHeight, colorScale.getColor(i));
                group.add(rect);
//		omPolys.add(rect);
//                add(group);
            }

            OMResizeableText text = new OMResizeableText(refreshable, ml,
                    x0 + rectWidth + rectHeight, y0 + rectHeight,
                    tagHTML.matcher(colorNames[i]).replaceAll(""), OMText.JUSTIFY_LEFT,
                    String.format("%s", colorNames[i]), true, true);
            // text.addKeyListener(i, colorNames);
            if (maxTextWidth < text.getWidth()) {
                maxTextWidth = text.getWidth();

            }
            group.add(text);
            //usualNames[i] = text;
            //omGroups.add(group);
            this.add(group);
        }
        //customizedTexts.setTexts(usualNames);
        //	OMResizeableText omTitle = null;
        if (title != null) { // center the title
            // omTitle.translate(...);
        }
        int newWidth = (int) (0.5 * rectHeight) + rectWidth + rectHeight + maxTextWidth;
        int newHeight = (int) ((0.5 + colorNames.length * 1.5) * rectHeight) + (title != null ? (int) (rectHeight * 1.2) : 0);
        //if(extend)
        this.extendBorder(newWidth - getWidth(), newHeight - getHeight());
        if (omTitle != null) {
            omTitle.translate(newWidth / 2, 0);
        }
    }

    public void extend(int dw, int dh) {
//        this.extendBorder(dw, dh);
        // % of growth ?
        double pw = 1;
        if (getWidth() != 0) {
            pw = (double) (getWidth() + dw) / getWidth();
        }
        double ph = 1;
        if (getHeight() != 0) {
            ph = (double) (getHeight() + dh) / getHeight();
        }

        if (getWidth() == 0 || getHeight() == 0) {
            return;
        }
//        for(OMResizeableGraphic r: resizeableGraphics){
//            if(r instanceof OMResizeableGroup)
//                ((OMResizeableGroup)r).forwardExtend(dw, dh);
//        }
        rectWidth *= pw;
        if (rectWidth <= 0) {
            rectWidth = 1; // can't shrink it to much!
        }
        rectHeight *= ph;
        if (rectHeight <= 0) {
            rectHeight = 1;
        }

        // System.out.println("OMResizeableColorScaleLegend.extend width="+getWidth()+" dw="+dw+" pw="+pw+" rectWidth="+rectWidth);
        // System.out.println("OMResizeableColorScaleLegend.extend height="+getHeight()+" dh="+dh+" ph="+ph+" rectHeight="+rectHeight);

        // save text properties except their position
        //Properties props = new Properties();
        //customizedTexts.removeAnchorFromProperties("", customizedTexts.getProperties("", props));
        clear();
        init();
        // restore text properties
        //customizedTexts.setProperties("", props);

        refreshGUI();
    }
    //================================================================================
    // PROPERTIES
    //================================================================================
    public static final String SCALE_LEGEND_PROPERTY_NAME = "scaleLegend";
    public static final String RECTANGLES_WIDTH_PROPERTY_NAME = "rectangles.width";
    public static final String RECTANGLES_HEIGHT_PROPERTY_NAME = "rectangles.height";

//    protected CustomizedLegendText customizedTexts = new CustomizedLegendText();
//    protected ArrayList<OMResizeableRect> omPolys = new ArrayList<OMResizeableRect>();
//    protected ArrayList<OMResizeableGroup> omGroups = new ArrayList<OMResizeableGroup>();
    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
	String tmpPrefix = PropUtils.getScopedPropertyPrefix(prefix) + SCALE_LEGEND_PROPERTY_NAME + ".";
        rectWidth = PropUtils.intFromProperties(properties, tmpPrefix + RECTANGLES_WIDTH_PROPERTY_NAME, rectWidth);
        rectHeight = PropUtils.intFromProperties(properties, tmpPrefix + RECTANGLES_HEIGHT_PROPERTY_NAME, rectHeight);
        init();
        super.setProperties(prefix, properties);
    }
//
    public Properties getProperties(String prefix, Properties properties) {
	properties = super.getProperties(prefix, properties);
	prefix = PropUtils.getScopedPropertyPrefix(prefix) + SCALE_LEGEND_PROPERTY_NAME + ".";
	properties.put(prefix + RECTANGLES_WIDTH_PROPERTY_NAME, Integer.toString(rectWidth));
        properties.put(prefix + RECTANGLES_HEIGHT_PROPERTY_NAME, Integer.toString(rectHeight));
	return properties;
    }
}
