// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.Environment;
import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.util.Properties;
import java.awt.Color;
import java.awt.Dimension;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class OMResizeableCompass extends OMResizeableGraphic{

    private OMPoly omWhitePoly,  omBlackPoly;
    private OMPoly northPoly;
    private int xOffset,  yOffset;
    private double radius;

    OMGraphicList graphics = new OMGraphicList();

    public OMResizeableCompass(Refreshable refreshable, MouseControlPointsListener ml, int px1, int py1, double radius) {
        super(refreshable, ml, "Compass");
        xOffset = px1;
        yOffset = py1;
        this.radius = radius;
	add(graphics);
	addOMControlPoint(new OMControlNPoint(px1, py1,
					      OMResizeableGraphic.OMControlPointRadius,
					      Color.yellow, Color.red, refreshable.getMapBean()));
	
    }
    

    private final static int[] whiteXs = {0, 0, 3, -3, 0, 0, -3, -14, 14, 3, 0};
    private final static int[] whiteYs = {0, 14, 3, -3, -14, 0, 3, 0, 0, -3, 0};
    private final static int[] blackXs = {0, 0, -3, 3, 0, 0, -3, -14, 14, 3, 0};
    private final static int[] blackYs = {0, 14, 3, -3, -14, 0, -3, 0, 0, 3, 0};
    private final static int[] northXs = {-2, -5, -5, -2, -2, 3, 3, 5, 5, 3, 3, -2, -2};
    private final static int[] northYs = {-12, -12, -20, -20, -19, -18, -20, -20, -12, -12, -15, -17, -12};

    private OMCompassElementDefinition[] compassElemDef0 = {
	new OMCompassElementDefinition(Color.WHITE, whiteXs, whiteYs),
	new OMCompassElementDefinition(Color.BLACK, blackXs, blackYs),
	new OMCompassElementDefinition(Color.BLACK, northXs, northYs) };
    OMCompassDefinition compassDef0 = 
	new OMCompassDefinition(14, compassElemDef0);


// N1.txt
    private final static int[] X1_0 = {-111, 112, 112, 11, 11, -111, };
    private final static int[] Y1_0 = {39, -265, 500, 500, 39, 39, };
    private final static int[] X1_1 = {-77, -77, -21, 61, 61, 113, 113, 56, -26, -26, -77, -77, };
    private final static int[] Y1_1 = {-290, -499, -499, -361, -499, -499, -290, -290, -427, -290, -290, -290, };
    private OMCompassElementDefinition[] compassElemDef1 = {
	new OMCompassElementDefinition(Color.BLACK, X1_0, Y1_0),
	new OMCompassElementDefinition(Color.BLACK, X1_1, Y1_1),
    };
    OMCompassDefinition compassDef1 = new OMCompassDefinition(500, compassElemDef1);
// N2.txt
    private final static int[] X2_0 = {-112, -112, -55, 27, 27, 78, 78, 22, -60, -60, -112, };
    private final static int[] Y2_0 = {-290, -499, -499, -361, -499, -499, -290, -290, -427, -290, -290, };
    private final static int[] X2_1 = {113, -10, -10, -112, -112, 113, };
    private final static int[] Y2_1 = {40, 40, 500, 500, -265, 40, };
    private OMCompassElementDefinition[] compassElemDef2 = {
	new OMCompassElementDefinition(Color.BLACK, X2_0, Y2_0),
	new OMCompassElementDefinition(Color.BLACK, X2_1, Y2_1),
    };
    OMCompassDefinition compassDef2 = new OMCompassDefinition(500, compassElemDef2);
// NT.txt
    private final static int[] X3_0 = {1, -364, 1, 1, };
    private final static int[] Y3_0 = {-192, 481, 397, -192, };
    private final static int[] X3_1 = {1, 390, 0, -389, 1, };
    private final static int[] Y3_1 = {-218, 500, 410, 500, -218, };
    private final static int[] X3_2 = {97, 97, 97, 97, 97, 97, 97, 97, 98, 97, 96, 94, 92, 89, 84, 76, 69, 63, 57, 51, 46, -33, -35, -39, -44, -57, -66, -71, -72, -72, -72, -71, -68, -64, -57, -46, -38, -38, -38, -38, -37, -37, -37, -37, -37, -37, -38, -38, -39, -43, -47, -52, -68, -78, -82, -87, -96, -111, -115, -119, -121, -122, -115, -100, -93, -92, -92, -92, -92, -92, -92, -93, -95, -97, -100, -103, -106, -110, -113, -117, -120, -124, -130, -130, -130, -130, -130, -131, -131, -131, -130, -130, -129, -129, -127, -114, -59, -48, -33, -15, -10, -6, -4, 77, 78, 78, 78, 76, 73, 69, 62, 51, 44, 45, 51, 60, 70, 79, 86, 88, 91, 97, 105, 113, 120, 126, 127, 120, 118, 117, 116, 109, 105, 102, 100, 99, 98, 97, 97, 97, };
    private final static int[] Y3_2 = {-401, -338, -273, -264, -258, -254, -250, -247, -244, -243, -241, -241, -240, -239, -239, -247, -255, -262, -269, -276, -284, -386, -390, -394, -400, -417, -428, -434, -414, -329, -297, -278, -271, -264, -259, -258, -257, -256, -255, -254, -253, -252, -251, -250, -249, -249, -248, -246, -245, -245, -245, -246, -246, -247, -247, -247, -246, -245, -245, -245, -245, -257, -258, -258, -276, -311, -324, -335, -342, -433, -444, -454, -462, -469, -475, -479, -481, -483, -484, -485, -486, -487, -488, -488, -489, -490, -491, -492, -492, -493, -494, -496, -497, -498, -499, -499, -499, -481, -461, -439, -433, -429, -426, -325, -334, -414, -445, -465, -473, -480, -485, -487, -488, -499, -499, -499, -498, -498, -498, -498, -498, -498, -498, -499, -499, -499, -488, -486, -486, -485, -485, -483, -480, -477, -474, -466, -453, -440, -423, -401, };
    private OMCompassElementDefinition[] compassElemDef3 = {
	new OMCompassElementDefinition(Color.WHITE, X3_0, Y3_0),
	new OMCompassElementDefinition(Color.BLACK, X3_1, Y3_1),
	new OMCompassElementDefinition(Color.BLACK, X3_2, Y3_2),
    };
    OMCompassDefinition compassDef3 = new OMCompassDefinition(500, compassElemDef3);
// Nfleche.txt
    private final static int[] X4_0 = {-2, -2, -42, -2, };
    private final static int[] Y4_0 = {445, 194, 476, 445, };
    private final static int[] X4_1 = {33, 2, 2, 33, };
    private final static int[] Y4_1 = {-17, -246, -30, -17, };
    private final static int[] X4_2 = {76, 69, 65, 63, 60, 59, 59, 59, 31, -38, -38, -38, -37, -35, -33, -29, -23, -20, -20, -75, -75, -69, -64, -62, -59, -58, -58, -58, -58, -59, -62, -64, -69, -75, -75, -12, 39, 39, 39, 38, 35, 33, 29, 23, 21, 21, 76, 76, 76, };
    private final static int[] Y4_2 = {-480, -480, -478, -475, -472, -464, -452, -344, -344, -462, -388, -376, -369, -366, -363, -361, -361, -361, -344, -344, -361, -361, -363, -367, -370, -378, -390, -452, -464, -471, -475, -478, -480, -480, -499, -499, -411, -452, -463, -471, -474, -478, -480, -480, -480, -499, -499, -480, -480, };
    private final static int[] X4_3 = {56, 0, -55, -0, -15, -25, -33, -40, -44, -44, -44, -40, -33, -27, -17, -4, -4, -44, 0, 45, 5, 5, 19, 29, 36, 43, 47, 47, 47, 43, 35, 27, 16, 1, 56, };
    private final static int[] Y4_3 = {500, 457, 500, 115, 112, 107, 99, 92, 82, 70, 59, 49, 41, 33, 27, 23, -15, 1, -331, 1, -15, 23, 27, 33, 41, 48, 58, 70, 82, 92, 99, 107, 112, 115, 500, };
    private OMCompassElementDefinition[] compassElemDef4 = {
	new OMCompassElementDefinition(Color.WHITE, X4_0, Y4_0),
	new OMCompassElementDefinition(Color.WHITE, X4_1, Y4_1),
	new OMCompassElementDefinition(Color.BLACK, X4_2, Y4_2),
	new OMCompassElementDefinition(Color.BLACK, X4_3, Y4_3),
    };
    OMCompassDefinition compassDef4 = new OMCompassDefinition(500, compassElemDef4);
// No.txt
    private final static int[] X5_0 = {135, 104, 104, 56, -60, -60, -23, -23, -130, -130, -99, -99, -130, -130, -46, 66, 66, 29, 29, 135, 135, 135, };
    private final static int[] Y5_0 = {-40, -40, 148, 148, -20, 122, 122, 148, 148, 122, 122, -40, -40, -66, -66, 95, -40, -40, -66, -66, -40, -40, };
    private final static int[] X5_1 = {187, 187, 169, 132, 96, 51, -0, -53, -98, -134, -171, -189, -189, -189, -171, -134, -98, -53, -0, 51, 96, 132, 169, 187, 187, };
    private final static int[] Y5_1 = {35, -15, -59, -94, -130, -148, -148, -148, -130, -95, -59, -16, 35, 87, 131, 167, 203, 221, 221, 221, 203, 167, 131, 87, 35, };
    private final static int[] X5_2 = {34, 34, -33, -33, -99, -151, -189, -226, -245, -245, -245, -225, -186, -147, -96, -32, -33, -223, -259, 0, 260, 223, 34, 34, 97, 147, 185, 223, 243, 243, 243, 224, 187, 150, 99, 34, };
    private final static int[] Y5_2 = {273, 500, 500, 273, 260, 232, 190, 148, 96, 35, -25, -78, -121, -165, -191, -200, -383, -269, -305, -499, -305, -269, -383, -200, -190, -163, -120, -76, -24, 35, 95, 146, 188, 231, 259, 273, };
    private OMCompassElementDefinition[] compassElemDef5 = {
	new OMCompassElementDefinition(Color.BLACK, X5_0, Y5_0),
	new OMCompassElementDefinition(Color.WHITE, X5_1, Y5_1),
	new OMCompassElementDefinition(Color.BLACK, X5_2, Y5_2),
    };
    OMCompassDefinition compassDef5 = new OMCompassDefinition(500, compassElemDef5);
// boussole4lettres.txt
    private final static int[] X6_0 = {91, 33, 33, 91, };
    private final static int[] Y6_0 = {-68, -280, -11, -68, };
    private final static int[] X6_1 = {314, 36, 93, 314, };
    private final static int[] Y6_1 = {-6, -6, 53, -6, };
    private final static int[] X6_2 = {31, 31, -27, 31, };
    private final static int[] Y6_2 = {268, -4, 55, 268, };
    private final static int[] X6_3 = {28, -29, -241, 28, };
    private final static int[] Y6_3 = {-9, -66, -9, -9, };
    private final static int[] X6_4 = {-300, -306, -311, -313, -314, -314, -314, -355, -360, -394, -433, -439, -481, -483, -485, -487, -489, -492, -496, -497, -436, -436, -439, -442, -444, -446, -447, -448, -448, -448, -448, -447, -447, -446, -446, -423, -399, -403, -404, -405, -406, -406, -407, -409, -410, -412, -417, -419, -421, -422, -422, -357, -357, -361, -364, -366, -367, -369, -370, -371, -369, -346, -326, -325, -325, -324, -324, -323, -323, -323, -324, -326, -328, -332, -337, -337, -299, -300, };
    private final static int[] Y6_4 = {-77, -75, -70, -61, -60, -59, -58, 62, 62, -35, 62, 62, -62, -68, -71, -73, -75, -76, -77, -82, -82, -77, -77, -77, -76, -74, -73, -70, -66, -64, -62, -60, -59, -57, -55, 12, -49, -64, -66, -68, -70, -71, -73, -74, -75, -75, -76, -76, -76, -76, -82, -82, -76, -76, -76, -76, -75, -73, -71, -69, -61, 8, -54, -55, -57, -59, -61, -64, -66, -70, -73, -74, -76, -76, -76, -82, -82, -77, };
    private final static int[] X6_5 = {355, 100, 32, -37, -292, -37, 32, 100, 355, };
    private final static int[] Y6_5 = {-6, 61, 316, 61, -6, -74, -329, -74, -6, };
    private final static int[] X6_6 = {103, 96, 91, 89, 87, 86, 86, 86, 80, -10, -10, -10, -9, -6, -3, 0, 7, 7, -38, -37, -30, -25, -23, -20, -19, -19, -19, -26, -27, -28, -29, -31, -34, -37, -38, 4, 76, 76, 76, 75, 73, 70, 65, 57, 57, 103, 103, };
    private final static int[] Y6_6 = {-494, -493, -492, -490, -487, -482, -475, -352, -352, -466, -380, -373, -368, -365, -363, -361, -361, -356, -356, -361, -362, -363, -365, -368, -373, -380, -478, -486, -486, -487, -488, -491, -493, -494, -499, -499, -409, -475, -482, -487, -489, -492, -493, -494, -499, -499, -494, };
    private final static int[] X6_7 = {50, 61, 69, 74, 78, 80, 80, 80, 76, 67, 58, 46, 31, 24, 17, 10, 2, -1, -2, -4, -5, -6, -7, -8, -9, -14, -14, -8, -5, -0, 6, 12, 21, 32, 40, 46, 51, 55, 57, 57, 57, 56, 54, 51, 47, 41, 22, 9, 0, -4, -9, -12, -12, -12, -8, -0, 6, 16, 28, 33, 43, 58, 60, 62, 63, 64, 66, 67, 68, 68, 69, 75, 75, 71, 66, 60, 54, 47, 40, 31, 25, 19, 16, 12, 10, 10, 10, 11, 14, 16, 22, 30, 50, };
    private final static int[] Y6_7 = {411, 417, 423, 430, 437, 445, 456, 469, 480, 488, 496, 500, 500, 500, 499, 496, 494, 493, 493, 493, 493, 494, 495, 497, 500, 500, 449, 449, 464, 475, 482, 489, 493, 493, 493, 491, 486, 482, 476, 469, 464, 460, 456, 453, 450, 447, 436, 430, 423, 417, 410, 402, 392, 380, 370, 362, 355, 351, 351, 351, 353, 357, 357, 358, 358, 358, 357, 356, 355, 353, 351, 351, 396, 396, 384, 374, 368, 361, 358, 358, 358, 360, 363, 367, 371, 377, 383, 388, 391, 394, 397, 401, 411, };
    private final static int[] X6_8 = {498, 488, 369, 369, 376, 381, 384, 387, 388, 388, 388, 388, 387, 384, 382, 377, 370, 370, 487, 487, 483, 482, 477, 470, 463, 452, 437, 431, 427, 425, 422, 421, 421, 421, 424, 433, 439, 444, 448, 452, 454, 459, 459, 454, 453, 450, 445, 441, 434, 424, 421, 421, 421, 422, 424, 426, 430, 436, 442, 454, 464, 473, 481, 487, 491, 498, 498, };
    private final static int[] Y6_8 = {15, 62, 62, 55, 55, 53, 50, 48, 43, 36, -55, -65, -71, -73, -76, -77, -77, -82, -82, -41, -41, -54, -63, -68, -74, -76, -76, -76, -76, -74, -72, -68, -62, -16, -16, -16, -19, -23, -28, -37, -48, -48, 22, 22, 10, 2, -2, -6, -9, -9, -9, 39, 44, 47, 49, 51, 52, 52, 52, 52, 49, 43, 36, 27, 15, 15, 15, };
    private OMCompassElementDefinition[] compassElemDef6 = {
	new OMCompassElementDefinition(Color.WHITE, X6_0, Y6_0),
	new OMCompassElementDefinition(Color.WHITE, X6_1, Y6_1),
	new OMCompassElementDefinition(Color.WHITE, X6_2, Y6_2),
	new OMCompassElementDefinition(Color.WHITE, X6_3, Y6_3),
	new OMCompassElementDefinition(Color.BLACK, X6_4, Y6_4),
	new OMCompassElementDefinition(Color.BLACK, X6_5, Y6_5),
	new OMCompassElementDefinition(Color.BLACK, X6_6, Y6_6),
	new OMCompassElementDefinition(Color.BLACK, X6_7, Y6_7),
	new OMCompassElementDefinition(Color.BLACK, X6_8, Y6_8),
    };
    OMCompassDefinition compassDef6 = new OMCompassDefinition(500, compassElemDef6);
// boussoleAvecRond.txt
    private final static int[] X7_0 = {113, 110, 100, 82, 65, 42, 13, 13, -83, -82, -73, -56, -40, -16, 14, 15, 113, 113, };
    private final static int[] Y7_0 = {-4, -34, -57, -75, -92, -102, -103, -5, -5, 23, 46, 63, 80, 90, 93, -4, -4, -4, };
    private final static int[] X7_1 = {-127, -130, -132, -134, -135, -136, -136, -297, -127, };
    private final static int[] Y7_1 = {44, 35, 27, 19, 11, 3, -4, -4, 44, };
    private final static int[] X7_2 = {13, 13, -35, -26, -17, -9, -1, 6, 13, };
    private final static int[] Y7_2 = {-156, -317, -147, -150, -152, -153, -155, -156, -156, };
    private final static int[] X7_3 = {64, 55, 47, 38, 30, 21, 13, 13, 64, };
    private final static int[] Y7_3 = {137, 140, 142, 143, 145, 146, 146, 307, 137, };
    private final static int[] X7_4 = {327, 156, 159, 161, 163, 164, 165, 165, 327, 327, };
    private final static int[] Y7_4 = {-4, -54, -45, -37, -28, -20, -12, -4, -4, -4, };
    private final static int[] X7_5 = {127, 127, 116, 94, 71, 45, 13, -17, -43, -65, -87, -98, -98, -98, -87, -65, -44, -17, 13, 45, 72, 94, 116, 127, 127, };
    private final static int[] Y7_5 = {-4, 26, 53, 75, 97, 108, 108, 108, 97, 75, 53, 26, -4, -36, -62, -84, -106, -117, -117, -117, -106, -84, -62, -36, -4, };
    private final static int[] X7_6 = {150, 150, 137, 110, 83, 51, 13, -23, -55, -81, -108, -121, -121, -121, -107, -81, -55, -23, 13, 51, 83, 110, 137, 150, 150, };
    private final static int[] Y7_6 = {-4, -42, -74, -100, -127, -140, -140, -140, -127, -101, -74, -42, -4, 33, 65, 91, 117, 130, 130, 130, 117, 91, 65, 33, -4, };
    private final static int[] X7_7 = {-442, -427, -430, -431, -433, -435, -437, -441, -444, -444, -386, -387, -390, -392, -394, -396, -398, -399, -398, -380, -361, -361, -361, -361, -360, -359, -359, -359, -361, -364, -366, -370, -375, -376, -320, -320, -326, -331, -334, -337, -340, -343, -375, -400, -417, -439, -464, -496, -497, -498, -501, -503, -506, -509, -510, -454, -454, -456, -458, -460, -460, -461, -461, -461, -461, -461, -460, -460, -460, -459, -442, };
    private final static int[] Y7_7 = {19, -27, -39, -43, -47, -50, -53, -55, -56, -72, -72, -56, -55, -55, -53, -52, -51, -50, -44, 19, -35, -36, -37, -38, -42, -45, -47, -49, -52, -53, -55, -56, -56, -72, -72, -55, -54, -52, -49, -46, -41, -34, 59, 59, -0, 59, 59, -42, -46, -49, -51, -53, -55, -55, -72, -72, -56, -56, -56, -56, -55, -54, -52, -50, -48, -45, -42, -41, -40, -39, 19, };
    private final static int[] X7_8 = {352, 153, 143, 132, 118, 105, 90, 72, 13, -42, -64, -82, -96, -109, -119, -125, -323, -125, -115, -104, -91, -77, -62, -43, 13, 70, 89, 106, 120, 134, 145, 153, 352, };
    private final static int[] Y7_8 = {-4, 52, 73, 90, 103, 116, 127, 134, 333, 135, 125, 113, 100, 86, 71, 53, -4, -62, -83, -100, -113, -127, -137, -144, -343, -145, -137, -126, -112, -99, -82, -63, -4, };
    private final static int[] X7_9 = {-52, -52, -46, -42, -40, -38, -37, -37, -37, -37, -38, -40, -42, -46, -52, -52, 2, 48, 48, 48, 48, 47, 46, 45, 44, 42, 41, 39, 37, 35, 33, 33, 81, 81, 75, 71, 69, 67, 66, 66, 66, 42, -18, -18, -18, -17, -15, -13, -9, -3, -3, -52, };
    private final static int[] Y7_9 = {-367, -383, -383, -385, -387, -390, -397, -407, -459, -469, -476, -478, -481, -483, -483, -499, -499, -423, -459, -466, -471, -474, -476, -478, -480, -481, -481, -482, -482, -483, -483, -499, -499, -483, -483, -481, -478, -476, -469, -459, -367, -367, -468, -407, -397, -390, -387, -385, -383, -383, -367, -367, };
    private final static int[] X7_10 = {-28, -43, -29, -22, -14, -6, 1, 10, 20, 27, 32, 37, 42, 44, 44, 44, 30, 1, -26, -40, -40, -40, -36, -26, -17, -5, 8, 13, 18, 23, 28, 33, 39, 42, 54, 65, 51, 45, 39, 32, 25, 18, 10, 3, -1, -5, -9, -11, -11, -11, -1, 18, 21, 24, 26, 43, 56, 63, 70, 73, 73, 73, 68, 57, 47, 34, 18, 12, 7, 1, -3, -9, -14, -16, -28, -28, };
    private final static int[] Y7_10 = {499, 450, 444, 457, 466, 472, 478, 482, 482, 482, 480, 477, 473, 469, 464, 458, 451, 444, 437, 424, 405, 393, 383, 375, 366, 362, 362, 362, 363, 365, 366, 369, 372, 364, 364, 404, 409, 400, 393, 388, 383, 381, 381, 381, 382, 385, 387, 391, 396, 401, 406, 411, 411, 412, 413, 417, 422, 428, 434, 443, 454, 468, 479, 487, 496, 500, 500, 500, 499, 498, 496, 493, 490, 499, 499, 499, };
    private final static int[] X7_11 = {380, 380, 385, 389, 391, 393, 395, 395, 395, 395, 394, 392, 390, 386, 382, 380, 380, 488, 506, 494, 486, 478, 470, 462, 453, 443, 438, 434, 432, 430, 429, 429, 429, 435, 442, 447, 450, 452, 454, 454, 466, 466, 454, 453, 451, 448, 445, 442, 437, 435, 429, 429, 429, 430, 432, 433, 437, 443, 453, 463, 472, 481, 490, 498, 511, 491, 380, 380, };
    private final static int[] Y7_11 = {59, 42, 42, 40, 37, 34, 28, 18, -31, -40, -47, -50, -53, -55, -55, -55, -72, -72, -32, -26, -37, -44, -49, -54, -56, -56, -56, -55, -53, -51, -46, -39, -21, -21, -21, -22, -25, -28, -33, -41, -41, 18, 18, 11, 6, 2, -1, -3, -3, -3, -3, 27, 34, 38, 40, 42, 43, 43, 43, 40, 34, 28, 19, 7, 14, 59, 59, 59, };
    private OMCompassElementDefinition[] compassElemDef7 = {
	new OMCompassElementDefinition(Color.WHITE, X7_0, Y7_0),
	new OMCompassElementDefinition(Color.WHITE, X7_1, Y7_1),
	new OMCompassElementDefinition(Color.WHITE, X7_2, Y7_2),
	new OMCompassElementDefinition(Color.WHITE, X7_3, Y7_3),
	new OMCompassElementDefinition(Color.WHITE, X7_4, Y7_4),
	new OMCompassElementDefinition(Color.BLACK, X7_5, Y7_5),
	new OMCompassElementDefinition(Color.WHITE, X7_6, Y7_6),
	new OMCompassElementDefinition(Color.BLACK, X7_7, Y7_7),
	new OMCompassElementDefinition(Color.BLACK, X7_8, Y7_8),
	new OMCompassElementDefinition(Color.BLACK, X7_9, Y7_9),
	new OMCompassElementDefinition(Color.BLACK, X7_10, Y7_10),
	new OMCompassElementDefinition(Color.BLACK, X7_11, Y7_11),
    };
    OMCompassDefinition compassDef7 = new OMCompassDefinition(500, compassElemDef7);


    OMCompassDefinition[] compassDef = {
	compassDef0, compassDef1, compassDef2, compassDef3, 
	compassDef4, compassDef5, compassDef6, compassDef7,
    };

    protected int getCompassDefIndex() { return compassDefIndex; }
    protected void setCompassDefIndex(int i) {
	if (i < 0) i = 0;
	if (i >= compassDef.length) i = 0; 
	compassDefIndex = i;
	init();
    }

    private int compassDefIndex = 0;
    
    
    private void init() {
	graphics.clear();
	// System.out.println("** Compass type "+ getCompassDefIndex());
	double ratio = radius / compassDef[getCompassDefIndex()].radius;
	for (OMCompassElementDefinition elem : compassDef[getCompassDefIndex()].elems) {
	    OMPoly omPoly = new OMPoly(new int[2*elem.Xs.length]);
	    int[] Xs = omPoly.getXs();
	    int[] Ys = omPoly.getYs();
	    
	    for (int i = 0 ; i < Xs.length; i++) {
		Xs[i] = (int) Math.round(xOffset + elem.Xs[i] * ratio);
		Ys[i] = (int) Math.round(yOffset + elem.Ys[i] * ratio);		
	    }
	    omPoly.setFillPaint(elem.fillColor);
	    graphics.add(omPoly);
	}
    }

    public void translate(int dx, int dy) {
        xOffset += dx;
        yOffset += dy;
        init();

        for (int i = 0; i < omControlPoints.size(); i++) {
            ((OMControlPoint) omControlPoints.getOMGraphicAt(i)).translate(dx, dy);
        }
	// System.out.println("OMCompass.translate("+dx+", "+dy+") --> ("+xOffset+", "+yOffset+")");
    }


    public static final String RADIUS_PROPERTY_NAME = "radius";
    public static final String TYPE_PROPERTY_NAME = "type";

    public void setProperties(String prefix, Properties props) {
	if (props == null)
	    return;
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        setRadius(PropUtils.doubleFromProperties(props, prefix + RADIUS_PROPERTY_NAME, getRadius()));
	setCompassDefIndex(PropUtils.intFromProperties(props, prefix + TYPE_PROPERTY_NAME, getCompassDefIndex()));
        super.setProperties(prefix, props);
	
    }

    public Properties getProperties(String prefix, Properties props) {
	if (props == null)
	    props = new Properties();
	prefix = PropUtils.getScopedPropertyPrefix(prefix);
	props = super.getProperties(prefix, props);
	props.put(prefix + RADIUS_PROPERTY_NAME, Double.toString(getRadius()));
	props.put(prefix + TYPE_PROPERTY_NAME, Integer.toString(getCompassDefIndex()));
	return props;
    }

    public double getRadius() { return radius;}

    public void setRadius(double r) {
	radius = r;
        init();
    }

    public void extend(int dw, int dh) {
    }

    public int getXAnchor() {
        return xOffset;
    }

    public int getYAnchor() {
        return yOffset;
    }

    public int getWidth() {
        return (int) (2 * radius);
    }
    
    public int getHeight() {
        return (int) (2 * radius);
    }

   @Override
   public JComponent getGUI(){
        JPanel pWrap = new JPanel();
        JComponent cp = getToolGUI();
        cp.setBorder(null);
        pWrap.add(cp);
        pWrap.add(new JLabel(Environment.getI18n().get(this,"compass","Compass")));
        pWrap.add(Box.createRigidArea(new Dimension(35,20)));
        pWrap.add(new JLabel(Environment.getI18n().get(this,"type","Type")));
	final JSpinner typeSpinner = new JSpinner(new SpinnerNumberModel(getCompassDefIndex(), 0, compassDef.length-1, 1));
	typeSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    setCompassDefIndex(((Integer)(typeSpinner.getValue())).intValue());
		    // System.out.println("Compass type "+ getCompassDefIndex());
		    refreshGraphics();
		}
	});
	pWrap.add(typeSpinner);

	pWrap.add(new JLabel(Environment.getI18n().get(this,"size","size")));
	final JSpinner radiusSpinner = new JSpinner(new SpinnerNumberModel(getRadius(), 1d, 100d, 1d));
	radiusSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    setRadius(((Double)(radiusSpinner.getValue())).doubleValue());
		    // System.out.println("Compass radius "+ getRadius());
		    refreshGraphics();
		}
	});
	pWrap.add(radiusSpinner);
        ComponentSizer.fixComponentSize(pWrap, new Dimension(300,35));
        return pWrap;
    }

}

class OMCompassElementDefinition  {
    Color fillColor;
    int[] Xs, Ys;
    OMCompassElementDefinition(Color fillColor, int[] Xs, int[] Ys) {
	this.fillColor = fillColor;
	this.Xs = Xs;
	this.Ys = Ys;
    }
}

class OMCompassDefinition {
    int radius;
    OMCompassElementDefinition[] elems;
    OMCompassDefinition(int radius, OMCompassElementDefinition[] elems) {
	this.radius = radius;
	this.elems = elems;
    }


}

