// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.PropUtils;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Paint;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;

/** 
 * OMResizeableDiscLegend construct a legend for proportionnal discs.
 * <pre>
 *         ..------------value
 *       ......
 *     ..........
 *     ..........
 *       ......
 *         ..
 * </pre>
 * The asciiart is copyrighted Yannick Lange ;-)</br>
 * In fact, there is no horizontal line between text ant disc in this version. 
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 */
public class OMResizeableDiscLegend extends OMResizeableFrame {

    private boolean mustBeReseted;
    private boolean reset3m;
    private String columnName;
    private Paint lineColor;
    private Paint fillColor;
    MinMeanMaxValuesAndSizes minMeanMaxValuesAndSizes;
    public static final String mustBeResetedPropertyString = "mustBeReseted";
    public static final String discLegendPropertyString = "discLegend";
    public static final String reset3mPropertyString = "reset3m";

    /** if columnName is null, draw only 1 disc (because they are of fixed size. */
    public OMResizeableDiscLegend(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, Resizer resizer, MinMaxValues minMaxValues, String columnName, Paint lineColor, Paint fillColor, String name) {
        super(r, ml, xAnchor, yAnchor, 0, 0, name, false); // we'll set width and height at the end of the ctor.
        this.columnName = columnName;
        this.fillColor = fillColor;
        this.lineColor = lineColor;
        minMeanMaxValuesAndSizes =
                new MinMeanMaxValuesAndSizes(resizer, minMaxValues);
        init();
    }

    public void init() {
        clear();
        final int offset = 15; // we should compute this offset from the extend of the current font !!!

        int x0 = getXAnchor() + offset;
        int y0 = getYAnchor() + offset;

        // place the title
        OMResizeableText title =
                new OMResizeableText(refreshable, ml, x0, y0 + offset / 2,
                columnName, OMText.JUSTIFY_LEFT, "disc title");
        title.setFontStyle(java.awt.Font.BOLD);
        this.add(title);
        int fontsize = title.getFontSize();

        //minMeanMaxValuesAndSizes.setResizer(resizer);
//        MinMeanMaxValuesAndSizes minMeanMaxValuesAndSizes =
//		    new MinMeanMaxValuesAndSizes(resizer, minMaxValues);

        //0 index is the biggest disc
        int biggestRadius = minMeanMaxValuesAndSizes.getSize(0);/*(int)Math.round(resizer.getSize(minMaxValues.getMax())) / 2;*/
        y0 += 2 * fontsize;

        int discIndex = minMeanMaxValuesAndSizes.getSize() - 1;
        if (columnName == null && discIndex >= 1)
            discIndex = 0;

        // in reverse order so that the smallest disc is on top.
        for (int i = discIndex; i >= 0; i--) {
            int radius = minMeanMaxValuesAndSizes.getSize(i);
            Debug.message("Legend2", "LegendDisc [" + (x0 + biggestRadius) + ", " + (y0 + 2 * biggestRadius - radius) + "] " + radius + " " + biggestRadius + " [" + (x0 + biggestRadius) + ", " + (y0 + 2 * biggestRadius - 2 * radius) + "] " + (minMeanMaxValuesAndSizes.getValue(i)));
            OMResizeableGroup group =
		new OMResizeableGroup(refreshable, ml,
				      x0, (int) Math.round(y0 + 2 * biggestRadius - radius), 0, 0, "Elem " + i);

            OMResizeableDisc disc =
		new OMResizeableDisc(minMeanMaxValuesAndSizes.getTypeValue(i),
				     refreshable, null, minMeanMaxValuesAndSizes, 
				     x0 + biggestRadius, 
				     y0 + 2 * biggestRadius - radius,
				     radius, biggestRadius + offset,
				     lineColor, fillColor);
            OMResizeableText text =
		new OMResizeableText(refreshable, ml, 
				     x0 + 2 * biggestRadius + offset, 
				     y0 + 2 * biggestRadius - 2 * radius,
				     minMeanMaxValuesAndSizes.getValue(i), 
				     OMText.JUSTIFY_LEFT, "label " + i, true, true);
            group.add(disc);
            group.add(text);
            this.add(group);
        }
	// todo ? recenter title @see OMResizeableRegularPolygonLegend
 
        this.extendBorder(x0 + 2 * biggestRadius + 100, y0 + 2 * biggestRadius + offset);
    }

    /** not extensible: we must not change the size of the discs! */
    //    public void extend(int dw, int dh) {
    //    }

    //===============================================================
    // PROPERTIES
    //===============================================================
    public static final String DISCS_TOTAL_PROPERTY_NAME = "discs.total";
    public static final String DISC_PROPERTY_NAME = "discs";
    public static final String GROUPS_PROPERTY_NAME = "groups";
    public static final String FRAME_PROPERTY_NAME = "frame";

    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + discLegendPropertyString + ".";
        minMeanMaxValuesAndSizes.setProperties(prefix, properties);
        mustBeReseted = PropUtils.booleanFromProperties(properties, prefix + mustBeResetedPropertyString, false);
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        //System.out.println("props avant = " + properties);
        if (mustBeReseted) {
            properties = removeAnchorFromProperties(prefix, properties);
            properties = removeSizeFromProperties(prefix, properties);
        }
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + discLegendPropertyString + ".";
        properties = minMeanMaxValuesAndSizes.getProperties(prefix, properties);
        properties.put(prefix + mustBeResetedPropertyString, String.valueOf(mustBeReseted));
        return properties;
    }

    public JComponent getToolGUI() {
        JComponent c = super.getToolGUI();
        //JButton resetLegend = new JButton(Util.loadIcon("undo.png"));
        final JCheckBox resetLegend = new JCheckBox(i18n.get(OMResizeableDiscLegend.class, "align", "auto align"));
        resetLegend.setBorder(BorderFactory.createEmptyBorder(0, 25, 0, 0));
        resetLegend.setSelected(mustBeReseted);
        resetLegend.setBackground(Color.WHITE);
        resetLegend.setOpaque(false);
        //ComponentSizer.fixComponentSize(resetLegend, new Dimension(20,20));
        resetLegend.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                mustBeReseted = resetLegend.isSelected();
            }
        });
        resetLegend.setToolTipText("Reset discs positions according to their new sizes");
        c.add(resetLegend);
        JButton bReset3m = new JButton(Util.loadIcon("undo.png"));
        bReset3m.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                reset3m = true;
            }
        });
        bReset3m.setBackground(Color.gray);
        bReset3m.setOpaque(false);
        ComponentSizer.fixComponentSize(bReset3m, new Dimension(20, 20));
        //c.add(bReset3m);
        return c;
    }
}
