// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import java.awt.Paint;
import java.awt.Point;
import java.util.Properties;
import java.util.ArrayList;

import com.bbn.openmap.omGraphics.OMText;
import com.bbn.openmap.util.PropUtils;


import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.ArrowHead;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.Refreshable;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.MinMaxValues;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.MinMeanMaxValuesAndSizes;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Resizer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Flow;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScale;
import fr.unicaen.openmap.util.Util;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;

public class OMResizeableFlowLegend extends OMResizeableFrame {

    public static int FLOW_SIZE = 120;
    //private MouseControlPointsListener ml;
    //    protected OMResizeableGroup polys;
    Flow flow;
    //private Resizer resizer;
    protected OMResizeableColorScaleLegend scl;
    protected OMResizeableText text;
    private MinMeanMaxValuesAndSizes mmmValuesAndSizes;
    private boolean mustBeReseted;

    public OMResizeableFlowLegend(Flow flow, Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, Resizer resizer, MinMaxValues minMaxValues, String columnName, Paint lineColor, Paint fillColor, ColorScale colorScale, String[] colorNames, String colorDataColumnName, String name) {
	super(r, ml, xAnchor, yAnchor, 0, 0, name, false);
	this.flow = flow;
	this.ml = ml;
	//this.resizer = resizer;
        mmmValuesAndSizes =
	    new MinMeanMaxValuesAndSizes(resizer, minMaxValues);
	init(minMaxValues, columnName, lineColor, fillColor, colorScale, colorNames, colorDataColumnName);
    }
    
    public void init(MinMaxValues minMaxValues,  String columnName, Paint  lineColor, Paint fillColor, ColorScale colorScale, String[] colorNames, String colorDataColumnName) {

	Properties props = new Properties();
	props = getProperties(" ", props);
//	resizeableGraphics.clear();
//	graphics.clear();
        clear();
	final int offset = 15; // we should compute this offset from the extend of the current font !!!
	int x0 = getXAnchor() + offset, y0 = getYAnchor() + offset; 
		
	//int biggestWidth = (int)Math.round(minMaxValues.getMax());

		
	OMResizeableText title =
	    new OMResizeableText(refreshable, ml, getXAnchor(), y0,//+biggestWidth,
				 columnName, OMText.JUSTIFY_CENTER, "polygon title");
	title.setFontStyle(java.awt.Font.BOLD);
	this.add(title);
	int maxTextWidth = title.getWidth();

	y0 += 2*title.getHeight();

	int polygonIndex = mmmValuesAndSizes.getSize() - 1;
	if (columnName == null && polygonIndex > 1)// only 1 polygon if columnName is null
	    polygonIndex = 0; 

	//OMResizeableText[] usualNames = new OMResizeableText[polygonIndex + 1];

	// in reverse order so that the smallest polygon is on top.
        int h = 0;
	for(int i = polygonIndex; i >= 0; i--) {
	    h = mmmValuesAndSizes.getSize(i);
	    y0 += (i == polygonIndex ? 0 : 2*h + offset);
	    OMResizeableGroup group = 
		new OMResizeableGroup(refreshable, ml, 
				      x0-2*OMControlPointRadius, y0-2*OMControlPointRadius, 0, 0, "Elem "+i);
		  
	    OMResizeableFlow poly =
		new OMResizeableFlow(refreshable, ml, flow, new Point(x0,y0),
				     new Point(x0 + FLOW_SIZE, y0),
				     /*ArrowHead.getModel(flow.getStartArrowModel())*/ArrowHead.ArrowHeadType.NOHEAD,
				     /*ArrowHead.getModel(flow.getEndArrowModel())*/ArrowHead.ArrowHeadType.NOHEAD,
				     h, fillColor, lineColor);
	    // poly.getGraphics().setLinePaint(lineColor);
	    // poly.getGraphics().setFillPaint(fillColor);
		    
	    text =
		new OMResizeableText(refreshable, ml,
				     x0+FLOW_SIZE+offset, y0,
				     mmmValuesAndSizes.getValue(i),
				     OMText.JUSTIFY_LEFT, "flow value "+i, true, true);
	    if (text.getWidth() > maxTextWidth)
		maxTextWidth = text.getWidth();
	    
	    //y0 += h*4 + offset * 2;//(i == 0 ? poly.getHeight()/2 : poly.getHeight()) + offset/3;
	    // y0 += (i == 0 ? width*3 : width*6) + offset/3;
	    group.add(poly);
	    group.add(text);
//	    usualNames[i] = text;
//	    omPolys.add(poly);
//	    omGroups.add(group);
	    this.add(group);
	}

	int frameWidth = FLOW_SIZE + 2*offset + maxTextWidth;
	//customizedTexts.setTexts(usualNames);
	//customizedTexts.setTitle(title);
	int width = x0+frameWidth;
	int height = y0 + 2*h + offset;

	title.translate(width /2, 0);

	if (colorScale != null) {	    
	    scl = new OMResizeableColorScaleLegend(refreshable, ml, getXAnchor(), y0 + offset, 
						   colorScale, colorNames, colorDataColumnName, 
						   colorDataColumnName);
	    scl.showBorder(false);
	    width = Math.max(width, scl.getWidth());
	    height += scl.getHeight();
	    this.add(scl);
	}   
	this.extendBorder(width, height);
    }

//    protected CustomizedLegendText customizedTexts = new CustomizedLegendText();
//    protected ArrayList<OMResizeableGroup> omGroups = new ArrayList<OMResizeableGroup>();
//    protected ArrayList<OMResizeableFlow> omPolys = new ArrayList<OMResizeableFlow>();

    //===============================================================
    // PROPERTIES
    //===============================================================
//    public static final String POLYS_TOTAL_PROPERTY_NAME = "polys.total";
//    public static final String POLYS_PROPERTY_NAME = "polys";
//    public static final String GROUPS_PROPERTY_NAME = "groups";
    public static final String COLORS_PROPERTY_NAME = "colors";
    public static final String FLOW_LEGEND_PROPERTY_NAME = "flowLegend";
    public static final String MUST_BE_RESETED_PROPERTY_NAME = "mustBeReseted";


    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + FLOW_LEGEND_PROPERTY_NAME + ".";
        mustBeReseted = PropUtils.booleanFromProperties(properties, prefix + MUST_BE_RESETED_PROPERTY_NAME, false);

//
//	// Flows
//	int total = PropUtils.intFromProperties(properties, prefix + POLYS_TOTAL_PROPERTY_NAME, 0);
//	for (int i = 0 ; i < total && i < omPolys.size() ; i++) {
//	    omGroups.get(i).setProperties(prefix + GROUPS_PROPERTY_NAME + Integer.toString(i), properties);
//	    omPolys.get(i).setProperties(prefix + POLYS_PROPERTY_NAME + Integer.toString(i), properties);
//	}
//
//	// Texts
//	customizedTexts.setProperties(prefix, properties);
	if (scl != null)
	    scl.setProperties(prefix + COLORS_PROPERTY_NAME, properties);
    }
    
    public Properties getProperties(String prefix, Properties properties) {
	 properties = super.getProperties(prefix, properties);
        if (mustBeReseted) {
            properties = removeAnchorFromProperties(prefix, properties);
            properties = removeSizeFromProperties(prefix, properties);
        }

        prefix = PropUtils.getScopedPropertyPrefix(prefix) + FLOW_LEGEND_PROPERTY_NAME + ".";
        	if (scl != null)
	    properties = scl.getProperties(prefix + COLORS_PROPERTY_NAME, properties);
        properties.put(prefix + MUST_BE_RESETED_PROPERTY_NAME, String.valueOf(mustBeReseted));

        return properties;

    }
    public JComponent getToolGUI() {
        JComponent c = super.getToolGUI();
        //JButton resetLegend = new JButton(Util.loadIcon("undo.png"));
        final JCheckBox resetLegend = new JCheckBox(i18n.get(OMResizeableDiscLegend.class, "align", "auto align"));
        resetLegend.setBorder(BorderFactory.createEmptyBorder(0, 25, 0, 0));
        resetLegend.setSelected(mustBeReseted);
        resetLegend.setBackground(Color.WHITE);
        resetLegend.setOpaque(false);
        resetLegend.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                mustBeReseted = resetLegend.isSelected();
            }
        });
        resetLegend.setToolTipText("Reset discs positions according to their new sizes");
        c.add(resetLegend);
        return c;
    }
   
}
