// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.MapBean;
import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.proj.*;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;

import javax.swing.*;
import java.util.*;
import java.awt.Color;
import java.awt.Dimension;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * A scale, that is not Resizeable, but only translatable.
 */
public class OMResizeableScale extends OMResizeableGroup{


    protected int nbSegs;
    protected double nbKmsPerSeg;
    protected Projection projection;

    private final int defaultWidth = 50;
    private final int defautlHeight = 8;


//    public OMResizeableScale(Refreshable r, MouseControlPointsListener ml, int x0, int y0, Projection proj, int width, int height) {
//	this(r, ml, x0, y0, proj, width, height, 4, 0);
//    }

    public OMResizeableScale(Refreshable r, MouseControlPointsListener ml, int x0, int y0, Projection proj) {
	this(r, ml, x0, y0, proj,4, 0);
    }


    public OMResizeableScale(Refreshable r, MouseControlPointsListener ml, int x0, int y0, Projection proj,int nbSegs, int nbKmsPerSeg) { 
	super(r, ml, x0, y0, 0, 0, "scale");
	if (proj == null)
	    return;
	this.nbSegs = nbSegs;
	this.nbKmsPerSeg = nbKmsPerSeg;
        this.projection = proj;
        width = defaultWidth;
        height = defautlHeight;
	//init();
    }

    public void init(/*Refreshable r, MouseControlPointsListener ml, int x0, int y0, Projection proj*/) {
        graphics.clear();
        add(omControlPoints);
	// find out how many kilometers represent width pixels long
	LatLonPoint.Double p1 = projection.inverse((double)xAnchor, (double)yAnchor, new LatLonPoint.Double());
	LatLonPoint.Double p2 = projection.inverse(xAnchor+width, yAnchor, new LatLonPoint.Double());
	// Get it in kilometer
	float widthInKm = 
	    Length.KM.fromRadians(GreatCircle.sphericalDistance(ProjMath.degToRad(p1.getLatitude()), 
								 ProjMath.degToRad(p1.getLongitude()),
								 ProjMath.degToRad(p2.getLatitude()),
								 ProjMath.degToRad(p2.getLongitude())));
	double nbPixelsPerKM = width/widthInKm;

	if (nbKmsPerSeg <= 0) { // find out a possible value
	    // round down total width to a power of 10	
	    double p = Math.pow(10, (int)Math.floor(Math.log10(widthInKm)));
	    switch ((int)(widthInKm/p)) {
	    case 0: nbSegs = 1; break;
	    case 1: nbSegs = 2; break;
	    case 2: nbSegs = 2; break;
	    case 3: nbSegs = 3; break;
	    case 4: nbSegs = 4; break;
	    case 5: nbSegs = 5; break;
	    case 6: nbSegs = 6; break;
	    case 7: nbSegs = 2; break;
	    case 8: nbSegs = 4; break;
	    case 9: nbSegs = 2; break;
	    }
	    double roundedWidthInKm = p*(int)(widthInKm/p);
	    nbKmsPerSeg = roundedWidthInKm/nbSegs;
	}

	int nbPixelsPerSeg = (int)(nbKmsPerSeg*nbPixelsPerKM);
	int x = xAnchor; // - nbPixelsPerKM*nbSegs/2; // center the drawing
	int y1 = yAnchor - height/2;
	
	int x0 = x; // for debugging message

	add(new OMResizeableText(refreshable, ml, x, y1, "0", OMText.JUSTIFY_CENTER, "0", false));
   
	add(new OMResizeableLine(refreshable, null, x, yAnchor, x, y1));
	for (int i = 0; i < nbSegs; i++, x += nbPixelsPerSeg) {
	    add(new OMResizeableRect(refreshable, null, x, yAnchor, nbPixelsPerSeg, height, i % 2 == 0 ? Color.black : Color.white));
	}
	int tmaxKm = (int)(nbSegs*nbKmsPerSeg);
	
	String stringTmaxKm = tmaxKm > 0 // in km or in meters ?
	    ? String.format("%d km", tmaxKm) 
	    : String.format("%d m", (int)(1000*nbSegs*nbKmsPerSeg));

	add(new OMResizeableText(refreshable, ml, x, y1, stringTmaxKm, OMText.JUSTIFY_CENTER, "max", false));
	add(new OMResizeableLine(refreshable, null, x, yAnchor, x, y1));

	setName("Scale");
        setVisible(getVisible());

	Debug.message("OMResizeableScale", "OMResizeableScale: "+width+" pixels => "+widthInKm+" km "+nbPixelsPerKM+" px/km, "+nbSegs+" segments of "+nbKmsPerSeg+" km, pixels ["+x0+", "+x+"]");
	
    }

    private void fitWidthAndHeight() {
        if (width < 1 || width > 500)
            width = defaultWidth;
        if (height < 1 || height > 50)
            height = defautlHeight;
    }


    /** scale extends according to the projection only */
    public void extend(int dw, int dh) {}

    
    public JComponent getGUI() {

        JPanel gui = new JPanel();
        gui.add(getToolGUI());
        	JLabel visibleLabel = new JLabel(i18n.get(this,"title", "Scale"));
	gui.add(visibleLabel);
	gui.add(Box.createRigidArea(new Dimension(50,20)));
	fitWidthAndHeight();
        JSpinner spWidth = new JSpinner(new SpinnerNumberModel(width,1,500,1));
        JSpinner spHeight = new JSpinner(new SpinnerNumberModel(height,1,50,1));
        spHeight.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent ce) {
                SpinnerModel spm = ((JSpinner)ce.getSource()).getModel();
                height = ((Integer)spm.getValue());
                init();
                refreshGraphics();
            }
        });
        spWidth.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent ce) {
                SpinnerModel spm = ((JSpinner)ce.getSource()).getModel();
                width = ((Integer)spm.getValue());
                init();
                refreshGraphics();
                //refreshInteractiveGraphics();
            }
        });
        gui.add(new JLabel(i18n.get(this,"height","H")));
        gui.add(spHeight);
        gui.add(Box.createRigidArea(new Dimension(20,20)));
        gui.add(new JLabel(i18n.get(this,"width","W")));
        gui.add(spWidth);

        return gui;
    }




    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
	prefix = PropUtils.getScopedPropertyPrefix(prefix);
        height = PropUtils.intFromProperties(properties, prefix + HEIGHT_PROPERTY_NAME, height);
        width = PropUtils.intFromProperties(properties, prefix + WIDTH_PROPERTY_NAME, width);
	fitWidthAndHeight();
        init();
        // setTextProperties(prefix, properties);
    }

    public Properties getProperties(String prefix, Properties properties) {
        if (properties == null)
	    properties = new Properties();
        properties = super.getProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);	
        properties.put(prefix  + WIDTH_PROPERTY_NAME, String.valueOf(width));
        properties.put(prefix  + HEIGHT_PROPERTY_NAME, String.valueOf(height));
	return properties;
    }    
}
