// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.util;

import fr.unicaen.openmap.event.RefreshableGUI;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.Container;
import java.text.*;
import java.util.Properties;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ResizerPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfigurationPanel;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import fr.unicaen.openmap.gui.component.ComponentTitledBorder;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import java.awt.CardLayout;
import java.awt.Dimension;
import org.jdesktop.swingx.JXCollapsiblePane;

/**
 * Utility class to resize a graphical object proportionally to its data value
 * <em>or</em> a fixed size.
 * 
 * @see Resizer
 */
public class SymbolResizer extends Resizer implements RefreshableGUI {

    public static final String dataColumnNamePropertyString = "dataColumnName";
    protected String dataColumnName = "";
    private boolean withRadius;
    //private JCheckBox checkShowNullAsPoint;
    private JPanel mainPanel;
    /**
     * Use I18N mecanism to allow internationalization. Use I18N.properties file
     * or I18N_fr_FR.properties for french version
     */
    protected I18n i18n = new BasicI18n();

    public String getDataColumnName() {
        return getIsFixedSize() ? null : dataColumnName;
    }

    public boolean setDataColumnName(String s) {
        if (dataColumnName.equals(s)) {
            Debug.message("Resizer", "SymbolResizer.setDataColumnName() skipped for " + s);
            return false;
        }

        dataColumnName = s;
        if (dc != null && dc instanceof DataColumnValueComputer) {
            ((DataColumnValueComputer) dc).setDataColumnName(dataColumnName);
        }
        Debug.message("Resizer", "SymbolResizer.setDataColumnName() changed to " + s);
        if (getDataStorageIterator() != null) {
            setMinMaxValue(getDataStorageIterator());
            refreshGUI();
        }
        fireResizerChanged();
        return true;
    }
    public static final String isFixedSizePropertyString = "isFixedSize";
    protected boolean isFixedSize = true;

    public boolean getIsFixedSize() {
        return isFixedSize;
    }

    // protected void setIsFixedSize(boolean b) { isFixedSize = b; }
    public void setIsFixedSize(boolean b) {
        isFixedSize = b;
        fireResizerChanged();
    }
    public static final String fixedSizePropertyString = "fixedSize";
    int fixedSize = 10;

    protected int getFixedSize() {
        return fixedSize;
    }

    public void setFixedSize(int s) {
        fixedSize = s;
        fireResizerChanged();
    }

    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);

        setDataColumnName(properties.getProperty(prefix + dataColumnNamePropertyString, dataColumnName));
        setIsFixedSize(Util.isTrue(properties.getProperty(prefix + isFixedSizePropertyString, "true")));
        setFixedSize(PropUtils.intFromProperties(properties, prefix + fixedSizePropertyString, 25));
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);

        properties.put(prefix + dataColumnNamePropertyString, dataColumnName);
        properties.put(prefix + isFixedSizePropertyString, new Boolean(
                getIsFixedSize()).toString());
        properties.put(prefix + fixedSizePropertyString, String.valueOf(getFixedSize()));
        return properties;
    }

    /**
     * Returns element's size (width, radius or anything determinating its size)
     * proportionnal to value parameter.
     */
    public float getSize(double value) {
        if (getIsFixedSize()) {
            return getFixedSize();
        }
        return super.getSize(value);
    }

    public float getSize(DataObject data) {
        if (isFixedSize) {
            return getFixedSize();
        }
        return super.getSize(data);
    }

    public String getString(DataObject data) {
        if (getIsFixedSize()) {
            return String.valueOf(getFixedSize());
        }
        return dc.getString(data);
    }

    public SymbolResizer(DataValueComputer dc) {
        super(dc);
    }

    public void setMinMaxValue(DataStorageIterator dsi) {
        if (!getIsFixedSize()) {  // min and max value are not needed for fixed size
            super.setMinMaxValue(dsi);
        }
    }
    private DataStorageIterator dsi;

    // protected DataStorageIterator getDataStorageIterator() { return dsi; }
    public DataStorageIterator getDataStorageIterator() {
        return dsi;
    }

    protected void setDataStorageIterator(DataStorageIterator dsi) {
        this.dsi = dsi;
    }

    public void refreshGUI(DataStorageIterator dsi) {
        setDataStorageIterator(dsi);
        if (proportionalGUI != null) {
            proportionalGUI.setDataList(this);
        }
    }

    public void refreshGUI() {
        if (proportionalGUI != null) {
            proportionalGUI.refresh();
        }
    }
    // useless now, panel is able to have dynamic refresh
    // protected void setGUIEnabled() {
    // }
    // why not to use Resizer.defaultGUI ???
    protected ResizerPanel proportionalGUI;
    // in order to manage this panel, if needed (sample in Pie which add a
    // radius size)
    //protected ButtonGroup fixedSizeRBGroup;
    protected JComboBox comboSize;
//    public ButtonGroup getFixedSizeRBGroup() {
//        return fixedSizeRBGroup;
//    }
//    protected ActionListener fixedSizeRBGroupListener;
    protected ItemListener comboSizeListener;
    private final int FIXE_INDEX = 1;
    private final int PROPORTIONAL_INDEX = 0;

    public ItemListener getFixedSizeRBGroupListener() {
        return comboSizeListener;
    }

    /** The GUI palette needs all the data column names */
    public JComponent getGUI(DataStorageIterator dsi) {
        setDataStorageIterator(dsi);
        if (maxSize == NO_MAX_SIZE) {
            setMaxSize(getMaxSizeAccordingToNbRecords(dsi.getDataStorage().getCount()));
        }
        if (confPanel != null) {
            return confPanel;
        }

        //fixedSizeRBGroup = new ButtonGroup();
        comboSize = new JComboBox(new String[]{"Proportionnelle", "Unique"});
        JPanel fixedGUI = new JPanel();
        //fixedGUI.setBorder(BorderFactory.createLineBorder(Color.blue));
        //JPanel wrapFixed = new JPanel(new BorderLayout());
        //wrapFixed.add(fixedGUI, BorderLayout.WEST);
        if (proportionalGUI == null) {
            proportionalGUI = new ResizerPanel(this);
        }

        //proportionalGUI.setBorder(BorderFactory.createLineBorder(Color.red));
        final JSpinner fixedSizeField = new JSpinner(new SpinnerNumberModel(
                getFixedSize(), 0, 1000, 1));
        fixedSizeField.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {

                setFixedSize(((Integer) fixedSizeField.getValue()).intValue());

            }
        });

        //final JRadioButton isFixedSizeRB = new JRadioButton(i18n.get(this,
        //        "fixedSizeButton", "Fixed Size"), getIsFixedSize());
        //fixedSizeRBGroup.add(isFixedSizeRB);
//        isFixedSizeRB.addItemListener(new ItemListener() {
//
//            public void itemStateChanged(ItemEvent e) {
//
//                boolean b = ((JRadioButton) (e.getItemSelectable())).isSelected();
//                fixedSizeField.setEnabled(b);
//                setIsFixedSize(b);
//            }
//        });

//        final JRadioButton isProportionalRB = new JRadioButton(i18n.get(this,
//                "proportionalSizeButton", "Proportional Size"),
//                !getIsFixedSize());
//
//        fixedSizeRBGroup.add(isProportionalRB);
//        isProportionalRB.addItemListener(new ItemListener() {
//
//            public void itemStateChanged(ItemEvent e) {
//
//                boolean b = ((JRadioButton) (e.getItemSelectable())).isSelected();
//                proportionalGUI.setEnabled(b);
//                setIsFixedSize(!b);
//            }
//        });



//        isFixedSizeRB.addActionListener(fixedSizeRBGroupListener);
//        isProportionalRB.addActionListener(fixedSizeRBGroupListener);
//        fixedGUI.add(isFixedSizeRB);
        fixedGUI.add(new JLabel("Taille en pixels"));
        fixedGUI.add(fixedSizeField);

        //Box panel = Box.createVerticalBox();


        // init... for ending
        //isFixedSizeRB.setSelected(getIsFixedSize());
        fixedSizeField.setEnabled(getIsFixedSize());

//        isProportionalRB.setSelected(!getIsFixedSize());
//        proportionalGUI.setEnabled(!getIsFixedSize());
//        //panel.setBorder(BorderFactory.createTitledBorder(""));
//        panel.add(wrapFixed);
//        JPanel wrapProp = new JPanel();
//        wrapProp.add(isProportionalRB);
//        wrapProp.add(proportionalGUI.getAndRemoveDataList());
//        proportionalGUI.setBorder(BorderFactory.createEmptyBorder(-10, 50 ,0 ,0));
//        JPanel wrapLeftProp = new JPanel(new BorderLayout());
//        wrapLeftProp.add(wrapProp, BorderLayout.WEST);
//        panel.add(wrapLeftProp);
//        //proportionalGUI.setBorder(new ComponentTitledBorder(isProportionalRB, panel, BorderFactory.createLineBorder(Color.red)));
//        panel.add(proportionalGUI);
        //ResizerPanel sumProp = new ResizerPanel(this, true, true, false);
        //sumProp.setEnabled(true);
        //sumProp.
        final JPanel panel = new JPanel();
        final CardLayout cl = new CardLayout();
        panel.setLayout(cl);
        panel.add("prop", proportionalGUI);
        panel.add("fix", fixedGUI);
        //panel.add("sumprop", new JPanel());
        comboSizeListener = new ItemListener() {

            public void itemStateChanged(ItemEvent arg0) {
                setIsFixedSize(comboSize.getSelectedIndex() == FIXE_INDEX);
                String card = "";
                switch(comboSize.getSelectedIndex()){
                    case FIXE_INDEX:
                        card = "fix";
                        break;
                    case PROPORTIONAL_INDEX:
                        card = "prop";
                        proportionalGUI.setDataListWanted(true);
                        break;
                    default:
                        card = "prop";
                        proportionalGUI.setDataListWanted(false);

                }
                cl.show(panel, card);
            }
        };

        mainPanel = new JPanel(new BorderLayout());
        comboSize.addItemListener(comboSizeListener);
        JPanel wrapNorth = new JPanel();
        wrapNorth.add(new JLabel("Taille"));
        wrapNorth.add(comboSize);
        mainPanel.add(wrapNorth, BorderLayout.NORTH);
        mainPanel.add(panel, BorderLayout.CENTER);
//        if(checkShowNullAsPoint != null)
//            mainPanel.add(checkShowNullAsPoint, BorderLayout.SOUTH);
        proportionalGUI.setEnabled(true);
        comboSize.setSelectedIndex(isFixedSize ? FIXE_INDEX : PROPORTIONAL_INDEX);
        //panel.add(mainPan);
        confPanel = ThematicConfPanel.createThematicConfPanel(i18n.get(ResizerPanel.class, "size", "Size"), false);
        //comboSize
        ((ThematicConfPanel) confPanel).setContent(mainPanel);

        return confPanel;
    }

    public boolean isWithRadius() {
        return withRadius;
    }

    public void setWithRadius(boolean withRadius) {
        this.withRadius = withRadius;
    }

    public ResizerPanel getProportionalGUI() {
        return proportionalGUI;
    }

    public void setShowNullAsPoint(JCheckBox cb){
        JPanel wrapSouth = new JPanel();
        wrapSouth.add(cb);
        mainPanel.add(wrapSouth, BorderLayout.SOUTH);
        mainPanel.updateUI();
    }

    public JComboBox getComboSize(){return comboSize;}
}
