// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import java.util.*;
import java.awt.event.*;
import javax.swing.*;

import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.*;
import fr.unicaen.openmap.util.classification.Classification;
import fr.unicaen.openmap.util.classification.ClassificationListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;

import com.bbn.openmap.omGraphics.*;

/**
 * Regular Polygons visualization mode colored under a classification.
 * <p>colorDataColumnName propertiy gives the column values that will determined the color of a disc.
 * Its size is determined with dataColumnName as for simple Disc visualization mode.  
 */
public class ColoredRegularPolygon extends RegularPolygon implements ClassificationListener {
    public static final String colorDataColumnNamePropertyString = "colorDataColumnName";

    protected String colorDataColumnName;
    protected Colorizer colorizer = new Colorizer();

    public ColoredRegularPolygon(ThematicDataDisplayLayer layer, String name) { 
	this(layer, name, i18n.get(ColoredDisc.class, "coloredRegularPolygon", "ColoredRegularPolygon"));
    }
    public ColoredRegularPolygon(ThematicDataDisplayLayer layer, String name, String typeName) { 
	super(layer, name, typeName); 
	colorizer.addClassificationListener(this);
    }

    JComboBox dataList;
    
    public void setProperties(String prefix, Properties properties) {
	super.setProperties(prefix, properties);
	colorizer.setProperties(prefix, properties);
	colorDataColumnName = properties.getProperty(prefix + colorDataColumnNamePropertyString, "color");
    }
    
    public Properties getProperties(String prefix, Properties properties) {
	properties = super.getProperties(prefix, properties);
	properties = colorizer.getProperties(prefix, properties);
	properties.put(prefix + colorDataColumnNamePropertyString, colorDataColumnName);
	return properties;
    }
    
    public OMGraphicList generateGraphics(DataStorageIterator dsi) {
 
	resizer.setMinMaxValue(dsi);
	colorizer.setData(dsi, colorDataColumnName);

	OMSizeableGraphicList omgl = new OMSizeableGraphicList<OMDisc>();
	OMGraphicList ompl = new OMGraphicList(); // for the OMPoint (Discs of null sizes)
	float startAngleInRadians = getStartAngleInRadians();
	while (dsi.hasNext()) {
	    DataObject data = dsi.next();
	    OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);
	    if (omdl == null)
		continue;

	    Centroid centroid = omdl.getCentroid();
	    if (centroid == null)
		continue;

	    double size = resizer.getSize(data);
	    OMGraphic p;
	    // 0.001 is to be adapted with LatLon Graphics
	    if (size < 0.001) { // null sized disc => OMPoint
		p = new OMPoint((float)centroid.getLat(), (float)centroid.getLon());
		ompl.add(p);
	    } else {
//                float lat = (float) centroid.getLat();
//                float lon = (float) centroid.getLon();
                int offX = 0;
                int offY = 0;
//                if (offseter.isIndividual()) {
//                    key = offseter.buildKey(data);
//                    if (!offseter.contains(key)) {
//                        offseter.insert(key, lat, lon);
//                    }
//                }
                String key = offseter.buildKey(data);
                offX = offseter.getX(key);
                offY = offseter.getY(key);
		p = new OMRegularPolygon(this, key, (float)centroid.getLat(), (float)centroid.getLon(),
					 nbEdges, (float)size, 
					 offX, offY,
					 startAngleInRadians, starCoef,
					 resizer.getString(data),offseter.isVisible(), offseter.isLineVisible());
		omgl.add(p);
                //((OMRegularPolygon)p).addOffsetableListener(offseter);
                mouseControlPointsListener.addOMControlPoints(((OMRegularPolygon)p).controlPoints);
	    }

	    p.setLinePaint(drawingAttributes.getLinePaint());
	    Object colorValue = data.getData(colorDataColumnName);
	    p.setFillPaint(colorizer.getColor(colorValue));
	    p.setStroke(drawingAttributes.getStroke());
	}
	omgl.sort();
	initMinMedianMaxValuesAndSizes(omgl);  // used for the legend
	if (ompl.size() > 0) // add null sized polygons
	    omgl.add(ompl);
	return omgl;
    }

    public JComponent getGUI(final DataStorageIterator dsi) {
	JPanel panel = (JPanel)super.getGUI(dsi);
	try {
	graphicsPanelToggleControlsDisplay(false, true, false, false, false);
	JPanel coloredRegularpolygonPanel = new JPanel();
	coloredRegularpolygonPanel.setBorder(BorderFactory.createTitledBorder(i18n.get(this,"coloredRegularPolygonTitle","Colored Regular Polygon")));
	coloredRegularpolygonPanel.setLayout(new BoxLayout(coloredRegularpolygonPanel, BoxLayout.Y_AXIS));

	final ColorizerPanel colorizerPanel = colorizer.getGUI();
	JPanel colorDataPanel = new JPanel(); // used to set and display colorDataColumnName
	colorDataPanel.setLayout(new BoxLayout(colorDataPanel, BoxLayout.X_AXIS));
	colorDataPanel.add(new JLabel(i18n.get(this,"colorDataColumnName","Color Data Column Name: ")));
	dataList = new JComboBox(dsi.getDataColumnNames());
	dataList.setSelectedIndex(dsi.getIndexForColumnName(colorDataColumnName));
	dataList.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
                    try {
		    int index = ((JComboBox)e.getSource()).getSelectedIndex();
		    setColorDataColumnName(dsi.getRealColumnName(index));
		    //		    colorizerPanel.refreshPanel();
                    } catch (ArrayIndexOutOfBoundsException ex) { }
		}});
	colorDataPanel.add(dataList);
	colorDataPanel.add(Box.createHorizontalGlue());

	coloredRegularpolygonPanel.add(colorDataPanel);


	coloredRegularpolygonPanel.add(colorizerPanel);

	panel.add(coloredRegularpolygonPanel);
	} catch (Exception e) { }
	return panel;
    }

    public void setColorDataColumnName(String name) {
	colorDataColumnName = name;
	refresh();
    }

    public void refreshGUI(DataStorageIterator dsi) {
        super.refreshGUI(dsi);
        if (dataList == null) return;
        dataList.removeAllItems();
	for (String item : dsi.getDataColumnNames())
	    dataList.addItem(item);
	dataList.setSelectedIndex(dsi.getIndexForColumnName(colorDataColumnName));
    }


    public void classificationChanged(Object source) {
	if (source != colorizer)
	    colorizer.setData(getLayer().getDataStorageIterator(), colorDataColumnName);
	refresh();
    }

}
