// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import java.util.HashMap;
import java.util.Set;

import java.util.Properties;

import javax.swing.JComponent;


import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.event.MapMouseListener;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;

import com.bbn.openmap.proj.Projection;


import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.dataStorage.Centroid;
import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.DataStorageIterator;
import fr.unicaen.openmap.dataStorage.OMDataLocation;


import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.FlowPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ResizerPanel;


import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicGUI;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicMouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.ArrowHead;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMFlow;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.Refreshable;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.HaveWidthAndData;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableFlowLegend;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableGraphic;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableFrame;


import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Colorizer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.MinMaxValues;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.DataColumnValueComputer;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Resizer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ResizerListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.SymbolResizer;
import fr.unicaen.openmap.util.classification.ClassificationListener;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.classification.Classification;
import fr.unicaen.openmap.util.classification.Quantile;
import java.util.ArrayList;

/**
 * Flow visualization mode
 *  
 * Bug: There are many reasons for BENDED_MODE, BENDED_DISTANCE and ARC_LENGTH to be non static in this class !!!
 * Curved points coords should be put in properties only if bendedMode is equal to BENDED_MANUEL_MODE ???
 * @author Jerome Chauveau
 */
public class Flow extends Raw implements ResizerListener, ClassificationListener {

    public enum type {

        FLOW, PATH
    };
    private type chosenType = type.PATH;
    private boolean dragging;
    //protected String toColumnName = "";
    protected String valueColumnName = "";
    private int startArrowModel = 1;//noHead
    private int endArrowModel = 1;
    private Projection projection;
    private FlowPanel flowPanel;
    private Object[] primaryKeyArray = {};
    private HashMap<String, LatLonPoint> curvedPoints;
    private boolean controlPointsVisibility = false;
    private MinMaxValues minMaxValues;
    private int flowNumber;
    private boolean classified;
    private OMGraphicList controlPoints;
    ThematicMouseControlPointsListener mouseControlPointsListener;
    protected Colorizer colorizer = new Colorizer();
    public static final String colorDataColumnNamePropertyString = "colorDataColumnName";
    protected String colorDataColumnName = "";
    protected SymbolResizer resizer = new SymbolResizer(new DataColumnValueComputer(""));
    private boolean showStartEqualsEnd = true;
    private int loopRadius;
    private ArrayList<String> stepColumns;
    private int nbStep;

    public Flow(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(Flow.class, "flow", "Flow"));
    }

    public Flow(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
        interactive = true;
        colorizer.addClassificationListener(this);
        //mouseControlPointsListener = new MouseFlowControlPointListener();
        mouseControlPointsListener = new ThematicMouseControlPointsListener(this);
        resizer.addResizerListener(this);
        resizer.setMaxSize(10);
        ((SymbolResizer) resizer).setFixedSize(10);
        curvedPoints = new HashMap<String,/*Point*/ LatLonPoint>();
        stepColumns = new ArrayList<String>();
    }
    //PROPERTIES
    // fromColumn is geometryColumn from raw visu
    //static String toColumnNamePropertyString = "toColumn";
    static String valueColumnNamePropertyString = "valueColumn";
    static String bendedModePropertyString = "bendedMode";
    static String arcLengthPropertyString = "arcLength";
    static String bendedDistancePropertyString = "bendedDistance";
    static String curvedPointPropertyString = "curvedPoint";
    static String flowNumberPropertyString = "flowNumber";
    static String flowNamePropertyString = "name";
    static String flowPrimaryKeyPropertyString = "primary_keys";
    static String flowStartArrowPropertyString = "start_arrow";
    static String flowEndArrowPropertyString = "end_arrow";
    static String flowModePropertyString = "mode";
    static String classifiedPropertyString = "classified";
    static String controlPointsVisibilityString = "controlPointsVisibility";
    static String showStartEqualsEndString = "showStartEqualsEnd";
    static String loopRadiusString = "showStartEqualsEnd";
    static String stepColumnsPropertyString = "stepColumns";
    static String nbStepPropertyString = "nbStep";

    @Override
    public void setProperties(String prefix, Properties properties) {

        super.setProperties(prefix, properties);
        resizer.setProperties(prefix, properties);
        resizer.addResizerListener(this);
        colorizer.setProperties(prefix, properties);
        colorDataColumnName = properties.getProperty(prefix + colorDataColumnNamePropertyString, "data");
        /*toColumnName = properties.getProperty(
        prefix + toColumnNamePropertyString, null);*/
        nbStep = PropUtils.intFromProperties(properties, prefix + nbStepPropertyString, 0);
        setChosenType(nbStep == 0 ? Flow.type.PATH : Flow.type.FLOW);
        String[] stepTab = PropUtils.stringArrayFromProperties(properties, prefix + stepColumnsPropertyString, " ");
        if (stepTab != null) {
            for (String s : stepTab) {
                stepColumns.add(s);
            }
        }
        valueColumnName = properties.getProperty(
                prefix + valueColumnNamePropertyString, null);
        OMFlow.BENDED_MODE = Integer.parseInt(
                properties.getProperty(prefix + bendedModePropertyString, "1"));
        OMFlow.BENDED_DISTANCE = Integer.parseInt(
                properties.getProperty(prefix + bendedDistancePropertyString, "0"));
        OMFlow.ARC_LENGTH = Integer.parseInt(
                properties.getProperty(prefix + arcLengthPropertyString, "5"));
        flowNumber = Integer.parseInt(
                properties.getProperty(prefix + flowNumberPropertyString, "0"));
        classified = Boolean.parseBoolean(properties.getProperty(prefix + classifiedPropertyString, "false"));
        showStartEqualsEnd = Boolean.parseBoolean(properties.getProperty(prefix + showStartEqualsEndString, "true"));
        controlPointsVisibility = Boolean.parseBoolean(properties.getProperty(prefix + controlPointsVisibilityString, "true"));
        startArrowModel = Integer.parseInt(properties.getProperty(prefix + flowStartArrowPropertyString, "1"));
        endArrowModel = Integer.parseInt(properties.getProperty(prefix + flowEndArrowPropertyString, "1"));
        setLoopRadius(Integer.parseInt(properties.getProperty(prefix + loopRadiusString, "25")));
        String t = properties.getProperty(prefix + flowModePropertyString, "FLOW");
        if (t.equals("PATH")) {
            chosenType = type.PATH;
        } else {
            chosenType = type.FLOW;
        }

        //primary keys array
        buildPrimaryKeyArray(properties.getProperty(prefix + flowPrimaryKeyPropertyString, ""));

        //curved points
        String key;
        double x, y;
        for (int i = 0; i < flowNumber; i++) {
            key = properties.getProperty(
                    prefix + i + "." + flowNamePropertyString, "");
            x = Double.parseDouble(
                    properties.getProperty(prefix + key + "." + "x", "0"));
            y = Double.parseDouble(
                    properties.getProperty(prefix + key + "." + "y", "0"));
            curvedPoints.put(key, /*new Point(x,y)*/ new LatLonPoint.Double(x, y));
        }
        setStepColumn(0, geometryColumnName);
    }

    @Override
    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        properties = resizer.getProperties(prefix, properties);
        //properties.put(prefix + toColumnNamePropertyString, toColumnName);
        String stepString = "";
        for (String s : stepColumns) {
            stepString += " " + s;
        }
        properties.put(prefix + stepColumnsPropertyString, stepString);
        if (valueColumnName != null) {
            properties.put(prefix + valueColumnNamePropertyString, valueColumnName);
        }
        properties.put(prefix + nbStepPropertyString, String.valueOf(nbStep));
        properties.put(prefix + bendedModePropertyString, String.valueOf(OMFlow.BENDED_MODE));
        properties.put(prefix + bendedDistancePropertyString, String.valueOf(OMFlow.BENDED_DISTANCE));
        properties.put(prefix + arcLengthPropertyString, String.valueOf(OMFlow.ARC_LENGTH));
        properties.put(prefix + flowPrimaryKeyPropertyString, primaryKeysToString());
        properties.put(prefix + flowNumberPropertyString, String.valueOf(curvedPoints.size()));
        properties.put(prefix + flowStartArrowPropertyString, String.valueOf(startArrowModel));
        properties.put(prefix + flowEndArrowPropertyString, String.valueOf(endArrowModel));
        properties.put(prefix + flowModePropertyString, chosenType.toString());
        properties.put(prefix + classifiedPropertyString, String.valueOf(classified));
        properties.put(prefix + isShowStartEqualsEnd(), String.valueOf(isShowStartEqualsEnd()));
        properties.put(prefix + controlPointsVisibilityString, String.valueOf(controlPointsVisibility));
        properties.put(prefix + loopRadiusString, String.valueOf(getLoopRadius()));
        { // should be put in properties only if bendedMode is equal to BENDED_MANUEL_MODE ???
            int i = 0;
            Set<String> keys = curvedPoints.keySet();
            for (String key : keys) {
                properties.put(prefix + i + "." + flowNamePropertyString, key);
                properties.put(prefix + key + "." + "x", String.valueOf(curvedPoints.get(key).getLatitude()));
                properties.put(prefix + key + "." + "y", String.valueOf(curvedPoints.get(key).getLongitude()));
                i++;
            }
        }
        properties = colorizer.getProperties(prefix, properties);
        properties.put(prefix + colorDataColumnNamePropertyString, colorDataColumnName);

        return properties;
    }

    @Override
    public OMGraphicList generateGraphics(DataStorageIterator dsi) {
        //OMGraphicList splines = new OMGraphicList();
        // OMGraphicList radials = new OMGraphicList();
        try {
            if (classified) {
                colorizer.setData(dsi, colorDataColumnName);
            }
            setPrimaryKeyArray(dsi.getDataColumnNames());
            resizer.setMinMaxValue(dsi);
            projection = getLayer().getProjection();
            cachedOMGraphicList = new OMGraphicList();
            controlPoints = new OMGraphicList();
            mouseControlPointsListener.clearControlPoints();

            if (chosenType == type.FLOW) {
                int id = 0;

                while (dsi.hasNext()) {
                    DataObject data = dsi.next();
                    for (int i = 0; i < nbStep; i++) {
                        // System.out.println("FROM " + stepColumns.get(i) + " - to: " + stepColumns.get(i + 1));
                        OMDataLocation omdl_from = data.getOMDataLocation(stepColumns.get(i)/*geometryColumnName*/);
                        OMDataLocation omdl_to = data.getOMDataLocation(stepColumns.get(i + 1)/*toColumnName*/);
                        if (omdl_from == null || omdl_to == null) {
                            continue;
                        }
                        Centroid centroid_from = omdl_from.getCentroid();
                        Centroid centroid_to = omdl_to.getCentroid();
                        if (centroid_from == null || centroid_to == null) {
                            continue;
                        }
                        String stringData = !resizer.getIsFixedSize() ? data.getData(valueColumnName).toString() : "";
                        if (!isInteractiveOMFlow(String.valueOf(id))) {
                            OMFlow omf = new OMFlow(id + "-" + i, this,
                                    centroid_from,
                                    centroid_to,
                                    ArrowHead.getModel(startArrowModel),
                                    ArrowHead.getModel(endArrowModel),
                                    (int) resizer.getSize(data),
                                    stringData);

                            if (classified) {
                                Object value = data.getData(colorDataColumnName);
                                omf.setFillPaint(colorizer.getColor(value));
                            }
                            cachedOMGraphicList.add(omf);
                            //splines.add(omf.getOMSpline());
                            //radials.add(omf.getRadials());
                            //splines.add(omf.getOMSpline());
                            mouseControlPointsListener.addOMControlPoints(omf.getControlPoints());
                            controlPoints.add(omf.getControlPoints());
                            id++;
                        }
                    }
                }
            } else {
                int id = 0;
                Centroid current = null;
                Centroid last = null;
                dsi.reset();
                while (dsi.hasNext()) {
                    DataObject data = dsi.next();
                    //System.out.println("id = " + id);
                    //key = buildKey(data);
                    OMDataLocation omdlFrom = data.getOMDataLocation(geometryColumnName);
                    if (omdlFrom == null) {
                        current = null;
                        last = null;
                        id++;
                        continue;
                    }
                    current = omdlFrom.getCentroid();
                    if (id == 0 || last == null) {
                        id++;
                        last = current;
                        continue;
                    }
                    String stringData = !resizer.getIsFixedSize() ? data.getData(valueColumnName).toString() : "";
                    if (!isInteractiveOMFlow(String.valueOf(id))) {
                        OMFlow omf = new OMFlow(String.valueOf(id), this,
                                last,
                                current,
                                ArrowHead.getModel(startArrowModel),
                                ArrowHead.getModel(endArrowModel),
                                (int) resizer.getSize(data),
                                stringData);

                        if (classified) {
                            Object value = data.getData(colorDataColumnName);
                            omf.setFillPaint(colorizer.getColor(value));
                        }
                        //OMDisc omf = new OMDisc(50, 50, 20, "");
                        cachedOMGraphicList.add(omf);
                        //splines.add(omf.getOMSpline());
                        //radials.add(omf.getRadials());
                        mouseControlPointsListener.addOMControlPoints(omf.getControlPoints());
                        controlPoints.add(omf.getControlPoints());
                        //mouseControlPointsListener.addOMControlPoints(controlPoints);
                        /*controlPoints.add(controlPoints);*/
                    }
                    last = current;
                    id++;
                }

            }

            cachedOMGraphicList.setLinePaint(drawingAttributes.getLinePaint());
            if (!classified) {
                cachedOMGraphicList.setFillPaint(drawingAttributes.getFillPaint());
            }
            cachedOMGraphicList.setStroke(drawingAttributes.getStroke());
            initMinMaxValues(cachedOMGraphicList);
            cachedOMGraphicList.add(0, controlPoints);
            ////cachedOMGraphicList.add(splines);
            ////splines.setLinePaint(Color.RED);
            ////cachedOMGraphicList.add(radials);
            ////radials.setLinePaint(Color.GREEN);
            //OMGraphicList graphics = new OMGraphicList();
            //graphics.add(splines);
            //graphics.add(cachedOMGraphicList);
            //fr.setVisible(true);
            //pFlow.repaint();
            return cachedOMGraphicList;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    //just for testing
//    class PanelFlow extends JPanel {
//
//        ArrayList<OMFlow> omfs;
//
//        public PanelFlow() {
//            super();
//            omfs = new ArrayList<OMFlow>();
//        }
//
//        public void addOMFlow(OMFlow omf){
//            omfs.add(omf);
//        }
//
//        public void paint(Graphics g) {
//            super.paint(g);
//            for(OMFlow omf : omfs){
//                g.drawPolygon(omf.getXBorderAsInt(), omf.getYBorderAsInt(), omf.getYBorderAsInt().length);
//            }
//
//        }
//    }
    private boolean isInteractiveOMFlow(String id) {
        //boolean res = false;
        for (int i = 0; i < interactiveGraphics.size(); i++) {
//            if (interactiveGraphics.getOMGraphicAt(i) instanceof OMFlow) {
//                OMFlow omf = (OMFlow) interactiveGraphics.getOMGraphicAt(i);
//                if (omf.getID().equals(id)) {
//                    res = true;
//                }
//            }
            //why not the following ?
            OMGraphic omg = interactiveGraphics.getOMGraphicAt(i);
            if (omg instanceof OMFlow && ((OMFlow) omg).getID().equals(id)) {
                return true;
            }
        }
        return false;
        //return res; // return false; ??
    }

    public void removeDraggedOMFlow(OMFlow omflow) {
        addInteractiveGraphic(omflow);
        //controlPoints.remove(omflow.getControlPoints());
        //addInteractiveGraphic(omflow.getControlPoints());
    }

    public void addDraggedOMFlow(OMFlow omflow) {
        //cachedOMGraphicList.add(omflow);
        // controlPoints.add(omflow.getControlPoints());
        clearInteractiveGraphics();
    }

    @Override
    public JComponent getGUI(final DataStorageIterator dsi) {
        thematicGUI = (ThematicGUI) super.getGUI(dsi);
//        if (toColumnName == null) {
//            toColumnName = dsi.getOMColumnNames()[0];
//        }

        if (valueColumnName != null) {
            resizer.setDataColumnName(valueColumnName);
        }
        flowPanel = new FlowPanel(
                this, dsi, startArrowModel, endArrowModel, guiOrientation, thematicGUI, colorizer);
        thematicGUI.add(flowPanel);
        if (colorizer.getClassification() == null) {
            Quantile quantile = new Quantile(null, 4);
            quantile.reInit(dsi, colorDataColumnName);
            colorizer.setClassification(quantile);
        }

        return thematicGUI;
    }

    public void setClassification(Classification classif) {
        colorizer.setClassification(classif);
    }

    protected void initMinMaxValues(OMGraphicList omgl) {
        if (omgl == null || omgl.size() == 0) {
            return;
        }
        HaveWidthAndData min = (HaveWidthAndData) (omgl.getOMGraphicAt(0));
        HaveWidthAndData max = min;
        for (int i = 1; i < omgl.size(); i++) {
            HaveWidthAndData hwad = (HaveWidthAndData) omgl.getOMGraphicAt(i);
            int size = hwad.getWidth();
            if (size > max.getWidth()) {
                max = hwad;
            } else if (size < min.getWidth()) {
                min = hwad;
            }
        }
        minMaxValues =
                new MinMaxValues(Util.parseDouble(min.getData()), Util.parseDouble(max.getData()));
    }

    public void refreshGUI(DataStorageIterator dsi) {
        super.refreshGUI(dsi);
        //sSystem.out.println("REFRESH GUI FLOW");
        resizer.refreshGUI(dsi);
        if (flowPanel != null) {
            flowPanel.refreshGUI(dsi);
        }
    }

    /**
     * gives curved point for the given id
     *
     * @param id
     * @return
     */
    public /*Point*/ LatLonPoint getCurvedPointForOMFlow(String id) {
        return curvedPoints.get(id);
    }

    public void setCurvedPointForOMFlow(String id, LatLonPoint p) {
        /*if(!existsCurvedPointForOMFlow(id))
        curvedPoints.add(id,p);
        else
        curvedPoints.set(id,p);*/
        curvedPoints.put(id, p);
    }

    public boolean existsCurvedPointForOMFlow(String id) {
        return curvedPoints.get(id) != null;
        /*try{
        Point pt = curvedPoints.get(id);
        if(Double.isNaN(pt.getX()))
        return false;
        return true;
        }catch(Exception e){
        return false;}	*/

    }

    public boolean isReady() {
        return true;
        /*if(resizer.getIsFixedSize())
        return
        super.isReady() &&
        toColumnName != null &&
        ! "".equals(toColumnName) &&
        primaryKeyArray !=null &&
        primaryKeyArray.length > 0;
        else
        return super.isReady() &&
        //! "".equals(fromColumnName) &&
        toColumnName != null && ! "".equals(toColumnName) &&
        (valueColumnName != null && ! "".equals(valueColumnName)) &&
        primaryKeyArray !=null &&
        primaryKeyArray.length > 0;*/
    }

    public String getMissingInfos() {
        String mess = getName() + ":\n";

        if ("".equals(valueColumnName) && !resizer.getIsFixedSize()) {
            mess += i18n.get(ResizerPanel.class, "dataColumnNameLabel", "Data Column Name") + "\n";
        }

//        if ("".equals(toColumnName)) {
//            mess += i18n.get(FlowPanel.class, "to", "to") + "\n";
//        }

        if (primaryKeyArray == null || primaryKeyArray.length == 0) {
            mess +=/*i18n.get(FlowPanel.class,"to","to")*/ "identification!" + "\n";
        }

        return mess;
    }

    public OMResizeableGraphic getLegend(Refreshable r,
            MouseControlPointsListener ml) {
        if (minMaxValues == null) {
            return null;
        }
        int bended_tmp = OMFlow.BENDED_DISTANCE;
        OMFlow.BENDED_DISTANCE = 0;
        OMResizeableFrame frame = new OMResizeableFlowLegend(this, r, ml, 0, 0,
                resizer, minMaxValues, valueColumnName,
                drawingAttributes.getLinePaint(), drawingAttributes.getFillPaint(),
                classified ? colorizer.getColorScale() : null,
                classified ? colorizer.getColorsTexts() : null,
                colorDataColumnName, getFullName());
        frame.setVisuFrame(true);
        OMFlow.BENDED_DISTANCE = bended_tmp;
        return frame;
    }

    public void setOMColumnName(String name) {
        super.setOMColumnName(name);
        setStepColumn(0, geometryColumnName);
    }

//    public String getToColumnName() {
//        return toColumnName;
//    }
//
//    public void setToColumnName(String toColumnName) {
//        this.toColumnName = toColumnName;
//    }
    public int getEndArrowModel() {
        return endArrowModel;
    }

    public void setEndArrowModel(int endArrowModel) {
        this.endArrowModel = endArrowModel;
    }

    public int getStartArrowModel() {
        return startArrowModel;
    }

    public void setStartArrowModel(int startArrowModel) {
        this.startArrowModel = startArrowModel;
    }

    public Projection getProjection() {
        return projection;
    }

    public String getValueColumnName() {
        return valueColumnName;
    }

    public void setControlPointsVisibility(boolean b) {
        controlPointsVisibility = b;
    }

    public boolean isControlPointsVisibility() {
        return controlPointsVisibility;
    }

    /*public void setControlPointSize(int value) {
    controlPointSize = value;

    }*/

    /*public int getControlPointSize() {
    return controlPointSize;
    }*/
    public SymbolResizer getResizer() {
        return resizer;
    }

    public void resizerChanged(Resizer resizer) {
        if (this.resizer == resizer) {
            valueColumnName = ((SymbolResizer) resizer).getDataColumnName();
            //setNeedsToRegenerateGraphics(true);
            setBadCache(true);
        }
    }

    public type getChosenType() {
        return chosenType;
    }

    public void setChosenType(type chosenType) {
        this.chosenType = chosenType;
    }

    public void setPrimaryKeyArray(Object[] primaryKeyArray) {
        this.primaryKeyArray = primaryKeyArray;
    }

    private String buildKey(DataObject dataObject) {
        String key = "";
        for (int i = 0; i < primaryKeyArray.length; i++) {
            key += dataObject.getData(primaryKeyArray[i].toString());
        }
        return key;
    }

    private String primaryKeysToString() {
        String keys = "";
        for (int i = 0; i < primaryKeyArray.length; i++) {
            keys += primaryKeyArray[i] + " ";
        }
        return keys;

    }

    private void buildPrimaryKeyArray(String st) {
        primaryKeyArray = st.split(" ");
    }

    public boolean isEmptyPrimaryKey() {
        return primaryKeyArray == null ||
                primaryKeyArray.length == 0;
    }

    public boolean isDragging() {
        return dragging;
    }

    public void setDragging(boolean dragging) {
        this.dragging = dragging;
    }

    @Override
    public boolean isInteractiveAndBadCache() {
        return (getBadCache() && interactive);
    }

    public String getColorDataColumnName() {
        return colorDataColumnName;
    }

    public void setColorDataColumnName(String s) {
        colorDataColumnName = s;
        classificationChanged(this);
    }

    public boolean isClassified() {
        return classified;
    }

    public void setClassified(boolean b) {
        classified = b;
    }

    public void classificationChanged(Object source) {
        Debug.message("ThematicDataDisplayLayer", this + " => classificationChanged(" + source + ")");
        if (source != colorizer) {
            //Debug.message("ThematicDataDisplayLayer", this + " <> classificationChanged(" + source + ")");
            colorizer.setData(getLayer().getDataStorageIterator(), colorDataColumnName);
        }
        refresh();
        //Debug.message("ThematicDataDisplayLayer", this + " <= classificationChanged(" + source + ")");
    }

    public ThematicMouseControlPointsListener getMouseControlPointsListener() {
        return mouseControlPointsListener;
    }

    public MapMouseListener getMapMouseListener() {
        super.getMapMouseListener();
        return mouseControlPointsListener;
    }

    /**
     * @return the showStartEqualsEnd
     */
    public boolean isShowStartEqualsEnd() {
        return showStartEqualsEnd;
    }

    public void setShowStartEqualsEnd(boolean b) {
        showStartEqualsEnd = b;
    }

    /**
     * @return the loopRadius
     */
    public int getLoopRadius() {
        return loopRadius;
    }

    /**
     * @param loopRadius the loopRadius to set
     */
    public void setLoopRadius(int loopRadius) {
        this.loopRadius = loopRadius;
    }

    public void setNbStep(int nb) {
        this.nbStep = nb;
    }

    public int getNbStep() {
        return nbStep;
    }

    public void setStepColumn(int index, String colName) {
        if (stepColumns.size() <= index) {
            stepColumns.add(colName);
        } else {
            stepColumns.set(index, colName);
        }
    }

    public String getStepColumnAt(int index) {
        try {
            return stepColumns.get(index);
        } catch (IndexOutOfBoundsException e) {
            return null;
        }
    }
}
