// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import java.util.*;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;


import com.bbn.openmap.layer.DeclutterMatrix;
import com.bbn.openmap.omGraphics.FontSizer;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMText;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicGUI;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMDisc;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMLabel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMRegularPolygon;
import fr.unicaen.openmap.omGraphics.DrawingAttributes;

/**
 * Label visualization mode : draws a text string near by a geographical centroid
 * <p>The text is found in data column: <code>dataColumnName</code>. 
 * <p> TODO: enable setting of x and y offset of the label!
 */
public class Label extends /*Raw*/ OffsetableRaw implements OffseterListener, FontSizerListener {

    public static final String dataColumnNamePropertyString = "dataColumnName";
    public static final String ROTATION_ANGLE_PROPERTY_NAME = "rotationAngle";
    protected JPanel dataListPanel;
    protected String dataColumnName;
    protected DeclutterMatrix declutterMatrix = new DeclutterMatrix();
    private boolean declutt = false;
    //protected Offseter offseter = new Offseter(0, 0);
    protected FontSizerPanel fontsizerPanel = new FontSizerPanel(new FontSizerShadow(200000000f, 1, 8, 20));
    // private JComponent panelGUI;

    public Label(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(Label.class, "label", "Label"));
    }

    public Label(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
        //offseter.addOffseterListener(this);
        fontsizerPanel.addFontSizerListener(this);
        interactive = true;
    }

    @Override
    protected void buildDrawingAttributes() {
        drawingAttributes = new DrawingAttributes(true, false, false, true, false, false);
    }

    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        dataColumnName = properties.getProperty(prefix + dataColumnNamePropertyString, "data");
        fontsizerPanel.setProperties(prefix, properties);
        angle = PropUtils.doubleFromProperties(properties, prefix + ROTATION_ANGLE_PROPERTY_NAME, 0);
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        properties.put(prefix + dataColumnNamePropertyString, dataColumnName);
        properties = fontsizerPanel.getProperties(prefix, properties);
        properties.put(prefix + ROTATION_ANGLE_PROPERTY_NAME, Double.toString(angle));

        return properties;
    }

    @Override
    public OMGraphicList generateGraphics(DataStorageIterator dsi) {
        OMGraphicList omgl = new OMGraphicList();
        mouseControlPointsListener.clearControlPoints();
//        if (declutt) {
//            Projection projection = getLayer().getProjection();
//            declutterMatrix.setWidth(projection.getWidth());
//            declutterMatrix.setHeight(projection.getHeight());
//            declutterMatrix.create();
//        }
        boolean offVis = offsetsOn ? offseter.isVisible() : false;
        boolean offLineVis = offsetsOn ? offseter.isLineVisible() : false;
        FontSizer fontsizer = fontsizerPanel.getFontSizer(getProjection());
        for (int i = 0; dsi.hasNext(); i++) {
            DataObject data = dsi.next();
            OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);
            // System.out.println("Label "+ i);

            if (omdl == null) {
                continue;
            }

            Object dataValue = data.getData(dataColumnName);
            if (dataValue == null) {
                // System.out.println(i + " null data value for " + dataColumnName);
                continue;
            }

            // <clipping>
            if (!getLayer().overlapsCurrentProjBB(omdl.getBoundingBox())) {
                continue;
            }
            // </clipping>

            Centroid centroid = omdl.getCentroid();
	    if (centroid == null) {
		System.err.println("* no centroid for "+omdl);
		continue;
	    }
		
            String name = dataValue.toString();
            float lat = (float) centroid.getLat();
            float lon = (float) centroid.getLon();
            int offX = 0;
            int offY = 0;
            String key = "";

            if (offsetsOn) {
                key = offseter.buildKey(data);
                offX = offseter.getX(key);
                offY = offseter.getY(key);
            }
            OMLabel omt = new OMLabel(this, key, lat, lon,
                    offX, offY,
                    name,
                    fontsizer.getFont(),
                    OMText.JUSTIFY_CENTER,
                    offVis, offLineVis);
            omt.getOMText().setRenderType(OMText.RENDERTYPE_OFFSET);
            omt.getOMText().setFontSizer(fontsizer);
            omt.setLinePaint(drawingAttributes.getLinePaint());
            omt.getOMText().setRotationAngle(getRotationAngle());

            //BasicLocation bl = new BasicLocation((float) centroid.getLat(), (float) centroid.getLon(), "", omt);
            // bl.generate(proj, declutterMatrix);
            if (offsetsOn) {
                //omt.addOffsetableListener(offseter);
            }
            mouseControlPointsListener.addOMControlPoints(omt.controlPoints);
            omgl.add(omt);
        }
        return omgl;
    }
    private double angle = 0;

    /**
     * In radians.
     */
    public double getRotationAngle() {
        return angle;
    }

    @Override
    public JComponent getGUI(final DataStorageIterator dsi) {

        if (thematicGUI != null) {
            return thematicGUI;
        }
        thematicGUI = (ThematicGUI) super.getGUI(dsi);

        graphicsPanelToggleControlsDisplay(false, true, false, false, false);

        Box textPosPanel = Box.createVerticalBox();
        JPanel colPanel = new JPanel();
        colPanel.add(new JLabel(i18n.get(this, "dataColumnName", "Data Column Name: ")));
        dataListPanel = new JPanel();
        initDataList(dsi);
        colPanel.add(dataListPanel);
        textPosPanel.add(colPanel);
        //textPosPanel.add(offseter.getGlobalGUI());

        final JSpinner rotationSpinner = new JSpinner(new SpinnerNumberModel(getRotationAngle(), -360d, 360d, 1d));
        rotationSpinner.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent ce) {
                angle = Math.toRadians(((Double) rotationSpinner.getValue()).doubleValue());
                refresh();
            }
        });
        JPanel rotPanel = new JPanel();
        rotPanel.add(new JLabel(i18n.get(Label.class, "angle", "Rotation Angle (degrees) : ")));
        rotPanel.add(rotationSpinner);
        textPosPanel.add(rotPanel);


        final Box panPosition = Box.createVerticalBox();
        panPosition.add(textPosPanel);

        ThematicConfPanel themaConfPanel = ThematicConfPanel.createThematicConfPanel(
                i18n.get(this, "position", "Text & Position"));
        themaConfPanel.setContent(panPosition);
        thematicGUI.addThematicConfigurationPanel(themaConfPanel);
        JPanel wrapConf = new JPanel();
        wrapConf.add(themaConfPanel);
        thematicGUI.add(wrapConf);

        ThematicConfPanel confp = ThematicConfPanel.createVerticalThematicConfPanel(
                i18n.get(FontSizerPanel.class, "title", "Fonts"));
        confp.setContent(fontsizerPanel.getFontPanel());
        JPanel wrapConf2 = new JPanel();
        wrapConf2.add(confp);
        thematicGUI.add(wrapConf2);
        thematicGUI.addThematicConfigurationPanel(confp);

        return thematicGUI;
    }

    public String getDataColumnName() {
        return dataColumnName;
    }

    public void setDataColumnName(String name) {
        dataColumnName = name;
        refresh();
    }

    public void refreshGUI(final DataStorageIterator dsi) {
        super.refreshGUI(dsi);
        initDataList(dsi);

    }

    /** init or reinit the comboBox used to choose dataColumnName */
    private void initDataList(final DataStorageIterator dsi) {
        if (dataListPanel == null) {
            getGUI(dsi);
        }
        // dataListPanel.add(new JLabel("plop"));

        dataListPanel.removeAll();
        JComboBox dataList = new JComboBox(dsi.getDataColumnNames());
        if (getDataColumnName() != null & !"".equals(getDataColumnName())) {
            dataList.setSelectedIndex(
                    dsi.getIndexForColumnName(getDataColumnName()));
        } else {
            if (dsi.getDataColumnNames()[0].length() > 0) {
                dataColumnName = dsi.getDataColumnNames()[0];
            }
        }

        dataList.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                try {
                    int index = ((JComboBox) e.getSource()).getSelectedIndex();
                    setDataColumnName(dsi.getRealColumnName(index));
                } catch (ArrayIndexOutOfBoundsException ex) {
                }
            }
        });
        dataListPanel.add(dataList);

    }

    //manage font with scale : we need to know current projection
    //to change when generateGraphics will be modified
    public void setLayer(com.bbn.openmap.Layer layer) {
        fontsizerPanel.setLayer(layer);
    }
    //In progress..................
    private Projection proj;

    public boolean needsToRegenerateGraphics() {
        //behind fontsizer and current proj
        return (fontsizerPanel.isModified(proj));
    }

//    @Override
//    public void offseterChanged(Offseter offseter) {
//        super.offseterChanged(offseter);
//        //setBadCache(true);
//    }

    public void fontsizerChanged(FontSizer fontsizer) {
        refresh();
    }
}
