// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.utils;

import java.net.URL;
import java.util.Properties;

import com.bbn.openmap.Layer;
import com.bbn.openmap.LayerHandler;
import com.bbn.openmap.PropertyHandler;
import fr.unicaen.openmap.layer.legendLayer.LegendLayer;
import fr.unicaen.openmap.util.TooManyLegendException;
import java.lang.reflect.Constructor;

public class LayerLoader {

    protected Properties properties;
    protected LayerHandler layerHandler;
    //private ArrayList<String> loadedLayers;
    protected String[] loadedLayers;
    public final static String KEY_LAYERS = "openmap.layers";
    public final static String KEY_CLASS = "class";

    public LayerLoader(URL url, LayerHandler layerHandler) {
        //System.out.println("URL LAYER LOADER -> " + url.toString());
        //Logger.getLogger(KEY_CLASS, KEY_CLASS);
        properties = PropertyHandler.fetchProperties(url);
        this.layerHandler = layerHandler;
    }

    public String[] findAvailableLayers() {
                String l = properties.getProperty(KEY_LAYERS, "");
                if (!l.equals("")) {
                    loadedLayers = l.split(" ");
                }
        return loadedLayers;
    }


    public Layer initLayer(String layerName) throws TooManyLegendException{
        String classProperty = layerName + ".class";
        String className = properties.getProperty(classProperty);
        Object obj = null;
        try {
            ClassLoader cl = Thread.currentThread().getContextClassLoader();
            if (cl == null) {
                cl = this.getClass().getClassLoader();
            }
            Class<?> newObjClass = Class.forName(className.trim(), true, cl);
            Constructor<?> constructor = newObjClass.getConstructor();
            obj = constructor.newInstance();
            if (obj instanceof Layer) {
                Layer l = (Layer) obj;
                if(layerHandler instanceof fr.unicaen.openmap.LayerHandler){
                    if(l instanceof LegendLayer && ((fr.unicaen.openmap.LayerHandler)layerHandler).isLegended())
                        throw new TooManyLegendException();
                }
                l.setName(layerName);
                l.setProperties(layerName, properties);
                layerHandler.addLayer(l, 0);
                return l;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

}
