// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util;

import com.bbn.openmap.util.Debug;

import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.dataStorage.DataStorage;
import fr.unicaen.openmap.dataStorage.DataStorageIterator;
import fr.unicaen.openmap.dataStorage.DataStorageIteratorAdapter;
import fr.unicaen.openmap.dataStorage.DataStorageListener;
import java.awt.*;
import java.awt.font.FontRenderContext;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.net.URL;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.ImageIcon;
import java.io.File;
import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.net.URLConnection;
import java.util.Properties;
import java.util.regex.Pattern;
import java.util.zip.GZIPInputStream;

public final class Util {

    /**
     * Property string for the OMGraphic Index in 
     */
    static public final String OMGraphicIndex = "index";
    static Pattern spaces = Pattern.compile("\u00a0");

    static public String className(Object o) {
        String s = o.getClass().getName();
        return s.substring(s.lastIndexOf('.') + 1, s.length());
    }

    static public ImageIcon loadIcon(String file) {
	try {
            URL url = Util.class.getResource("/fr/unicaen/openmap/data/images/" + file);
	    if (url != null)
                return new ImageIcon(url, file);
	    String name = locateFile(file);
	    if (name != null)
		return new ImageIcon(name, file);
            return null;
	} catch (Exception e) { e.printStackTrace();return null; }
    }

    static public Vector<String> textToVector(String text) {
        Vector<String> vector = new Vector<String>();
        StringTokenizer st = new StringTokenizer(text, "\n");
        for (; st.hasMoreTokens();) {
            String s = st.nextToken().trim();
            if (!vector.contains(s)) {
                vector.addElement(s);
            }
        }
        return vector;
    }

    static public Color getDefaultTextColor(Color c) {
        int r = c.getRed();
        int g = c.getGreen();
        int b = c.getBlue();
        if (r > 240 || g > 240) {
            return Color.black;
        } else {
            return Color.white;
        }
    }

    static public String splitString(String message, int max) {
        if (message.length() <= max) {
            return message;
        }
        int index = 0;
        int tmp = 0;
        while (tmp != -1 && tmp < max) {
            index = tmp;
            tmp = message.indexOf(' ', index + 1);
        }
        return message.substring(0, index) + "\n" +
                splitString(message.substring(index + 1, message.length()), max);
    }

    /**
     * Returns true if string is equals (case insensitive) to on, yes, true.
     */
    public static boolean isTrue(String bool) {
        return ((bool.equalsIgnoreCase("on")) ||
                (bool.equalsIgnoreCase("yes")) ||
                (bool.equalsIgnoreCase("true")));
    }

    /**
     *
     */
    public static int getMax(int[] values) {
        int max = Integer.MIN_VALUE;

        for (int i = 0; i < values.length; i++) {
            if (values[i] > max) {
                max = values[i];
            }
        }
        return max;
    }

    /**
     * Returns a Rectangle2D which is the bounding box of the string s in the default font of the graphics
     */
    public static Rectangle2D getStringBounds(String s, Graphics g) {
        return getStringBounds(s, g.getFont());
    }

    /**
     * Returns a Rectangle2D which is the bounding box of the string s in the given font
     */
    public static Rectangle2D getStringBounds(String s, Font f) {
        return f.getStringBounds(s, new FontRenderContext(null, true, true));
    }

    /**
     * Locate file 'fileName' in classpath, if it is not an absolute file name.
     * @return absolute name of the file as a string if found, null otherwise.
     * borrowed from ShapeLayer
     */
    public static String locateFile(String name) {
        //System.out.println("NAME = " + name);
        File file = new File(name);
        if (file.exists()) {
            return name;
        } else {
            URL url = ClassLoader.getSystemResource(name);
            //System.out.println("URL = " + url);
            //OK, now we want to look around for the file, in the
            //classpaths, and as a resource.  It may be a file in
            //a classpath, available for direct access.
            if (url != null) {
                String newname = url.getFile();
                //System.out.println("New Name = " + newname);
                file = new File(newname);
                if (file.exists()) {
                    return newname;
                }
            }
        }
        return name;
    }

    /**
     * Locate file 'fileName' in classpath, or as an absolute file name.
     * @return URL of the file if found, null otherwise.
     */
    public static URL getURLforFileName(String name) {
        File file = new File(name);
        try {
            if (file.exists()) {
                return file.toURI().toURL();
            }
        } catch (java.net.MalformedURLException e) {
            e.printStackTrace();
            return null;
        }

        return ClassLoader.getSystemResource(name);
    }

    static public DataStorage loadDataStorage(String className, String prefix, Properties properties, DataStorageListener dsl) {

        try {
            DataStorage ds = createDataStorage(className, dsl);


//            try{throw new Exception("LOAD DS");}
//            catch(Exception e){e.printStackTrace();}
            if (ds == null) {
                System.err.println("DataStorage is null !!! " + className);
                return null;
            }
	    // System.out.println("## Set Properties for: " + ds);
            ds.setProperties(prefix, properties);
            return ds;
        } catch (Exception e) {
            System.err.println("Error loading DataStorage !!! " + className + " encountred " + e);
            e.printStackTrace();
        }
        return null;
    }

    static public DataStorage createDataStorage(String className, DataStorageListener dsl) {
        try {
//            try{throw new Exception("CREATE DS");}
//            catch(Exception e){e.printStackTrace();}
            Class cls = Class.forName(className);
            Constructor[] constructors = cls.getConstructors();
            for (int i = 0; i < constructors.length; i++) {
                Class params[] = constructors[i].getParameterTypes();
                if (params.length == 1 && params[0].getName().equals("fr.unicaen.openmap.dataStorage.DataStorageListener")) {
                    Object obj = constructors[i].newInstance(new Object[]{dsl}); //
		    // System.out.println("## New DataStorage constructed: " + obj);
                    return ((DataStorage) obj);
                }
            }
            System.err.println("Util : no constructor found for " + className);
            return null;
        } catch (Exception e) {
            System.err.println("Util : createDataStorage failure for class \"" + className + "\" raised " + e.getClass().getName());
            e.printStackTrace();
            return null;
        }
    }

    public static boolean equals(double x1, double x2, double precision) {
	return (Math.abs(x1) < precision*precision && Math.abs(x2) < precision*precision)
	    || Math.abs(x1-x2) < Math.abs(x1*precision);
    }

    /** Robust, crude and expensive parseDouble method. 
     *	<p>Todo: Should optionnaly treat -99 999 as null as in some DBF convention </p>
     */
    public static double parseDouble(Object o /*, boolean Minus5NineIsNull*/) {
        if (o == null) {
            //System.out.println(Double.MIN_VALUE);
            return 0;
        }
        if (o instanceof Double) {
            return (Double) o;
        }
        String number = o.toString();
        if ("".equals(number)) {
            return 0.0;
        }
        /*	if (Minus5NineIsNull && "-99999.0".equals(number))
        return ; */
        try {
            number = spaces.matcher(number).replaceAll("");
            return Double.parseDouble(number);
        } catch (NumberFormatException e) {
            System.err.println("NumberFormatException for " + o);
        }
        return 0.0;
    }

    /** Compare */
    public static int compare(Object o1, Object o2) {
        try {
            if (o1 instanceof Double) {
                return ((Double) o1).compareTo((Double) o2);
            }
        } catch (ClassCastException e) {
        }
        return o1.toString().compareTo(o2.toString());
    }

    public static InputStream open(String fileName) {
	return open(fileName, false);
    }
    
    public static InputStream open(String fileName, boolean tryToUnzip) {
        InputStream in = null;
        URL url = null;
        try {
            url = PropUtils.getResourceOrFileOrURL(fileName);
        } catch (java.net.MalformedURLException e) {
        }

        if (url != null) {
            try {
                in = url.openStream();
            } catch (java.io.IOException e) {
                System.err.println("IOException for file " + fileName);
                in = null;
            }
        }

        if (in != null && tryToUnzip) { // try to uncompress...
            try {
                in = new GZIPInputStream(in);
            } catch (Exception notGZIPexception) {
                // It's not a zip file, then we re-open the url
                try {
                    in = url.openStream();
                } catch (java.io.IOException e) {
                    System.err.println("IOException for file " + fileName);
                    in = null;
                }
            }
        }
        return in;
    }

}
