// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.classification;

import java.util.ArrayList;
import java.util.Collections;
import javax.swing.SpinnerNumberModel;

/**
 * Produce the same classes used in box plots :
 * 0-1%, 1%-25%, 25%-75%, 75%-99%, 99%-100%
 */
public class BoxPlot extends ClassificationOfDouble {

    public BoxPlot(ArrayList<Double> data, int nbClasses) {
	super(data, 5);
	nbClasses = 5;
    }
    
    /** fixed number of classes */
    public SpinnerNumberModel getClassificationNumberModel() {
	return new SpinnerNumberModel (nbClasses, nbClasses, nbClasses, 0);
    }

    protected void init() {
	if (data == null){
            return;
        }
	Collections.sort(data);
	int N = data.size();
	if (N == 0 || nbClasses == 0) {
	    System.err.println("[Classification] *** Error *** empty data");
	    return;
	}

	breaks = new double[nbClasses+1];
	breaks[0] = data.get(0); // first break is the min value;
	breaks[nbClasses] = data.get(data.size()-1); // last break is the max value

	int totalCounts = 0; // effective count of values
	counts = new int[nbClasses];

	int currentClass = 0; // class index
	int currentClassSize = 0;
	int classSize[] = { 1, 24, 50, 24, 1 };
	for (int i = 0; i < nbClasses; i++)
	    classSize[i] = (int)(classSize[i]*N/100.0);
	// System.out.println("class size " + currentClass + " = " + classSize);
       		   
	for (int dd = 0; dd < data.size(); dd++) {
	    double d = data.get(dd);
	    currentClassSize++;
	    // System.out.println(dd + " " + currentClassSize + " " + d + " " + classSize[currentClass]);

	    if (currentClass >= nbClasses)
		break;
	    if (currentClassSize >= classSize[currentClass]) {
		breaks[currentClass] = // the mean value of d and next d
		    dd+1 < data.size() ? (d+data.get(dd+1))/2.0 : d;
		counts[currentClass] = currentClassSize;
		totalCounts += currentClassSize;
	        // System.out.println("breaks["+currentClass+"]="+breaks[currentClass]+ " ("+currentClassSize+")");
		currentClass++;
		currentClassSize = 0;
	    }
	}
	if (currentClassSize != 0) 
	    counts[nbClasses-1] = currentClassSize;
    }

}
