// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.classification;

import java.util.ArrayList;
import java.util.Collections;

/**
 * Nested Means classification.
 * The weight (i.e. sum of its element values) of the class number i of a k classes
 * classificaition is: totalWeight * (i*2+1)/(k*k) 
 */
public class NestedMeans extends ClassificationOfDouble {

    public NestedMeans(ArrayList<Double> data, int nbClasses) {
	super(data, nbClasses);
    }
    
    protected void init() {
	if (data == null) return;
	Collections.sort(data);
	int N = data.size();
	if (N == 0) {
	    System.err.println("[Classification] *** Error *** empty data");
	    return;
	}

	breaks = new double[nbClasses+1];
	breaks[0] = data.get(0); // first break is the min value;
	breaks[nbClasses] = data.get(data.size()-1); // last break is the max value

	double totalWeight = 0;
	for(double d : data)
	    totalWeight += d;
	double cumulativeWeight = 0;
	int currentClass = 1;
	double classExpectedWeight = 
	    totalWeight*((currentClass-1)*2+1)/(nbClasses*nbClasses);
	// System.err.println("Total weight: "+ totalWeight+"\nclass expected weight " + currentClass + " = "  + classExpectedWeight + " ----" + (((double)(currentClass-1)*2+1)/(nbClasses*nbClasses)));
	double currentClassWeight = 0;
	int currentClassSize = 0;

	counts = new int[breaks.length-1];
	
	for (int dd = 0; dd < data.size(); dd++) {
	    double d = data.get(dd);
	    currentClassSize++;
	    currentClassWeight += d;
	    // in case of equal following values, we extend the current class
	    if (currentClassWeight >= classExpectedWeight 
		&& (dd+1 < data.size() ? d != data.get(dd+1) : true)) {
		// we get to a new break
		breaks[currentClass] = // the mean value of d and next d
		    dd+1 < data.size() ? (d+data.get(dd+1))/2.0 : d;
		counts[currentClass-1] = currentClassSize;
		cumulativeWeight += currentClassWeight;
		// System.err.println("breaks["+currentClass+"]="+breaks[currentClass]+ " ("+currentClassSize+") "+currentClassWeight);
		currentClass++;
		currentClassSize = 0;
		currentClassWeight = 0;
		classExpectedWeight = currentClass < nbClasses+1 
		    ? totalWeight*((currentClass-1)*2+1)/(nbClasses*nbClasses)
		    : totalWeight - cumulativeWeight;
		// System.err.println("class expected weight " + currentClass + " = "  + classExpectedWeight + " ----" + ((currentClass-1)*2+1)+ " " +(((double)(currentClass-1)*2+1)/(nbClasses*nbClasses)));
	    }
	}
	if (currentClassSize != 0) 
	    counts[nbClasses-1] = currentClassSize;
    }
    
}

