// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.omgraphics;

import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.proj.Projection;

import fr.unicaen.openmap.util.Colors;
import fr.unicaen.openmap.util.ColorTable;
import fr.unicaen.openmap.util.find.Distance;


//fabrique succession de rectangles colorés selon classes & couleurs données => légende
//basée sur OMLegendPiece (d'ailleurs je pourrais fusionner les 2)
public class OMColorScale extends OMGraphicList {
    
    /**
     * BEMOL : je considère le positionnement de l'OMScale par rapport à un gros objet (un gros objet carré en plus)
     * => devrait en faire une pièce autonome (pour faciliter un positionnement manuel)
     * la distance servirait alors pour la taille de l'échelle -et plus pour le positionnement-
     * distance : utilisée pour avoir une taille à peu près proportionnel à un élément
     * x,y : position
     * colors : contient les couleurs suivant les classes
     * breaks : toutes les limites de classes
     * default_color : si une classe n'a pas de couleurs associé
     * title : titre de l'élément
     */
    
    public OMColorScale(double distance, Projection proj, int x, int y, ColorTable colors, Colors default_color, double[] breaks, String title) {
        LatLonPoint mapCenter = proj.getCenter();
        LatLonPoint pt = new LatLonPoint.Double(mapCenter.getLatitude(), mapCenter.getLongitude() + (float)distance);
        int maxH = Distance.computeDistanceInPixel(mapCenter, pt, proj); // à revoir
        int width = maxH/3;
        
        OMRect rect;
        OMText text;
        Colors c;
        int step = 20;
        
        int xpos = x + maxH/2;
        int ypos = y + step + 2*maxH;
        
        add(new OMText(xpos, ypos,
                title,
                OMText.JUSTIFY_LEFT));
        
        
        for(int i = 0; i<breaks.length-1;i++) {
            rect = new OMRect(xpos, ypos,
                    xpos + maxH/2, ypos += width);
            
            text = new OMText(xpos + maxH/2 + 5, ypos,
                    breaks[i]+" - "+breaks[i+1],
                    OMText.JUSTIFY_LEFT);
            
            c = colors.getColors(breaks[i]);
            if (c == null)
                c = default_color;
            
            rect.setFillPaint(c.getFillColor());
            rect.setLinePaint(c.getLineColor());
            
            add(rect);
            add(text);
        }
    }
    
    
    
}
